<!-- PROJECT LOGO -->
<p align="center">
    <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/vgrid192.svg" alt="Logo">
  <h2 align="center">Vgrid</h2>
  <p align="center">
    <b><i>DGGS and Cell-based Geocoding Utilities</i><b>
    <br />
  </p>
</p>
<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/dggs.png">
</div>

<!-- TABLE OF CONTENTS -->
<details open="open">
  <summary>Vgrid User Guide</summary>  
  <ol>
    <li>
      <a href="#vgrid-installation">Vgrid Installation</a>     
    </li>
    <li>
      <a href="#dggs-conversion">DGGS Conversion</a>
      <ul>
        <li><a href="#lat-lon-to-dggs">Lat lon to DGGS</a></li>
        <li><a href="#dggs-to-geojson">DGGS to GeoJSON</a></li>
        <li><a href="#vector-to-dggs">Vector to DGGS</a></li>
        <li><a href="#raster-to-dggs">Raster to DGGS</a></li>
        <li><a href="#dggs-compact">DGGS Compact</a></li>
        <li><a href="#dggs-expand">DGGS Expand</a></li>
        <li><a href="#csv-to-dggs">CSV to DGGS</a></li>
      </ul>
        <li><a href="#dggs-binning">DGGS Binning</a></li>
        <li><a href="#dggs-resampling">DGGS Resampling</a></li>
        <li><a href="#dggs-generator">DGGS Generator</a></li>
        <li><a href="#dggs-stats">DGGS Stats</a></li>
        <li><a href="#polyhedra-generator">Polyhedra Generator</a></li>
    </li>   
  </ol>
</details>

## Vgrid Installation
[![PyPI version](https://badge.fury.io/py/vgrid.svg)](https://badge.fury.io/py/vgrid)
[![PyPI downloads](https://img.shields.io/pypi/dm/vgrid.svg)](https://pypistats.org/packages/vgrid)
![Total downloads](https://static.pepy.tech/personalized-badge/vgrid?period=total&units=international_system&left_color=grey&right_color=blue&left_text=total)

- Using pip:   
    ``` bash 
    pip install vgrid --upgrade
    ```
- Vgrid Home:  [Vgrid DGGS](https://vgrid.vn)

## ***The following Vgrid user guide is intended for use in a Command Line Interface (CLI) environment.***

## DGGS Conversion

### Lat lon to DGGS

Convert lat, long in WGS84 CRS to DGGS cellID (H3, S2, rHEALPix, OpenEaggr ISEA4T and ISEA3H (Windows only), EASE-DGGS, QTM, OLC/ OpenLocationCode/ Google Plus Code, Geohash, GEOREF, MGRS, Tilecode, Quadkey, Maidenhead, GARS).

<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/latlon2dggs.png">
</div>

``` bash
> latlon2h3 10.775276 106.706797 13 # latlon2h3 <lat> <lon> <resolution> [0..15] 
> latlon2s2 10.775276 106.706797 21 # latlon2s2 <lat> <lon> <resolution> [0..30]
> latlon2rhealpix 10.775276 106.706797 14 # latlon2rhealpix <lat> <lon> <resolution> [1..15]
> latlon2isea4t 10.775276 106.706797 21 # latlon2isea4t <lat> <lon> <resolution> [0..39]
> latlon2isea3h 10.775276 106.706797 27 # latlon2isea3h <lat> <lon> <resolution> [0..40]
> latlon2ease 10.775276 106.706797 6 # latlon2easedggs <lat> <lon> <resolution> [0..6]
> latlon2dggrid 10.775276 106.706797 FULLER4D 10 # Linux only: latlon2dggrid <lat> <lon> <dggs_type> <resolution> 
> latlon2qtm 10.775276 106.706797 11  # latlon2qtm <lat> <lon> <resolution> [1..24]
> latlon2olc 10.775276 106.706797 11 # latlon2olc <lat> <lon> <resolution> [2,4,6,8,10,11..15]
> latlon2geohash 10.775276 106.706797 9 # latlon2geohash <lat> <lon> <resolution>[1..10]
> latlon2georef 10.775276 106.706797 4 # latlon2georef <lat> <lon> <resolution> [0..5]
> latlon2mgrs 10.775276 106.706797 4 # latlon2mgrs <lat> <lon> <resolution> [0..5]
> latlon2tilecode 10.775276 106.706797 23 # latlon2tilecode <lat> <lon> <resolution> [0..29]
> latlon2quadkey 10.775276 106.706797 23 # latlon2quadkey <lat> <lon> <resolution> [0..29]
> latlon2maidenhead 10.775276 106.706797 4 # latlon2maidenhead <lat> <lon> <resolution> [1..4]
> latlon2gars 10.775276 106.706797 1 # latlon2gars <lat> <lon> <resolution> [30,15,5,1] minutes
```

### DGGS to GeoJSON
Convert DGGS cell ID to GeoJSON.

```geojson
{"type": "FeatureCollection", "features": [{"type": "Feature", "geometry": {"type": "Polygon", "coordinates": [[[106.70789209957029, 10.77601109480422], [106.70745212203926, 10.777918172217145], [106.7055792173179, 10.778494886227104], [106.70414629547702, 10.777164500398262], [106.70458630070586, 10.775257408234628], [106.70645920007833, 10.774680716650128], [106.70789209957029, 10.77601109480422]]]}, "properties": {"h3": "8965b56628fffff", "resolution": 9, "center_lat": 10.7765878, "center_lon": 106.7060192, "avg_edge_len": 215.295, "cell_area": 120421.396}}]}
```

``` bash
> h32geojson 8d65b56628e46bf 
> s22geojson 31752f45cc94 
> rhealpix2geojson R31260335553825
> isea4t2geojson 13102313331320133331133 # Windows only
> isea3h2geojson 1327916769,-55086 # Windows only
> ease2geojson L6.165767.02.02.22.45.63.05
> dggrid2geojson 8478420 FULLER4D 10 # Linux only:  dggrid2geojson <DGGRID code in SEQNUM format> <DGGS Type> <resolution>
# <DGGS Type> chosen from [SUPERFUND,PLANETRISK,ISEA3H,ISEA4H,ISEA4T,ISEA4D,ISEA43H,ISEA7H,IGEO7,FULLER3H,FULLER4H,FULLER4T,FULLER4D,FULLER43H,FULLER7H]
> qtm2geojson 42012323131
> olc2geojson 7P28QPG4+4P7
> geohash2geojson w3gvk1td8
> georef2geojson VGBL42404651
> mgrs2geojson 34TGK56063228
> gzd # Create Grid Zone Designators - used by MGRS
> tilecode2geojson z23x6680749y3941729
> quadkey2geojson 13223011131020212310000
> maidenhead2geojson OK30is46 
> gars2geojson 574JK1918
```

### Vector to DGGS
Convert Vector layers (Point/ Multipoint, Linestring/ Multilinestring, polygon/ Multipolygon) in GeoJSON to DGGS.

- **Uncompact:**
```geojson
{
"type": "FeatureCollection",
"name": "s2_17",
"crs": { "type": "name", "properties": { "name": "urn:ogc:def:crs:OGC:1.3:CRS84" } },
"features": [
{ "type": "Feature", "properties": { "s2": "317528cd34", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.789733499846557 ], [ 106.690295128739763, 10.789692863129195 ], [ 106.690295128739763, 10.790373634530699 ], [ 106.689558101859916, 10.790414273689612 ], [ 106.689558101859916, 10.789733499846557 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528cd4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.789774134673007 ], [ 106.689558101859916, 10.789733499846557 ], [ 106.689558101859916, 10.790414273689612 ], [ 106.688821077455032, 10.790454910957493 ], [ 106.688821077455032, 10.789774134673007 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528cd54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.789814767608535 ], [ 106.688821077455032, 10.789774134673007 ], [ 106.688821077455032, 10.790454910957493 ], [ 106.688084055525565, 10.790495546334341 ], [ 106.688084055525565, 10.789814767608535 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528cd5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.790495546334341 ], [ 106.688821077455032, 10.790454910957493 ], [ 106.688821077455032, 10.791135692378973 ], [ 106.688084055525565, 10.791176330197136 ], [ 106.688084055525565, 10.790495546334341 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d28c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.791938394041752 ], [ 106.687347036071799, 10.791897757564833 ], [ 106.687347036071799, 10.792578554141945 ], [ 106.686610019094246, 10.79261919305994 ], [ 106.686610019094246, 10.791938394041752 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d294", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.791257600159977 ], [ 106.687347036071799, 10.791216966124139 ], [ 106.687347036071799, 10.791897757564833 ], [ 106.686610019094246, 10.791938394041752 ], [ 106.686610019094246, 10.791257600159977 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d29c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.791216966124139 ], [ 106.688084055525565, 10.791176330197136 ], [ 106.688084055525565, 10.791857119196633 ], [ 106.687347036071799, 10.791897757564833 ], [ 106.687347036071799, 10.791216966124139 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.790536179820146 ], [ 106.688084055525565, 10.790495546334341 ], [ 106.688084055525565, 10.791176330197136 ], [ 106.687347036071799, 10.791216966124139 ], [ 106.687347036071799, 10.790536179820146 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.789855398653133 ], [ 106.688084055525565, 10.789814767608535 ], [ 106.688084055525565, 10.790495546334341 ], [ 106.687347036071799, 10.790536179820146 ], [ 106.687347036071799, 10.789855398653133 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.789896027806794 ], [ 106.687347036071799, 10.789855398653133 ], [ 106.687347036071799, 10.790536179820146 ], [ 106.686610019094246, 10.790576811414896 ], [ 106.686610019094246, 10.789896027806794 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.790576811414896 ], [ 106.687347036071799, 10.790536179820146 ], [ 106.687347036071799, 10.791216966124139 ], [ 106.686610019094246, 10.791257600159977 ], [ 106.686610019094246, 10.790576811414896 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.790617441118588 ], [ 106.686610019094246, 10.790576811414896 ], [ 106.686610019094246, 10.791257600159977 ], [ 106.685873004593077, 10.791298232304639 ], [ 106.685873004593077, 10.790617441118588 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.789936655069512 ], [ 106.686610019094246, 10.789896027806794 ], [ 106.686610019094246, 10.790576811414896 ], [ 106.685873004593077, 10.790617441118588 ], [ 106.685873004593077, 10.789936655069512 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.789977280441276 ], [ 106.685873004593077, 10.789936655069512 ], [ 106.685873004593077, 10.790617441118588 ], [ 106.68513599256886, 10.790658068931213 ], [ 106.68513599256886, 10.789977280441276 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.790658068931213 ], [ 106.685873004593077, 10.790617441118588 ], [ 106.685873004593077, 10.791298232304639 ], [ 106.68513599256886, 10.791338862558119 ], [ 106.68513599256886, 10.790658068931213 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.791338862558119 ], [ 106.685873004593077, 10.791298232304639 ], [ 106.685873004593077, 10.791979028627379 ], [ 106.68513599256886, 10.792019661321712 ], [ 106.68513599256886, 10.791338862558119 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.791298232304639 ], [ 106.686610019094246, 10.791257600159977 ], [ 106.686610019094246, 10.791938394041752 ], [ 106.685873004593077, 10.791979028627379 ], [ 106.685873004593077, 10.791298232304639 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.791979028627379 ], [ 106.686610019094246, 10.791938394041752 ], [ 106.686610019094246, 10.79261919305994 ], [ 106.685873004593077, 10.79265983008653 ], [ 106.685873004593077, 10.791979028627379 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.792019661321712 ], [ 106.685873004593077, 10.791979028627379 ], [ 106.685873004593077, 10.79265983008653 ], [ 106.68513599256886, 10.792700465221705 ], [ 106.68513599256886, 10.792019661321712 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d304", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.792060292124736 ], [ 106.68513599256886, 10.792019661321712 ], [ 106.68513599256886, 10.792700465221705 ], [ 106.684398983021822, 10.792741098465463 ], [ 106.684398983021822, 10.792060292124736 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d30c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.792100921036448 ], [ 106.684398983021822, 10.792060292124736 ], [ 106.684398983021822, 10.792741098465463 ], [ 106.683661975952418, 10.792781729817792 ], [ 106.683661975952418, 10.792100921036448 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d314", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.7914201173915 ], [ 106.684398983021822, 10.791379490920407 ], [ 106.684398983021822, 10.792060292124736 ], [ 106.683661975952418, 10.792100921036448 ], [ 106.683661975952418, 10.7914201173915 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d31c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.791379490920407 ], [ 106.68513599256886, 10.791338862558119 ], [ 106.68513599256886, 10.792019661321712 ], [ 106.684398983021822, 10.792060292124736 ], [ 106.684398983021822, 10.791379490920407 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d324", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.790698694852763 ], [ 106.68513599256886, 10.790658068931213 ], [ 106.68513599256886, 10.791338862558119 ], [ 106.684398983021822, 10.791379490920407 ], [ 106.684398983021822, 10.790698694852763 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d32c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.790017903922081 ], [ 106.68513599256886, 10.789977280441276 ], [ 106.68513599256886, 10.790658068931213 ], [ 106.684398983021822, 10.790698694852763 ], [ 106.684398983021822, 10.790017903922081 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d334", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.790058525511915 ], [ 106.684398983021822, 10.790017903922081 ], [ 106.684398983021822, 10.790698694852763 ], [ 106.683661975952418, 10.790739318883229 ], [ 106.683661975952418, 10.790058525511915 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d33c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.790739318883229 ], [ 106.684398983021822, 10.790698694852763 ], [ 106.684398983021822, 10.791379490920407 ], [ 106.683661975952418, 10.7914201173915 ], [ 106.683661975952418, 10.790739318883229 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d344", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.790779941022603 ], [ 106.683661975952418, 10.790739318883229 ], [ 106.683661975952418, 10.7914201173915 ], [ 106.682924971360933, 10.791460741971385 ], [ 106.682924971360933, 10.790779941022603 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d34c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.790099145210778 ], [ 106.683661975952418, 10.790058525511915 ], [ 106.683661975952418, 10.790739318883229 ], [ 106.682924971360933, 10.790779941022603 ], [ 106.682924971360933, 10.790099145210778 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d354", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.790139763018653 ], [ 106.682924971360933, 10.790099145210778 ], [ 106.682924971360933, 10.790779941022603 ], [ 106.682187969247821, 10.790820561270881 ], [ 106.682187969247821, 10.790139763018653 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d35c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.790820561270881 ], [ 106.682924971360933, 10.790779941022603 ], [ 106.682924971360933, 10.791460741971385 ], [ 106.682187969247821, 10.791501364660057 ], [ 106.682187969247821, 10.790820561270881 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d364", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.791501364660057 ], [ 106.682924971360933, 10.791460741971385 ], [ 106.682924971360933, 10.792141548056842 ], [ 106.682187969247821, 10.792182173185902 ], [ 106.682187969247821, 10.791501364660057 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d36c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.791460741971385 ], [ 106.683661975952418, 10.7914201173915 ], [ 106.683661975952418, 10.792100921036448 ], [ 106.682924971360933, 10.792141548056842 ], [ 106.682924971360933, 10.791460741971385 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d374", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.792141548056842 ], [ 106.683661975952418, 10.792100921036448 ], [ 106.683661975952418, 10.792781729817792 ], [ 106.682924971360933, 10.792822359278684 ], [ 106.682924971360933, 10.792141548056842 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d37c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.792182173185902 ], [ 106.682924971360933, 10.792141548056842 ], [ 106.682924971360933, 10.792822359278684 ], [ 106.682187969247821, 10.792862986848132 ], [ 106.682187969247821, 10.792182173185902 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d49c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.791541985457505 ], [ 106.682187969247821, 10.791501364660057 ], [ 106.682187969247821, 10.792182173185902 ], [ 106.681450969613365, 10.792222796423626 ], [ 106.681450969613365, 10.791541985457505 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.790861179628049 ], [ 106.682187969247821, 10.790820561270881 ], [ 106.682187969247821, 10.791501364660057 ], [ 106.681450969613365, 10.791541985457505 ], [ 106.681450969613365, 10.790861179628049 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.790180378935537 ], [ 106.682187969247821, 10.790139763018653 ], [ 106.682187969247821, 10.790820561270881 ], [ 106.681450969613365, 10.790861179628049 ], [ 106.681450969613365, 10.790180378935537 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.790220992961421 ], [ 106.681450969613365, 10.790180378935537 ], [ 106.681450969613365, 10.790861179628049 ], [ 106.680713972457966, 10.790901796094102 ], [ 106.680713972457966, 10.790220992961421 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.790901796094102 ], [ 106.681450969613365, 10.790861179628049 ], [ 106.681450969613365, 10.791541985457505 ], [ 106.680713972457966, 10.791582604363727 ], [ 106.680713972457966, 10.790901796094102 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.790942410669034 ], [ 106.680713972457966, 10.790901796094102 ], [ 106.680713972457966, 10.791582604363727 ], [ 106.679976977782019, 10.791623221378709 ], [ 106.679976977782019, 10.790942410669034 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.790261605096298 ], [ 106.680713972457966, 10.790220992961421 ], [ 106.680713972457966, 10.790901796094102 ], [ 106.679976977782019, 10.790942410669034 ], [ 106.679976977782019, 10.790261605096298 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.790302215340159 ], [ 106.679976977782019, 10.790261605096298 ], [ 106.679976977782019, 10.790942410669034 ], [ 106.679239985585866, 10.790983023352835 ], [ 106.679239985585866, 10.790302215340159 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.790983023352835 ], [ 106.679976977782019, 10.790942410669034 ], [ 106.679976977782019, 10.791623221378709 ], [ 106.679239985585866, 10.791663836502444 ], [ 106.679239985585866, 10.790983023352835 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d524", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.7910236341455 ], [ 106.679239985585866, 10.790983023352835 ], [ 106.679239985585866, 10.791663836502444 ], [ 106.678502995869849, 10.791704449734929 ], [ 106.678502995869849, 10.7910236341455 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d52c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.790342823692997 ], [ 106.679239985585866, 10.790302215340159 ], [ 106.679239985585866, 10.790983023352835 ], [ 106.678502995869849, 10.7910236341455 ], [ 106.678502995869849, 10.790342823692997 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d534", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.790383430154805 ], [ 106.678502995869849, 10.790342823692997 ], [ 106.678502995869849, 10.7910236341455 ], [ 106.677766008634421, 10.791064243047016 ], [ 106.677766008634421, 10.790383430154805 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d54c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.790424034725573 ], [ 106.677766008634421, 10.790383430154805 ], [ 106.677766008634421, 10.791064243047016 ], [ 106.677029023879811, 10.79110485005738 ], [ 106.677029023879811, 10.790424034725573 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d554", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.790464637405295 ], [ 106.677029023879811, 10.790424034725573 ], [ 106.677029023879811, 10.79110485005738 ], [ 106.67629204160653, 10.791145455176579 ], [ 106.67629204160653, 10.790464637405295 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "3175292aac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.790505238193962 ], [ 106.67629204160653, 10.790464637405295 ], [ 106.67629204160653, 10.791145455176579 ], [ 106.675555061814862, 10.791186058404611 ], [ 106.675555061814862, 10.790505238193962 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ecd8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.663763722948886, 10.787069546163119 ], [ 106.664500662984608, 10.787028990257635 ], [ 106.664500662984608, 10.787709816217969 ], [ 106.663763722948886, 10.787750374561005 ], [ 106.663763722948886, 10.787069546163119 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ecdf4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.663763722948886, 10.786388722904052 ], [ 106.664500662984608, 10.786348169436126 ], [ 106.664500662984608, 10.787028990257635 ], [ 106.663763722948886, 10.787069546163119 ], [ 106.663763722948886, 10.786388722904052 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed134", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.783341078603238 ], [ 106.670396272789503, 10.783300519766486 ], [ 106.670396272789503, 10.783981300525131 ], [ 106.669659312864042, 10.78402186180038 ], [ 106.669659312864042, 10.783341078603238 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed144", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.668922355423547, 10.784062421185652 ], [ 106.669659312864042, 10.78402186180038 ], [ 106.669659312864042, 10.784702650137508 ], [ 106.668922355423547, 10.784743211961157 ], [ 106.668922355423547, 10.784062421185652 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed14c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.78402186180038 ], [ 106.670396272789503, 10.783981300525131 ], [ 106.670396272789503, 10.78466208642377 ], [ 106.669659312864042, 10.784702650137508 ], [ 106.669659312864042, 10.78402186180038 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed154", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.784702650137508 ], [ 106.670396272789503, 10.78466208642377 ], [ 106.670396272789503, 10.785342877462115 ], [ 106.669659312864042, 10.785383443614341 ], [ 106.669659312864042, 10.784702650137508 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed15c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.668922355423547, 10.784743211961157 ], [ 106.669659312864042, 10.784702650137508 ], [ 106.669659312864042, 10.785383443614341 ], [ 106.668922355423547, 10.78542400787636 ], [ 106.668922355423547, 10.784743211961157 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed164", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.784783771894704 ], [ 106.668922355423547, 10.784743211961157 ], [ 106.668922355423547, 10.78542400787636 ], [ 106.66818540046836, 10.785464570248163 ], [ 106.66818540046836, 10.784783771894704 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed16c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.78410297868094 ], [ 106.668922355423547, 10.784062421185652 ], [ 106.668922355423547, 10.784743211961157 ], [ 106.66818540046836, 10.784783771894704 ], [ 106.66818540046836, 10.78410297868094 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed17c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.784824329938147 ], [ 106.66818540046836, 10.784783771894704 ], [ 106.66818540046836, 10.785464570248163 ], [ 106.667448447998822, 10.785505130729748 ], [ 106.667448447998822, 10.784824329938147 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed184", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.666711498015275, 10.784864886091478 ], [ 106.667448447998822, 10.784824329938147 ], [ 106.667448447998822, 10.785505130729748 ], [ 106.666711498015275, 10.785545689321102 ], [ 106.666711498015275, 10.784864886091478 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed20c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.786348169436126 ], [ 106.665237605507798, 10.786307614077824 ], [ 106.665237605507798, 10.786988432461662 ], [ 106.664500662984608, 10.787028990257635 ], [ 106.664500662984608, 10.786348169436126 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed214", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.786307614077824 ], [ 106.665974550518172, 10.786267056829157 ], [ 106.665974550518172, 10.786947872775208 ], [ 106.665237605507798, 10.786988432461662 ], [ 106.665237605507798, 10.786307614077824 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed21c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.785626800833098 ], [ 106.665974550518172, 10.785586246022225 ], [ 106.665974550518172, 10.786267056829157 ], [ 106.665237605507798, 10.786307614077824 ], [ 106.665237605507798, 10.785626800833098 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed224", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665974550518172, 10.785586246022225 ], [ 106.666711498015275, 10.785545689321102 ], [ 106.666711498015275, 10.786226497690134 ], [ 106.665974550518172, 10.786267056829157 ], [ 106.665974550518172, 10.785586246022225 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed22c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.666711498015275, 10.785545689321102 ], [ 106.667448447998822, 10.785505130729748 ], [ 106.667448447998822, 10.786185936660754 ], [ 106.666711498015275, 10.786226497690134 ], [ 106.666711498015275, 10.785545689321102 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed234", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.666711498015275, 10.786226497690134 ], [ 106.667448447998822, 10.786185936660754 ], [ 106.667448447998822, 10.786866747730887 ], [ 106.666711498015275, 10.786907311198281 ], [ 106.666711498015275, 10.786226497690134 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed23c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665974550518172, 10.786267056829157 ], [ 106.666711498015275, 10.786226497690134 ], [ 106.666711498015275, 10.786907311198281 ], [ 106.665974550518172, 10.786947872775208 ], [ 106.665974550518172, 10.786267056829157 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed244", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665974550518172, 10.786947872775208 ], [ 106.666711498015275, 10.786907311198281 ], [ 106.666711498015275, 10.787588129845266 ], [ 106.665974550518172, 10.787628693860093 ], [ 106.665974550518172, 10.786947872775208 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed24c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.666711498015275, 10.786907311198281 ], [ 106.667448447998822, 10.786866747730887 ], [ 106.667448447998822, 10.78754756393986 ], [ 106.666711498015275, 10.787588129845266 ], [ 106.666711498015275, 10.786907311198281 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed254", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.666711498015275, 10.787588129845266 ], [ 106.667448447998822, 10.78754756393986 ], [ 106.667448447998822, 10.788228385287391 ], [ 106.666711498015275, 10.788268953630807 ], [ 106.666711498015275, 10.787588129845266 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed25c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665974550518172, 10.787628693860093 ], [ 106.666711498015275, 10.787588129845266 ], [ 106.666711498015275, 10.788268953630807 ], [ 106.665974550518172, 10.788309520083525 ], [ 106.665974550518172, 10.787628693860093 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed264", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.78766925598433 ], [ 106.665974550518172, 10.787628693860093 ], [ 106.665974550518172, 10.788309520083525 ], [ 106.665237605507798, 10.788350084645538 ], [ 106.665237605507798, 10.78766925598433 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed26c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.786988432461662 ], [ 106.665974550518172, 10.786947872775208 ], [ 106.665974550518172, 10.787628693860093 ], [ 106.665237605507798, 10.78766925598433 ], [ 106.665237605507798, 10.786988432461662 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed274", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.787028990257635 ], [ 106.665237605507798, 10.786988432461662 ], [ 106.665237605507798, 10.78766925598433 ], [ 106.664500662984608, 10.787709816217969 ], [ 106.664500662984608, 10.787028990257635 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed27c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.787709816217969 ], [ 106.665237605507798, 10.78766925598433 ], [ 106.665237605507798, 10.788350084645538 ], [ 106.664500662984608, 10.788390647316842 ], [ 106.664500662984608, 10.787709816217969 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed284", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.788390647316842 ], [ 106.665237605507798, 10.788350084645538 ], [ 106.665237605507798, 10.789030918445011 ], [ 106.664500662984608, 10.789071483553974 ], [ 106.664500662984608, 10.788390647316842 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed28c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.788350084645538 ], [ 106.665974550518172, 10.788309520083525 ], [ 106.665974550518172, 10.788990351445225 ], [ 106.665237605507798, 10.789030918445011 ], [ 106.665237605507798, 10.788350084645538 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed294", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.789030918445011 ], [ 106.665974550518172, 10.788990351445225 ], [ 106.665974550518172, 10.789671187944908 ], [ 106.665237605507798, 10.789711757382463 ], [ 106.665237605507798, 10.789030918445011 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed29c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.789071483553974 ], [ 106.665237605507798, 10.789030918445011 ], [ 106.665237605507798, 10.789711757382463 ], [ 106.664500662984608, 10.789752324929079 ], [ 106.664500662984608, 10.789071483553974 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed2a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.789752324929079 ], [ 106.665237605507798, 10.789711757382463 ], [ 106.665237605507798, 10.79039260145761 ], [ 106.664500662984608, 10.790433171441876 ], [ 106.664500662984608, 10.789752324929079 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed2bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.789711757382463 ], [ 106.665974550518172, 10.789671187944908 ], [ 106.665974550518172, 10.790352029582291 ], [ 106.665237605507798, 10.79039260145761 ], [ 106.665237605507798, 10.789711757382463 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed2e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.666711498015275, 10.788949782554617 ], [ 106.667448447998822, 10.7889092117732 ], [ 106.667448447998822, 10.789590043397 ], [ 106.666711498015275, 10.789630616616417 ], [ 106.666711498015275, 10.788949782554617 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed2ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665974550518172, 10.788990351445225 ], [ 106.666711498015275, 10.788949782554617 ], [ 106.666711498015275, 10.789630616616417 ], [ 106.665974550518172, 10.789671187944908 ], [ 106.665974550518172, 10.788990351445225 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed2f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665974550518172, 10.788309520083525 ], [ 106.666711498015275, 10.788268953630807 ], [ 106.666711498015275, 10.788949782554617 ], [ 106.665974550518172, 10.788990351445225 ], [ 106.665974550518172, 10.788309520083525 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed2fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.666711498015275, 10.788268953630807 ], [ 106.667448447998822, 10.788228385287391 ], [ 106.667448447998822, 10.7889092117732 ], [ 106.666711498015275, 10.788949782554617 ], [ 106.666711498015275, 10.788268953630807 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed304", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.788228385287391 ], [ 106.66818540046836, 10.788187815053288 ], [ 106.66818540046836, 10.788868639100979 ], [ 106.667448447998822, 10.7889092117732 ], [ 106.667448447998822, 10.788228385287391 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed30c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.788187815053288 ], [ 106.668922355423547, 10.788147242928504 ], [ 106.668922355423547, 10.788828064537961 ], [ 106.66818540046836, 10.788868639100979 ], [ 106.66818540046836, 10.788187815053288 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed31c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.7889092117732 ], [ 106.66818540046836, 10.788868639100979 ], [ 106.66818540046836, 10.789549468286664 ], [ 106.667448447998822, 10.789590043397 ], [ 106.667448447998822, 10.7889092117732 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed374", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.668922355423547, 10.788147242928504 ], [ 106.669659312864042, 10.788106668913048 ], [ 106.669659312864042, 10.788787488084155 ], [ 106.668922355423547, 10.788828064537961 ], [ 106.668922355423547, 10.788147242928504 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed37c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.788106668913048 ], [ 106.670396272789503, 10.788066093006927 ], [ 106.670396272789503, 10.788746909739572 ], [ 106.669659312864042, 10.788787488084155 ], [ 106.669659312864042, 10.788106668913048 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed384", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.787425854880231 ], [ 106.670396272789503, 10.787385281412577 ], [ 106.670396272789503, 10.788066093006927 ], [ 106.669659312864042, 10.788106668913048 ], [ 106.669659312864042, 10.787425854880231 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed38c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.786745045985985 ], [ 106.670396272789503, 10.786704474956805 ], [ 106.670396272789503, 10.787385281412577 ], [ 106.669659312864042, 10.787425854880231 ], [ 106.669659312864042, 10.786745045985985 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed394", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.668922355423547, 10.78678561512473 ], [ 106.669659312864042, 10.786745045985985 ], [ 106.669659312864042, 10.787425854880231 ], [ 106.668922355423547, 10.787466426457335 ], [ 106.668922355423547, 10.78678561512473 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed39c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.668922355423547, 10.787466426457335 ], [ 106.669659312864042, 10.787425854880231 ], [ 106.669659312864042, 10.788106668913048 ], [ 106.668922355423547, 10.788147242928504 ], [ 106.668922355423547, 10.787466426457335 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.787506996143881 ], [ 106.668922355423547, 10.787466426457335 ], [ 106.668922355423547, 10.788147242928504 ], [ 106.66818540046836, 10.788187815053288 ], [ 106.66818540046836, 10.787506996143881 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.78754756393986 ], [ 106.66818540046836, 10.787506996143881 ], [ 106.66818540046836, 10.788187815053288 ], [ 106.667448447998822, 10.788228385287391 ], [ 106.667448447998822, 10.78754756393986 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.786866747730887 ], [ 106.66818540046836, 10.786826182373034 ], [ 106.66818540046836, 10.787506996143881 ], [ 106.667448447998822, 10.78754756393986 ], [ 106.667448447998822, 10.786866747730887 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.786826182373034 ], [ 106.668922355423547, 10.78678561512473 ], [ 106.668922355423547, 10.787466426457335 ], [ 106.66818540046836, 10.787506996143881 ], [ 106.66818540046836, 10.786826182373034 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.786145373741032 ], [ 106.668922355423547, 10.786104808930977 ], [ 106.668922355423547, 10.78678561512473 ], [ 106.66818540046836, 10.786826182373034 ], [ 106.66818540046836, 10.786145373741032 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.786185936660754 ], [ 106.66818540046836, 10.786145373741032 ], [ 106.66818540046836, 10.786826182373034 ], [ 106.667448447998822, 10.786866747730887 ], [ 106.667448447998822, 10.786185936660754 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.785505130729748 ], [ 106.66818540046836, 10.785464570248163 ], [ 106.66818540046836, 10.786145373741032 ], [ 106.667448447998822, 10.786185936660754 ], [ 106.667448447998822, 10.785505130729748 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.785464570248163 ], [ 106.668922355423547, 10.78542400787636 ], [ 106.668922355423547, 10.786104808930977 ], [ 106.66818540046836, 10.786145373741032 ], [ 106.66818540046836, 10.785464570248163 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.668922355423547, 10.78542400787636 ], [ 106.669659312864042, 10.785383443614341 ], [ 106.669659312864042, 10.786064242230593 ], [ 106.668922355423547, 10.786104808930977 ], [ 106.668922355423547, 10.78542400787636 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.668922355423547, 10.786104808930977 ], [ 106.669659312864042, 10.786064242230593 ], [ 106.669659312864042, 10.786745045985985 ], [ 106.668922355423547, 10.78678561512473 ], [ 106.668922355423547, 10.786104808930977 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.786064242230593 ], [ 106.670396272789503, 10.786023673639889 ], [ 106.670396272789503, 10.786704474956805 ], [ 106.669659312864042, 10.786745045985985 ], [ 106.669659312864042, 10.786064242230593 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.785383443614341 ], [ 106.670396272789503, 10.785342877462115 ], [ 106.670396272789503, 10.786023673639889 ], [ 106.669659312864042, 10.786064242230593 ], [ 106.669659312864042, 10.785383443614341 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed404", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.785342877462115 ], [ 106.671133235199477, 10.785302309419697 ], [ 106.671133235199477, 10.785983103158872 ], [ 106.670396272789503, 10.786023673639889 ], [ 106.670396272789503, 10.785342877462115 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed40c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.786023673639889 ], [ 106.671133235199477, 10.785983103158872 ], [ 106.671133235199477, 10.786663902037196 ], [ 106.670396272789503, 10.786704474956805 ], [ 106.670396272789503, 10.786023673639889 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed414", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.785983103158872 ], [ 106.671870200093736, 10.785942530787551 ], [ 106.671870200093736, 10.786623327227169 ], [ 106.671133235199477, 10.786663902037196 ], [ 106.671133235199477, 10.785983103158872 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed41c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.785302309419697 ], [ 106.671870200093736, 10.785261739487087 ], [ 106.671870200093736, 10.785942530787551 ], [ 106.671133235199477, 10.785983103158872 ], [ 106.671133235199477, 10.785302309419697 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed424", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.785261739487087 ], [ 106.67260716747171, 10.785221167664297 ], [ 106.67260716747171, 10.785901956525931 ], [ 106.671870200093736, 10.785942530787551 ], [ 106.671870200093736, 10.785261739487087 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed42c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.785221167664297 ], [ 106.673344137333231, 10.785180593951329 ], [ 106.673344137333231, 10.785861380374028 ], [ 106.67260716747171, 10.785901956525931 ], [ 106.67260716747171, 10.785221167664297 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed434", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.785901956525931 ], [ 106.673344137333231, 10.785861380374028 ], [ 106.673344137333231, 10.786542171935885 ], [ 106.67260716747171, 10.786582750526728 ], [ 106.67260716747171, 10.785901956525931 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed43c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.785942530787551 ], [ 106.67260716747171, 10.785901956525931 ], [ 106.67260716747171, 10.786582750526728 ], [ 106.671870200093736, 10.786623327227169 ], [ 106.671870200093736, 10.785942530787551 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed444", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.786623327227169 ], [ 106.67260716747171, 10.786582750526728 ], [ 106.67260716747171, 10.787263549666394 ], [ 106.671870200093736, 10.787304128805651 ], [ 106.671870200093736, 10.786623327227169 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed44c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.786582750526728 ], [ 106.673344137333231, 10.786542171935885 ], [ 106.673344137333231, 10.78722296863662 ], [ 106.67260716747171, 10.787263549666394 ], [ 106.67260716747171, 10.786582750526728 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed454", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.787263549666394 ], [ 106.673344137333231, 10.78722296863662 ], [ 106.673344137333231, 10.787903770475946 ], [ 106.67260716747171, 10.787944353944649 ], [ 106.67260716747171, 10.787263549666394 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed45c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.787304128805651 ], [ 106.67260716747171, 10.787263549666394 ], [ 106.67260716747171, 10.787944353944649 ], [ 106.671870200093736, 10.787984935522722 ], [ 106.671870200093736, 10.787304128805651 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed464", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.78734470605438 ], [ 106.671870200093736, 10.787304128805651 ], [ 106.671870200093736, 10.787984935522722 ], [ 106.671133235199477, 10.788025515210148 ], [ 106.671133235199477, 10.78734470605438 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed46c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.786663902037196 ], [ 106.671870200093736, 10.786623327227169 ], [ 106.671870200093736, 10.787304128805651 ], [ 106.671133235199477, 10.78734470605438 ], [ 106.671133235199477, 10.786663902037196 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed474", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.786704474956805 ], [ 106.671133235199477, 10.786663902037196 ], [ 106.671133235199477, 10.78734470605438 ], [ 106.670396272789503, 10.787385281412577 ], [ 106.670396272789503, 10.786704474956805 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed47c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.787385281412577 ], [ 106.671133235199477, 10.78734470605438 ], [ 106.671133235199477, 10.788025515210148 ], [ 106.670396272789503, 10.788066093006927 ], [ 106.670396272789503, 10.787385281412577 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed484", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.788066093006927 ], [ 106.671133235199477, 10.788025515210148 ], [ 106.671133235199477, 10.788706329504215 ], [ 106.670396272789503, 10.788746909739572 ], [ 106.670396272789503, 10.788066093006927 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed48c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.788025515210148 ], [ 106.671870200093736, 10.787984935522722 ], [ 106.671870200093736, 10.788665747378094 ], [ 106.671133235199477, 10.788706329504215 ], [ 106.671133235199477, 10.788025515210148 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed494", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.788706329504215 ], [ 106.671870200093736, 10.788665747378094 ], [ 106.671870200093736, 10.789346564371487 ], [ 106.671133235199477, 10.789387148936298 ], [ 106.671133235199477, 10.788706329504215 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed49c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.788746909739572 ], [ 106.671133235199477, 10.788706329504215 ], [ 106.671133235199477, 10.789387148936298 ], [ 106.670396272789503, 10.789427731610227 ], [ 106.670396272789503, 10.788746909739572 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.789387148936298 ], [ 106.671870200093736, 10.789346564371487 ], [ 106.671870200093736, 10.790027386502617 ], [ 106.671133235199477, 10.790067973506108 ], [ 106.671133235199477, 10.789387148936298 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.789346564371487 ], [ 106.67260716747171, 10.789305977915806 ], [ 106.67260716747171, 10.789986797608133 ], [ 106.671870200093736, 10.790027386502617 ], [ 106.671870200093736, 10.789346564371487 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.789986797608133 ], [ 106.673344137333231, 10.789946206822675 ], [ 106.673344137333231, 10.790627029213551 ], [ 106.67260716747171, 10.790667622437921 ], [ 106.67260716747171, 10.789986797608133 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.789305977915806 ], [ 106.673344137333231, 10.789265389569259 ], [ 106.673344137333231, 10.789946206822675 ], [ 106.67260716747171, 10.789986797608133 ], [ 106.67260716747171, 10.789305977915806 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.788625163361216 ], [ 106.673344137333231, 10.788584577453591 ], [ 106.673344137333231, 10.789265389569259 ], [ 106.67260716747171, 10.789305977915806 ], [ 106.67260716747171, 10.788625163361216 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.788665747378094 ], [ 106.67260716747171, 10.788625163361216 ], [ 106.67260716747171, 10.789305977915806 ], [ 106.671870200093736, 10.789346564371487 ], [ 106.671870200093736, 10.788665747378094 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.787984935522722 ], [ 106.67260716747171, 10.787944353944649 ], [ 106.67260716747171, 10.788625163361216 ], [ 106.671870200093736, 10.788665747378094 ], [ 106.671870200093736, 10.787984935522722 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.787944353944649 ], [ 106.673344137333231, 10.787903770475946 ], [ 106.673344137333231, 10.788584577453591 ], [ 106.67260716747171, 10.788625163361216 ], [ 106.67260716747171, 10.787944353944649 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed504", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.787903770475946 ], [ 106.674081109677786, 10.787863185116619 ], [ 106.674081109677786, 10.788543989655222 ], [ 106.673344137333231, 10.788584577453591 ], [ 106.673344137333231, 10.787903770475946 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed50c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.787863185116619 ], [ 106.674818084505148, 10.78782259786667 ], [ 106.674818084505148, 10.788503399966121 ], [ 106.674081109677786, 10.788543989655222 ], [ 106.674081109677786, 10.787863185116619 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed514", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.788543989655222 ], [ 106.674818084505148, 10.788503399966121 ], [ 106.674818084505148, 10.789184207203608 ], [ 106.674081109677843, 10.789224799331857 ], [ 106.674081109677786, 10.788543989655222 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed51c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.788584577453591 ], [ 106.674081109677786, 10.788543989655222 ], [ 106.674081109677843, 10.789224799331857 ], [ 106.673344137333231, 10.789265389569259 ], [ 106.673344137333231, 10.788584577453591 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed524", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.789265389569259 ], [ 106.674081109677843, 10.789224799331857 ], [ 106.674081109677786, 10.789905614146242 ], [ 106.673344137333231, 10.789946206822675 ], [ 106.673344137333231, 10.789265389569259 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed52c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.789946206822675 ], [ 106.674081109677786, 10.789905614146242 ], [ 106.674081109677843, 10.790586434098095 ], [ 106.673344137333231, 10.790627029213551 ], [ 106.673344137333231, 10.789946206822675 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed534", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.789905614146242 ], [ 106.674818084505148, 10.789865019578849 ], [ 106.674818084505148, 10.790545837091564 ], [ 106.674081109677843, 10.790586434098095 ], [ 106.674081109677786, 10.789905614146242 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed53c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677843, 10.789224799331857 ], [ 106.674818084505148, 10.789184207203608 ], [ 106.674818084505148, 10.789865019578849 ], [ 106.674081109677786, 10.789905614146242 ], [ 106.674081109677843, 10.789224799331857 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed544", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.789184207203608 ], [ 106.675555061814862, 10.789143613184518 ], [ 106.675555061814862, 10.789824423120502 ], [ 106.674818084505148, 10.789865019578849 ], [ 106.674818084505148, 10.789184207203608 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed54c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.789865019578849 ], [ 106.675555061814862, 10.789824423120502 ], [ 106.675555061814862, 10.790505238193962 ], [ 106.674818084505148, 10.790545837091564 ], [ 106.674818084505148, 10.789865019578849 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed554", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.789824423120502 ], [ 106.67629204160653, 10.789783824771202 ], [ 106.67629204160653, 10.790464637405295 ], [ 106.675555061814862, 10.790505238193962 ], [ 106.675555061814862, 10.789824423120502 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed55c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.789143613184518 ], [ 106.67629204160653, 10.789103017274595 ], [ 106.67629204160653, 10.789783824771202 ], [ 106.675555061814862, 10.789824423120502 ], [ 106.675555061814862, 10.789143613184518 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed564", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.788462808386294 ], [ 106.67629204160653, 10.78842221491575 ], [ 106.67629204160653, 10.789103017274595 ], [ 106.675555061814862, 10.789143613184518 ], [ 106.675555061814862, 10.788462808386294 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed56c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.788503399966121 ], [ 106.675555061814862, 10.788462808386294 ], [ 106.675555061814862, 10.789143613184518 ], [ 106.674818084505148, 10.789184207203608 ], [ 106.674818084505148, 10.788503399966121 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed574", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.78782259786667 ], [ 106.675555061814862, 10.787782008726113 ], [ 106.675555061814862, 10.788462808386294 ], [ 106.674818084505148, 10.788503399966121 ], [ 106.674818084505148, 10.78782259786667 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed57c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.787782008726113 ], [ 106.67629204160653, 10.787741417694953 ], [ 106.67629204160653, 10.78842221491575 ], [ 106.675555061814862, 10.788462808386294 ], [ 106.675555061814862, 10.787782008726113 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed584", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.787101214204256 ], [ 106.67629204160653, 10.787060625612483 ], [ 106.67629204160653, 10.787741417694953 ], [ 106.675555061814862, 10.787782008726113 ], [ 106.675555061814862, 10.787101214204256 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed58c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.78642042482101 ], [ 106.67629204160653, 10.786379838668628 ], [ 106.67629204160653, 10.787060625612483 ], [ 106.675555061814862, 10.787101214204256 ], [ 106.675555061814862, 10.78642042482101 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed594", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.786461009083018 ], [ 106.675555061814862, 10.78642042482101 ], [ 106.675555061814862, 10.787101214204256 ], [ 106.674818084505148, 10.787141800905543 ], [ 106.674818084505148, 10.786461009083018 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed59c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.787141800905543 ], [ 106.675555061814862, 10.787101214204256 ], [ 106.675555061814862, 10.787782008726113 ], [ 106.674818084505148, 10.78782259786667 ], [ 106.674818084505148, 10.787141800905543 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677843, 10.787182385716331 ], [ 106.674818084505148, 10.787141800905543 ], [ 106.674818084505148, 10.78782259786667 ], [ 106.674081109677786, 10.787863185116619 ], [ 106.674081109677843, 10.787182385716331 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.78722296863662 ], [ 106.674081109677843, 10.787182385716331 ], [ 106.674081109677786, 10.787863185116619 ], [ 106.673344137333231, 10.787903770475946 ], [ 106.673344137333231, 10.78722296863662 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.786542171935885 ], [ 106.674081109677786, 10.786501591454643 ], [ 106.674081109677843, 10.787182385716331 ], [ 106.673344137333231, 10.78722296863662 ], [ 106.673344137333231, 10.786542171935885 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.786501591454643 ], [ 106.674818084505148, 10.786461009083018 ], [ 106.674818084505148, 10.787141800905543 ], [ 106.674081109677843, 10.787182385716331 ], [ 106.674081109677786, 10.786501591454643 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677843, 10.785820802331838 ], [ 106.674818084505148, 10.785780222399378 ], [ 106.674818084505148, 10.786461009083018 ], [ 106.674081109677786, 10.786501591454643 ], [ 106.674081109677843, 10.785820802331838 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.785861380374028 ], [ 106.674081109677843, 10.785820802331838 ], [ 106.674081109677786, 10.786501591454643 ], [ 106.673344137333231, 10.786542171935885 ], [ 106.673344137333231, 10.785861380374028 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.785180593951329 ], [ 106.674081109677786, 10.7851400183482 ], [ 106.674081109677843, 10.785820802331838 ], [ 106.673344137333231, 10.785861380374028 ], [ 106.673344137333231, 10.785180593951329 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.7851400183482 ], [ 106.674818084505148, 10.785099440854909 ], [ 106.674818084505148, 10.785780222399378 ], [ 106.674081109677843, 10.785820802331838 ], [ 106.674081109677786, 10.7851400183482 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.785099440854909 ], [ 106.675555061814862, 10.78505886147147 ], [ 106.675555061814862, 10.785739640576651 ], [ 106.674818084505148, 10.785780222399378 ], [ 106.674818084505148, 10.785099440854909 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.785780222399378 ], [ 106.675555061814862, 10.785739640576651 ], [ 106.675555061814862, 10.78642042482101 ], [ 106.674818084505148, 10.786461009083018 ], [ 106.674818084505148, 10.785780222399378 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.785739640576651 ], [ 106.67629204160653, 10.785699056863669 ], [ 106.67629204160653, 10.786379838668628 ], [ 106.675555061814862, 10.78642042482101 ], [ 106.675555061814862, 10.785739640576651 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.78505886147147 ], [ 106.67629204160653, 10.785018280197887 ], [ 106.67629204160653, 10.785699056863669 ], [ 106.675555061814862, 10.785739640576651 ], [ 106.675555061814862, 10.78505886147147 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed604", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.784378087505743 ], [ 106.67629204160653, 10.784337508671566 ], [ 106.67629204160653, 10.785018280197887 ], [ 106.675555061814862, 10.78505886147147 ], [ 106.675555061814862, 10.784378087505743 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed60c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.784418664449891 ], [ 106.675555061814862, 10.784378087505743 ], [ 106.675555061814862, 10.78505886147147 ], [ 106.674818084505148, 10.785099440854909 ], [ 106.674818084505148, 10.784418664449891 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed614", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.783737893184611 ], [ 106.675555061814862, 10.783697318679758 ], [ 106.675555061814862, 10.784378087505743 ], [ 106.674818084505148, 10.784418664449891 ], [ 106.674818084505148, 10.783737893184611 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed61c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.783697318679758 ], [ 106.67629204160653, 10.783656742284991 ], [ 106.67629204160653, 10.784337508671566 ], [ 106.675555061814862, 10.784378087505743 ], [ 106.675555061814862, 10.783697318679758 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed624", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.783016554993795 ], [ 106.67629204160653, 10.782975981038442 ], [ 106.67629204160653, 10.783656742284991 ], [ 106.675555061814862, 10.783697318679758 ], [ 106.675555061814862, 10.783016554993795 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed62c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.782335796448136 ], [ 106.67629204160653, 10.782295224932202 ], [ 106.67629204160653, 10.782975981038442 ], [ 106.675555061814862, 10.783016554993795 ], [ 106.675555061814862, 10.782335796448136 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed634", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.782376366074388 ], [ 106.675555061814862, 10.782335796448136 ], [ 106.675555061814862, 10.783016554993795 ], [ 106.674818084505148, 10.783057127059347 ], [ 106.674818084505148, 10.782376366074388 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed63c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.783057127059347 ], [ 106.675555061814862, 10.783016554993795 ], [ 106.675555061814862, 10.783697318679758 ], [ 106.674818084505148, 10.783737893184611 ], [ 106.674818084505148, 10.783057127059347 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed644", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677843, 10.783097697235098 ], [ 106.674818084505148, 10.783057127059347 ], [ 106.674818084505148, 10.783737893184611 ], [ 106.674081109677843, 10.783778465799543 ], [ 106.674081109677843, 10.783097697235098 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed64c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.782416933810946 ], [ 106.674818084505148, 10.782376366074388 ], [ 106.674818084505148, 10.783057127059347 ], [ 106.674081109677843, 10.783097697235098 ], [ 106.674081109677786, 10.782416933810946 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed654", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.782457499657808 ], [ 106.674081109677786, 10.782416933810946 ], [ 106.674081109677843, 10.783097697235098 ], [ 106.673344137333231, 10.783138265521034 ], [ 106.673344137333231, 10.782457499657808 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed65c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.783138265521034 ], [ 106.674081109677843, 10.783097697235098 ], [ 106.674081109677843, 10.783778465799543 ], [ 106.673344137333231, 10.78381903652455 ], [ 106.673344137333231, 10.783138265521034 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed664", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.78381903652455 ], [ 106.674081109677843, 10.783778465799543 ], [ 106.674081109677786, 10.784459239504006 ], [ 106.673344137333231, 10.784499812668075 ], [ 106.673344137333231, 10.78381903652455 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed66c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677843, 10.783778465799543 ], [ 106.674818084505148, 10.783737893184611 ], [ 106.674818084505148, 10.784418664449891 ], [ 106.674081109677786, 10.784459239504006 ], [ 106.674081109677843, 10.783778465799543 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed674", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.784459239504006 ], [ 106.674818084505148, 10.784418664449891 ], [ 106.674818084505148, 10.785099440854909 ], [ 106.674081109677786, 10.7851400183482 ], [ 106.674081109677786, 10.784459239504006 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed67c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.784499812668075 ], [ 106.674081109677786, 10.784459239504006 ], [ 106.674081109677786, 10.7851400183482 ], [ 106.673344137333231, 10.785180593951329 ], [ 106.673344137333231, 10.784499812668075 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed684", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.784540383942097 ], [ 106.673344137333231, 10.784499812668075 ], [ 106.673344137333231, 10.785180593951329 ], [ 106.67260716747171, 10.785221167664297 ], [ 106.67260716747171, 10.784540383942097 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed68c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.78385960535962 ], [ 106.673344137333231, 10.78381903652455 ], [ 106.673344137333231, 10.784499812668075 ], [ 106.67260716747171, 10.784540383942097 ], [ 106.67260716747171, 10.78385960535962 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed694", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.783900172304746 ], [ 106.67260716747171, 10.78385960535962 ], [ 106.67260716747171, 10.784540383942097 ], [ 106.671870200093736, 10.784580953326058 ], [ 106.671870200093736, 10.783900172304746 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed69c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.784580953326058 ], [ 106.67260716747171, 10.784540383942097 ], [ 106.67260716747171, 10.785221167664297 ], [ 106.671870200093736, 10.785261739487087 ], [ 106.671870200093736, 10.784580953326058 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.784621520819952 ], [ 106.671870200093736, 10.784580953326058 ], [ 106.671870200093736, 10.785261739487087 ], [ 106.671133235199477, 10.785302309419697 ], [ 106.671133235199477, 10.784621520819952 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.78466208642377 ], [ 106.671133235199477, 10.784621520819952 ], [ 106.671133235199477, 10.785302309419697 ], [ 106.670396272789503, 10.785342877462115 ], [ 106.670396272789503, 10.78466208642377 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.783981300525131 ], [ 106.671133235199477, 10.78394073735992 ], [ 106.671133235199477, 10.784621520819952 ], [ 106.670396272789503, 10.78466208642377 ], [ 106.670396272789503, 10.783981300525131 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.78394073735992 ], [ 106.671870200093736, 10.783900172304746 ], [ 106.671870200093736, 10.784580953326058 ], [ 106.671133235199477, 10.784621520819952 ], [ 106.671133235199477, 10.78394073735992 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.783259959039881 ], [ 106.671870200093736, 10.783219396423434 ], [ 106.671870200093736, 10.783900172304746 ], [ 106.671133235199477, 10.78394073735992 ], [ 106.671133235199477, 10.783259959039881 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.783300519766486 ], [ 106.671133235199477, 10.783259959039881 ], [ 106.671133235199477, 10.78394073735992 ], [ 106.670396272789503, 10.783981300525131 ], [ 106.670396272789503, 10.783300519766486 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.782579185860124 ], [ 106.671870200093736, 10.782538625682404 ], [ 106.671870200093736, 10.783219396423434 ], [ 106.671133235199477, 10.783259959039881 ], [ 106.671133235199477, 10.782579185860124 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.782538625682404 ], [ 106.67260716747171, 10.782498063614963 ], [ 106.67260716747171, 10.783178831917146 ], [ 106.671870200093736, 10.783219396423434 ], [ 106.671870200093736, 10.782538625682404 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.783219396423434 ], [ 106.67260716747171, 10.783178831917146 ], [ 106.67260716747171, 10.78385960535962 ], [ 106.671870200093736, 10.783900172304746 ], [ 106.671870200093736, 10.783219396423434 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.783178831917146 ], [ 106.673344137333231, 10.783138265521034 ], [ 106.673344137333231, 10.78381903652455 ], [ 106.67260716747171, 10.78385960535962 ], [ 106.67260716747171, 10.783178831917146 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.782498063614963 ], [ 106.673344137333231, 10.782457499657808 ], [ 106.673344137333231, 10.783138265521034 ], [ 106.67260716747171, 10.783178831917146 ], [ 106.67260716747171, 10.782498063614963 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed704", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.78181730045335 ], [ 106.673344137333231, 10.781776738935159 ], [ 106.673344137333231, 10.782457499657808 ], [ 106.67260716747171, 10.782498063614963 ], [ 106.67260716747171, 10.78181730045335 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.781776738935159 ], [ 106.674081109677786, 10.781736175527376 ], [ 106.674081109677786, 10.782416933810946 ], [ 106.673344137333231, 10.782457499657808 ], [ 106.673344137333231, 10.781776738935159 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.781736175527376 ], [ 106.674818084505148, 10.781695610230008 ], [ 106.674818084505148, 10.782376366074388 ], [ 106.674081109677786, 10.782416933810946 ], [ 106.674081109677786, 10.781736175527376 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.781055422384666 ], [ 106.674818084505148, 10.781014859526501 ], [ 106.674818084505148, 10.781695610230008 ], [ 106.674081109677786, 10.781736175527376 ], [ 106.674081109677786, 10.781055422384666 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.781014859526501 ], [ 106.675555061814862, 10.780974294778872 ], [ 106.675555061814862, 10.781655043043068 ], [ 106.674818084505148, 10.781695610230008 ], [ 106.674818084505148, 10.781014859526501 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.781695610230008 ], [ 106.675555061814862, 10.781655043043068 ], [ 106.675555061814862, 10.782335796448136 ], [ 106.674818084505148, 10.782376366074388 ], [ 106.674818084505148, 10.781695610230008 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.781655043043068 ], [ 106.67629204160653, 10.78161447396656 ], [ 106.67629204160653, 10.782295224932202 ], [ 106.675555061814862, 10.782335796448136 ], [ 106.675555061814862, 10.781655043043068 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.780974294778872 ], [ 106.67629204160653, 10.780933728141791 ], [ 106.67629204160653, 10.78161447396656 ], [ 106.675555061814862, 10.781655043043068 ], [ 106.675555061814862, 10.780974294778872 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.780293551655827 ], [ 106.67629204160653, 10.780252987458182 ], [ 106.67629204160653, 10.780933728141791 ], [ 106.675555061814862, 10.780974294778872 ], [ 106.675555061814862, 10.780293551655827 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752edf8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.770083021941073 ], [ 106.67629204160653, 10.770042494335451 ], [ 106.67629204160653, 10.770723157867735 ], [ 106.675555061814862, 10.770763687912856 ], [ 106.675555061814862, 10.770083021941073 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752edfe4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.768762226274051 ], [ 106.675555061814862, 10.768721705434828 ], [ 106.675555061814862, 10.769402361114967 ], [ 106.674818084505148, 10.769442884393589 ], [ 106.674818084505148, 10.768762226274051 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752edff4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.769402361114967 ], [ 106.67629204160653, 10.76936183594885 ], [ 106.67629204160653, 10.770042494335451 ], [ 106.675555061814862, 10.770083021941073 ], [ 106.675555061814862, 10.769402361114967 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752edffc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.768721705434828 ], [ 106.67629204160653, 10.768681182708217 ], [ 106.67629204160653, 10.76936183594885 ], [ 106.675555061814862, 10.769402361114967 ], [ 106.675555061814862, 10.768721705434828 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee004", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.76804105490093 ], [ 106.67629204160653, 10.768000534613833 ], [ 106.67629204160653, 10.768681182708217 ], [ 106.675555061814862, 10.768721705434828 ], [ 106.675555061814862, 10.76804105490093 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee00c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.768081573300758 ], [ 106.675555061814862, 10.76804105490093 ], [ 106.675555061814862, 10.768721705434828 ], [ 106.674818084505148, 10.768762226274051 ], [ 106.674818084505148, 10.768081573300758 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee014", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.767400925473986 ], [ 106.675555061814862, 10.767360409513559 ], [ 106.675555061814862, 10.76804105490093 ], [ 106.674818084505148, 10.768081573300758 ], [ 106.674818084505148, 10.767400925473986 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee01c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.767360409513559 ], [ 106.67629204160653, 10.767319891665981 ], [ 106.67629204160653, 10.768000534613833 ], [ 106.675555061814862, 10.76804105490093 ], [ 106.675555061814862, 10.767360409513559 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee024", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.766679769273001 ], [ 106.67629204160653, 10.766639253864945 ], [ 106.67629204160653, 10.767319891665981 ], [ 106.675555061814862, 10.767360409513559 ], [ 106.675555061814862, 10.766679769273001 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee03c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.76672028279402 ], [ 106.675555061814862, 10.766679769273001 ], [ 106.675555061814862, 10.767360409513559 ], [ 106.674818084505148, 10.767400925473986 ], [ 106.674818084505148, 10.76672028279402 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee06c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.767441439547252 ], [ 106.674818084505148, 10.767400925473986 ], [ 106.674818084505148, 10.768081573300758 ], [ 106.674081109677786, 10.768122089813311 ], [ 106.674081109677786, 10.767441439547252 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee074", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.768122089813311 ], [ 106.674818084505148, 10.768081573300758 ], [ 106.674818084505148, 10.768762226274051 ], [ 106.674081109677843, 10.768802745225889 ], [ 106.674081109677786, 10.768122089813311 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1894", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.766914609398494 ], [ 106.684398983021822, 10.766874070792747 ], [ 106.684398983021822, 10.767554686898322 ], [ 106.683661975952418, 10.767595227944849 ], [ 106.683661975952418, 10.766914609398494 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f189c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.767595227944849 ], [ 106.684398983021822, 10.767554686898322 ], [ 106.684398983021822, 10.768235308150475 ], [ 106.683661975952418, 10.76827585163778 ], [ 106.683661975952418, 10.767595227944849 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.767635767104196 ], [ 106.683661975952418, 10.767595227944849 ], [ 106.683661975952418, 10.76827585163778 ], [ 106.682924971360933, 10.768316393237788 ], [ 106.682924971360933, 10.767635767104196 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.767676304376351 ], [ 106.682924971360933, 10.767635767104196 ], [ 106.682924971360933, 10.768316393237788 ], [ 106.682187969247821, 10.768356932950493 ], [ 106.682187969247821, 10.767676304376351 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.766995680948776 ], [ 106.682924971360933, 10.766955146117171 ], [ 106.682924971360933, 10.767635767104196 ], [ 106.682187969247821, 10.767676304376351 ], [ 106.682187969247821, 10.766995680948776 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.766955146117171 ], [ 106.683661975952418, 10.766914609398494 ], [ 106.683661975952418, 10.767595227944849 ], [ 106.682924971360933, 10.767635767104196 ], [ 106.682924971360933, 10.766955146117171 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.766274530277006 ], [ 106.683661975952418, 10.766233995998997 ], [ 106.683661975952418, 10.766914609398494 ], [ 106.682924971360933, 10.766955146117171 ], [ 106.682924971360933, 10.766274530277006 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.766315062668054 ], [ 106.682924971360933, 10.766274530277006 ], [ 106.682924971360933, 10.766955146117171 ], [ 106.682187969247821, 10.766995680948776 ], [ 106.682187969247821, 10.766315062668054 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.765634449534463 ], [ 106.682924971360933, 10.765593919583974 ], [ 106.682924971360933, 10.766274530277006 ], [ 106.682187969247821, 10.766315062668054 ], [ 106.682187969247821, 10.765634449534463 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1ed4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.76499436717147 ], [ 106.682187969247821, 10.764953841548287 ], [ 106.682187969247821, 10.765634449534463 ], [ 106.681450969613365, 10.765674977598096 ], [ 106.681450969613365, 10.76499436717147 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1edc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.765034890907909 ], [ 106.681450969613365, 10.76499436717147 ], [ 106.681450969613365, 10.765674977598096 ], [ 106.680713972457966, 10.765715503774869 ], [ 106.680713972457966, 10.765034890907909 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f04", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.765796550467805 ], [ 106.679976977782019, 10.765756028064775 ], [ 106.679976977782019, 10.766436648519358 ], [ 106.679239985585866, 10.766477173362489 ], [ 106.679239985585866, 10.765796550467805 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f0c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.766477173362489 ], [ 106.679976977782019, 10.766436648519358 ], [ 106.679976977782019, 10.767117274121061 ], [ 106.679239985585866, 10.76715780140429 ], [ 106.679239985585866, 10.766477173362489 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f14", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.766436648519358 ], [ 106.680713972457966, 10.766396121789237 ], [ 106.680713972457966, 10.767076744950728 ], [ 106.679976977782019, 10.767117274121061 ], [ 106.679976977782019, 10.766436648519358 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f1c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.765756028064775 ], [ 106.680713972457966, 10.765715503774869 ], [ 106.680713972457966, 10.766396121789237 ], [ 106.679976977782019, 10.766436648519358 ], [ 106.679976977782019, 10.765756028064775 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f24", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.765715503774869 ], [ 106.681450969613365, 10.765674977598096 ], [ 106.681450969613365, 10.766355593172133 ], [ 106.680713972457966, 10.766396121789237 ], [ 106.680713972457966, 10.765715503774869 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f2c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.765674977598096 ], [ 106.682187969247821, 10.765634449534463 ], [ 106.682187969247821, 10.766315062668054 ], [ 106.681450969613365, 10.766355593172133 ], [ 106.681450969613365, 10.765674977598096 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f34", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.766355593172133 ], [ 106.682187969247821, 10.766315062668054 ], [ 106.682187969247821, 10.766995680948776 ], [ 106.681450969613365, 10.767036213893299 ], [ 106.681450969613365, 10.766355593172133 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f3c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.766396121789237 ], [ 106.681450969613365, 10.766355593172133 ], [ 106.681450969613365, 10.767036213893299 ], [ 106.680713972457966, 10.767076744950728 ], [ 106.680713972457966, 10.766396121789237 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f44", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.767076744950728 ], [ 106.681450969613365, 10.767036213893299 ], [ 106.681450969613365, 10.767716839761308 ], [ 106.680713972457966, 10.767757373259062 ], [ 106.680713972457966, 10.767076744950728 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.767036213893299 ], [ 106.682187969247821, 10.766995680948776 ], [ 106.682187969247821, 10.767676304376351 ], [ 106.681450969613365, 10.767716839761308 ], [ 106.681450969613365, 10.767036213893299 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.767716839761308 ], [ 106.682187969247821, 10.767676304376351 ], [ 106.682187969247821, 10.768356932950493 ], [ 106.681450969613365, 10.768397470775884 ], [ 106.681450969613365, 10.767716839761308 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.767757373259062 ], [ 106.681450969613365, 10.767716839761308 ], [ 106.681450969613365, 10.768397470775884 ], [ 106.680713972457966, 10.768438006713955 ], [ 106.680713972457966, 10.767757373259062 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f64", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.767797904869605 ], [ 106.680713972457966, 10.767757373259062 ], [ 106.680713972457966, 10.768438006713955 ], [ 106.679976977782019, 10.768478540764699 ], [ 106.679976977782019, 10.767797904869605 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f6c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.767117274121061 ], [ 106.680713972457966, 10.767076744950728 ], [ 106.680713972457966, 10.767757373259062 ], [ 106.679976977782019, 10.767797904869605 ], [ 106.679976977782019, 10.767117274121061 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f74", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.76715780140429 ], [ 106.679976977782019, 10.767117274121061 ], [ 106.679976977782019, 10.767797904869605 ], [ 106.679239985585866, 10.767838434592921 ], [ 106.679239985585866, 10.76715780140429 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f7c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.767838434592921 ], [ 106.679976977782019, 10.767797904869605 ], [ 106.679976977782019, 10.768478540764699 ], [ 106.679239985585866, 10.768519072928106 ], [ 106.679239985585866, 10.767838434592921 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f84", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.767878962429014 ], [ 106.679239985585866, 10.767838434592921 ], [ 106.679239985585866, 10.768519072928106 ], [ 106.678502995869849, 10.768559603204169 ], [ 106.678502995869849, 10.767878962429014 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.767919488377867 ], [ 106.678502995869849, 10.767878962429014 ], [ 106.678502995869849, 10.768559603204169 ], [ 106.677766008634421, 10.76860013159288 ], [ 106.677766008634421, 10.767919488377867 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f94", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.767238850309395 ], [ 106.678502995869849, 10.767198326800402 ], [ 106.678502995869849, 10.767878962429014 ], [ 106.677766008634421, 10.767919488377867 ], [ 106.677766008634421, 10.767238850309395 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f9c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.767198326800402 ], [ 106.679239985585866, 10.76715780140429 ], [ 106.679239985585866, 10.767838434592921 ], [ 106.678502995869849, 10.767878962429014 ], [ 106.678502995869849, 10.767198326800402 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fa4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.766517696318621 ], [ 106.679239985585866, 10.766477173362489 ], [ 106.679239985585866, 10.76715780140429 ], [ 106.678502995869849, 10.767198326800402 ], [ 106.678502995869849, 10.766517696318621 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.765837070983951 ], [ 106.679239985585866, 10.765796550467805 ], [ 106.679239985585866, 10.766477173362489 ], [ 106.678502995869849, 10.766517696318621 ], [ 106.678502995869849, 10.765837070983951 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fb4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.765877589613204 ], [ 106.678502995869849, 10.765837070983951 ], [ 106.678502995869849, 10.766517696318621 ], [ 106.677766008634421, 10.766558217387745 ], [ 106.677766008634421, 10.765877589613204 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fbc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.766558217387745 ], [ 106.678502995869849, 10.766517696318621 ], [ 106.678502995869849, 10.767198326800402 ], [ 106.677766008634421, 10.767238850309395 ], [ 106.677766008634421, 10.766558217387745 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fc4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.766598736569856 ], [ 106.677766008634421, 10.766558217387745 ], [ 106.677766008634421, 10.767238850309395 ], [ 106.677029023879811, 10.767279371931256 ], [ 106.677029023879811, 10.766598736569856 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fdc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.766639253864945 ], [ 106.677029023879811, 10.766598736569856 ], [ 106.677029023879811, 10.767279371931256 ], [ 106.67629204160653, 10.767319891665981 ], [ 106.67629204160653, 10.766639253864945 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fe4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.767319891665981 ], [ 106.677029023879811, 10.767279371931256 ], [ 106.677029023879811, 10.767960012439476 ], [ 106.67629204160653, 10.768000534613833 ], [ 106.67629204160653, 10.767319891665981 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.767279371931256 ], [ 106.677766008634421, 10.767238850309395 ], [ 106.677766008634421, 10.767919488377867 ], [ 106.677029023879811, 10.767960012439476 ], [ 106.677029023879811, 10.767279371931256 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1ff4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.767960012439476 ], [ 106.677766008634421, 10.767919488377867 ], [ 106.677766008634421, 10.76860013159288 ], [ 106.677029023879811, 10.768640658094233 ], [ 106.677029023879811, 10.767960012439476 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1ffc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.768000534613833 ], [ 106.677029023879811, 10.767960012439476 ], [ 106.677029023879811, 10.768640658094233 ], [ 106.67629204160653, 10.768681182708217 ], [ 106.67629204160653, 10.768000534613833 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2004", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.768681182708217 ], [ 106.677029023879811, 10.768640658094233 ], [ 106.677029023879811, 10.769321308895242 ], [ 106.67629204160653, 10.76936183594885 ], [ 106.67629204160653, 10.768681182708217 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f200c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.76936183594885 ], [ 106.677029023879811, 10.769321308895242 ], [ 106.677029023879811, 10.770001964842223 ], [ 106.67629204160653, 10.770042494335451 ], [ 106.67629204160653, 10.76936183594885 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2014", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.769321308895242 ], [ 106.677766008634421, 10.769280779954151 ], [ 106.677766008634421, 10.769961433461399 ], [ 106.677029023879811, 10.770001964842223 ], [ 106.677029023879811, 10.769321308895242 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f201c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.768640658094233 ], [ 106.677766008634421, 10.76860013159288 ], [ 106.677766008634421, 10.769280779954151 ], [ 106.677029023879811, 10.769321308895242 ], [ 106.677029023879811, 10.768640658094233 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2024", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.76860013159288 ], [ 106.678502995869849, 10.768559603204169 ], [ 106.678502995869849, 10.769240249125588 ], [ 106.677766008634421, 10.769280779954151 ], [ 106.677766008634421, 10.76860013159288 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f202c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.768559603204169 ], [ 106.679239985585866, 10.768519072928106 ], [ 106.679239985585866, 10.769199716409554 ], [ 106.678502995869849, 10.769240249125588 ], [ 106.678502995869849, 10.768559603204169 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2034", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.769240249125588 ], [ 106.679239985585866, 10.769199716409554 ], [ 106.679239985585866, 10.76988036503699 ], [ 106.678502995869849, 10.769920900192984 ], [ 106.678502995869849, 10.769240249125588 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f203c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.769280779954151 ], [ 106.678502995869849, 10.769240249125588 ], [ 106.678502995869849, 10.769920900192984 ], [ 106.677766008634421, 10.769961433461399 ], [ 106.677766008634421, 10.769280779954151 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2044", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.769961433461399 ], [ 106.678502995869849, 10.769920900192984 ], [ 106.678502995869849, 10.77060155640608 ], [ 106.677766008634421, 10.77064209211434 ], [ 106.677766008634421, 10.769961433461399 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f204c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.769920900192984 ], [ 106.679239985585866, 10.76988036503699 ], [ 106.679239985585866, 10.770561018810126 ], [ 106.678502995869849, 10.77060155640608 ], [ 106.678502995869849, 10.769920900192984 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2054", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.77060155640608 ], [ 106.679239985585866, 10.770561018810126 ], [ 106.679239985585866, 10.77124167772868 ], [ 106.678502995869849, 10.77128221776459 ], [ 106.678502995869849, 10.77060155640608 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f205c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.77064209211434 ], [ 106.678502995869849, 10.77060155640608 ], [ 106.678502995869849, 10.77128221776459 ], [ 106.677766008634421, 10.771322755912689 ], [ 106.677766008634421, 10.77064209211434 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2064", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.770682625934894 ], [ 106.677766008634421, 10.77064209211434 ], [ 106.677766008634421, 10.771322755912689 ], [ 106.677029023879811, 10.771363292172968 ], [ 106.677029023879811, 10.770682625934894 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f206c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.770001964842223 ], [ 106.677766008634421, 10.769961433461399 ], [ 106.677766008634421, 10.77064209211434 ], [ 106.677029023879811, 10.770682625934894 ], [ 106.677029023879811, 10.770001964842223 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2074", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.770042494335451 ], [ 106.677029023879811, 10.770001964842223 ], [ 106.677029023879811, 10.770682625934894 ], [ 106.67629204160653, 10.770723157867735 ], [ 106.67629204160653, 10.770042494335451 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f207c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.770723157867735 ], [ 106.677029023879811, 10.770682625934894 ], [ 106.677029023879811, 10.771363292172968 ], [ 106.67629204160653, 10.77140382654542 ], [ 106.67629204160653, 10.770723157867735 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f208c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.771363292172968 ], [ 106.677766008634421, 10.771322755912689 ], [ 106.677766008634421, 10.772003424856168 ], [ 106.677029023879811, 10.772043963556166 ], [ 106.677029023879811, 10.771363292172968 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f20dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.772643555916725 ], [ 106.679239985585866, 10.772603011000918 ], [ 106.679239985585866, 10.773283685354031 ], [ 106.678502995869849, 10.773324232709779 ], [ 106.678502995869849, 10.772643555916725 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f20e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.771962884268232 ], [ 106.679239985585866, 10.771922341792374 ], [ 106.679239985585866, 10.772603011000918 ], [ 106.678502995869849, 10.772643555916725 ], [ 106.678502995869849, 10.771962884268232 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f20ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.772003424856168 ], [ 106.678502995869849, 10.771962884268232 ], [ 106.678502995869849, 10.772643555916725 ], [ 106.677766008634421, 10.77268409894449 ], [ 106.677766008634421, 10.772003424856168 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f20f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.771322755912689 ], [ 106.678502995869849, 10.77128221776459 ], [ 106.678502995869849, 10.771962884268232 ], [ 106.677766008634421, 10.772003424856168 ], [ 106.677766008634421, 10.771322755912689 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f20fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.77128221776459 ], [ 106.679239985585866, 10.77124167772868 ], [ 106.679239985585866, 10.771922341792374 ], [ 106.678502995869849, 10.771962884268232 ], [ 106.678502995869849, 10.77128221776459 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2104", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.77124167772868 ], [ 106.679976977782019, 10.771201135804967 ], [ 106.679976977782019, 10.771881797428593 ], [ 106.679239985585866, 10.771922341792374 ], [ 106.679239985585866, 10.77124167772868 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f210c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.771201135804967 ], [ 106.680713972457966, 10.771160591993457 ], [ 106.680713972457966, 10.771841251176902 ], [ 106.679976977782019, 10.771881797428593 ], [ 106.679976977782019, 10.771201135804967 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2114", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.771881797428593 ], [ 106.680713972457966, 10.771841251176902 ], [ 106.680713972457966, 10.772521915505211 ], [ 106.679976977782019, 10.772562464197078 ], [ 106.679976977782019, 10.771881797428593 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f211c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.771922341792374 ], [ 106.679976977782019, 10.771881797428593 ], [ 106.679976977782019, 10.772562464197078 ], [ 106.679239985585866, 10.772603011000918 ], [ 106.679239985585866, 10.771922341792374 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2124", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.772603011000918 ], [ 106.679976977782019, 10.772562464197078 ], [ 106.679976977782019, 10.773243136110136 ], [ 106.679239985585866, 10.773283685354031 ], [ 106.679239985585866, 10.772603011000918 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f212c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.773283685354031 ], [ 106.679976977782019, 10.773243136110136 ], [ 106.679976977782019, 10.773923813167487 ], [ 106.679239985585866, 10.773964364851436 ], [ 106.679239985585866, 10.773283685354031 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2134", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.773243136110136 ], [ 106.680713972457966, 10.773202584978097 ], [ 106.680713972457966, 10.77388325959528 ], [ 106.679976977782019, 10.773923813167487 ], [ 106.679976977782019, 10.773243136110136 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f213c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.772562464197078 ], [ 106.680713972457966, 10.772521915505211 ], [ 106.680713972457966, 10.773202584978097 ], [ 106.679976977782019, 10.773243136110136 ], [ 106.679976977782019, 10.772562464197078 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2144", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.772521915505211 ], [ 106.681450969613365, 10.772481364925326 ], [ 106.681450969613365, 10.773162031957924 ], [ 106.680713972457966, 10.773202584978097 ], [ 106.680713972457966, 10.772521915505211 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f214c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.773202584978097 ], [ 106.681450969613365, 10.773162031957924 ], [ 106.681450969613365, 10.773842704134827 ], [ 106.680713972457966, 10.77388325959528 ], [ 106.680713972457966, 10.773202584978097 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2154", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.773162031957924 ], [ 106.682187969247821, 10.773121477049628 ], [ 106.682187969247821, 10.773802146786133 ], [ 106.681450969613365, 10.773842704134827 ], [ 106.681450969613365, 10.773162031957924 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f215c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.772481364925326 ], [ 106.682187969247821, 10.772440812457431 ], [ 106.682187969247821, 10.773121477049628 ], [ 106.681450969613365, 10.773162031957924 ], [ 106.681450969613365, 10.772481364925326 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2164", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.771800703037307 ], [ 106.682187969247821, 10.77176015300982 ], [ 106.682187969247821, 10.772440812457431 ], [ 106.681450969613365, 10.772481364925326 ], [ 106.681450969613365, 10.771800703037307 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f216c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.771841251176902 ], [ 106.681450969613365, 10.771800703037307 ], [ 106.681450969613365, 10.772481364925326 ], [ 106.680713972457966, 10.772521915505211 ], [ 106.680713972457966, 10.771841251176902 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2174", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.771160591993457 ], [ 106.681450969613365, 10.771120046294159 ], [ 106.681450969613365, 10.771800703037307 ], [ 106.680713972457966, 10.771841251176902 ], [ 106.680713972457966, 10.771160591993457 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f217c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.771120046294159 ], [ 106.682187969247821, 10.771079498707081 ], [ 106.682187969247821, 10.77176015300982 ], [ 106.681450969613365, 10.771800703037307 ], [ 106.681450969613365, 10.771120046294159 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2184", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.77043939469616 ], [ 106.682187969247821, 10.770398849549496 ], [ 106.682187969247821, 10.771079498707081 ], [ 106.681450969613365, 10.771120046294159 ], [ 106.681450969613365, 10.77043939469616 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f218c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.769758748243591 ], [ 106.682187969247821, 10.769718205537346 ], [ 106.682187969247821, 10.770398849549496 ], [ 106.681450969613365, 10.77043939469616 ], [ 106.681450969613365, 10.769758748243591 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2194", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.769799289062284 ], [ 106.681450969613365, 10.769758748243591 ], [ 106.681450969613365, 10.77043939469616 ], [ 106.680713972457966, 10.770479937955157 ], [ 106.680713972457966, 10.769799289062284 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f219c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.770479937955157 ], [ 106.681450969613365, 10.77043939469616 ], [ 106.681450969613365, 10.771120046294159 ], [ 106.680713972457966, 10.771160591993457 ], [ 106.680713972457966, 10.770479937955157 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.770520479326482 ], [ 106.680713972457966, 10.770479937955157 ], [ 106.680713972457966, 10.771160591993457 ], [ 106.679976977782019, 10.771201135804967 ], [ 106.679976977782019, 10.770520479326482 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.770561018810126 ], [ 106.679976977782019, 10.770520479326482 ], [ 106.679976977782019, 10.771201135804967 ], [ 106.679239985585866, 10.77124167772868 ], [ 106.679239985585866, 10.770561018810126 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.76988036503699 ], [ 106.679976977782019, 10.769839827993421 ], [ 106.679976977782019, 10.770520479326482 ], [ 106.679239985585866, 10.770561018810126 ], [ 106.679239985585866, 10.76988036503699 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.769839827993421 ], [ 106.680713972457966, 10.769799289062284 ], [ 106.680713972457966, 10.770479937955157 ], [ 106.679976977782019, 10.770520479326482 ], [ 106.679976977782019, 10.769839827993421 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.769159181806065 ], [ 106.680713972457966, 10.769118645315125 ], [ 106.680713972457966, 10.769799289062284 ], [ 106.679976977782019, 10.769839827993421 ], [ 106.679976977782019, 10.769159181806065 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.769199716409554 ], [ 106.679976977782019, 10.769159181806065 ], [ 106.679976977782019, 10.769839827993421 ], [ 106.679239985585866, 10.76988036503699 ], [ 106.679239985585866, 10.769199716409554 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.768519072928106 ], [ 106.679976977782019, 10.768478540764699 ], [ 106.679976977782019, 10.769159181806065 ], [ 106.679239985585866, 10.769199716409554 ], [ 106.679239985585866, 10.768519072928106 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.768478540764699 ], [ 106.680713972457966, 10.768438006713955 ], [ 106.680713972457966, 10.769118645315125 ], [ 106.679976977782019, 10.769159181806065 ], [ 106.679976977782019, 10.768478540764699 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.768438006713955 ], [ 106.681450969613365, 10.768397470775884 ], [ 106.681450969613365, 10.769078106936739 ], [ 106.680713972457966, 10.769118645315125 ], [ 106.680713972457966, 10.768438006713955 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.769118645315125 ], [ 106.681450969613365, 10.769078106936739 ], [ 106.681450969613365, 10.769758748243591 ], [ 106.680713972457966, 10.769799289062284 ], [ 106.680713972457966, 10.769118645315125 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.769078106936739 ], [ 106.682187969247821, 10.769037566670921 ], [ 106.682187969247821, 10.769718205537346 ], [ 106.681450969613365, 10.769758748243591 ], [ 106.681450969613365, 10.769078106936739 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.768397470775884 ], [ 106.682187969247821, 10.768356932950493 ], [ 106.682187969247821, 10.769037566670921 ], [ 106.681450969613365, 10.769078106936739 ], [ 106.681450969613365, 10.768397470775884 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2204", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.768356932950493 ], [ 106.682924971360933, 10.768316393237788 ], [ 106.682924971360933, 10.768997024517672 ], [ 106.682187969247821, 10.769037566670921 ], [ 106.682187969247821, 10.768356932950493 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f220c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.768316393237788 ], [ 106.683661975952418, 10.76827585163778 ], [ 106.683661975952418, 10.768956480477005 ], [ 106.682924971360933, 10.768997024517672 ], [ 106.682924971360933, 10.768316393237788 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2214", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.768997024517672 ], [ 106.683661975952418, 10.768956480477005 ], [ 106.683661975952418, 10.769637114462242 ], [ 106.682924971360933, 10.769677660943561 ], [ 106.682924971360933, 10.768997024517672 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f221c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.769037566670921 ], [ 106.682924971360933, 10.768997024517672 ], [ 106.682924971360933, 10.769677660943561 ], [ 106.682187969247821, 10.769718205537346 ], [ 106.682187969247821, 10.769037566670921 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2224", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.769718205537346 ], [ 106.682924971360933, 10.769677660943561 ], [ 106.682924971360933, 10.770358302515175 ], [ 106.682187969247821, 10.770398849549496 ], [ 106.682187969247821, 10.769718205537346 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f222c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.770398849549496 ], [ 106.682924971360933, 10.770358302515175 ], [ 106.682924971360933, 10.77103894923223 ], [ 106.682187969247821, 10.771079498707081 ], [ 106.682187969247821, 10.770398849549496 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2234", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.770358302515175 ], [ 106.683661975952418, 10.770317753593204 ], [ 106.683661975952418, 10.770998397869615 ], [ 106.682924971360933, 10.77103894923223 ], [ 106.682924971360933, 10.770358302515175 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f223c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.769677660943561 ], [ 106.683661975952418, 10.769637114462242 ], [ 106.683661975952418, 10.770317753593204 ], [ 106.682924971360933, 10.770358302515175 ], [ 106.682924971360933, 10.769677660943561 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2244", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.769637114462242 ], [ 106.684398983021822, 10.769596566093393 ], [ 106.684398983021822, 10.770277202783593 ], [ 106.683661975952418, 10.770317753593204 ], [ 106.683661975952418, 10.769637114462242 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f224c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.770317753593204 ], [ 106.684398983021822, 10.770277202783593 ], [ 106.684398983021822, 10.770957844619243 ], [ 106.683661975952418, 10.770998397869615 ], [ 106.683661975952418, 10.770317753593204 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2254", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.770277202783593 ], [ 106.68513599256886, 10.770236650086348 ], [ 106.68513599256886, 10.770917289481122 ], [ 106.684398983021822, 10.770957844619243 ], [ 106.684398983021822, 10.770277202783593 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f225c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.769596566093393 ], [ 106.68513599256886, 10.769556015837027 ], [ 106.68513599256886, 10.770236650086348 ], [ 106.684398983021822, 10.770277202783593 ], [ 106.684398983021822, 10.769596566093393 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2264", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.768915934548927 ], [ 106.68513599256886, 10.768875386733443 ], [ 106.68513599256886, 10.769556015837027 ], [ 106.684398983021822, 10.769596566093393 ], [ 106.684398983021822, 10.768915934548927 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f226c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.768956480477005 ], [ 106.684398983021822, 10.768915934548927 ], [ 106.684398983021822, 10.769596566093393 ], [ 106.683661975952418, 10.769637114462242 ], [ 106.683661975952418, 10.768956480477005 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2274", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.76827585163778 ], [ 106.684398983021822, 10.768235308150475 ], [ 106.684398983021822, 10.768915934548927 ], [ 106.683661975952418, 10.768956480477005 ], [ 106.683661975952418, 10.76827585163778 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f227c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.768235308150475 ], [ 106.68513599256886, 10.768194762775879 ], [ 106.68513599256886, 10.768875386733443 ], [ 106.684398983021822, 10.768915934548927 ], [ 106.684398983021822, 10.768235308150475 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f228c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.768875386733443 ], [ 106.685873004593077, 10.768834837030564 ], [ 106.685873004593077, 10.769515463693152 ], [ 106.68513599256886, 10.769556015837027 ], [ 106.68513599256886, 10.768875386733443 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f22dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.770155539028986 ], [ 106.687347036071799, 10.770114980668888 ], [ 106.687347036071799, 10.770795612740349 ], [ 106.686610019094189, 10.770836173541667 ], [ 106.686610019094246, 10.770155539028986 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f22e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.770196095501477 ], [ 106.686610019094246, 10.770155539028986 ], [ 106.686610019094189, 10.770836173541667 ], [ 106.685873004593077, 10.770876732455262 ], [ 106.685873004593077, 10.770196095501477 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f22ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.769515463693152 ], [ 106.686610019094246, 10.769474909661769 ], [ 106.686610019094246, 10.770155539028986 ], [ 106.685873004593077, 10.770196095501477 ], [ 106.685873004593077, 10.769515463693152 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f22f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.769556015837027 ], [ 106.685873004593077, 10.769515463693152 ], [ 106.685873004593077, 10.770196095501477 ], [ 106.68513599256886, 10.770236650086348 ], [ 106.68513599256886, 10.769556015837027 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f22fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.770236650086348 ], [ 106.685873004593077, 10.770196095501477 ], [ 106.685873004593077, 10.770876732455262 ], [ 106.68513599256886, 10.770917289481122 ], [ 106.68513599256886, 10.770236650086348 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2304", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.770917289481122 ], [ 106.685873004593077, 10.770876732455262 ], [ 106.685873004593077, 10.771557374554224 ], [ 106.68513599256886, 10.771597934021068 ], [ 106.68513599256886, 10.770917289481122 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f230c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.771597934021068 ], [ 106.685873004593077, 10.771557374554224 ], [ 106.685873004593077, 10.772238021798078 ], [ 106.68513599256886, 10.772278583705907 ], [ 106.68513599256886, 10.771597934021068 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2314", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.771557374554224 ], [ 106.686610019094189, 10.771516813199527 ], [ 106.686610019094189, 10.772197458002289 ], [ 106.685873004593077, 10.772238021798078 ], [ 106.685873004593077, 10.771557374554224 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f231c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.770876732455262 ], [ 106.686610019094189, 10.770836173541667 ], [ 106.686610019094189, 10.771516813199527 ], [ 106.685873004593077, 10.771557374554224 ], [ 106.685873004593077, 10.770876732455262 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2324", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.770836173541667 ], [ 106.687347036071799, 10.770795612740349 ], [ 106.687347036071799, 10.771476249956995 ], [ 106.686610019094189, 10.771516813199527 ], [ 106.686610019094189, 10.770836173541667 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f232c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.770795612740349 ], [ 106.688084055525565, 10.77075505005131 ], [ 106.688084055525565, 10.771435684826626 ], [ 106.687347036071799, 10.771476249956995 ], [ 106.687347036071799, 10.770795612740349 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2334", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.771476249956995 ], [ 106.688084055525565, 10.771435684826626 ], [ 106.688084055525565, 10.772116324746849 ], [ 106.687347036071799, 10.772156892318543 ], [ 106.687347036071799, 10.771476249956995 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f233c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.771516813199527 ], [ 106.687347036071799, 10.771476249956995 ], [ 106.687347036071799, 10.772156892318543 ], [ 106.686610019094189, 10.772197458002289 ], [ 106.686610019094189, 10.771516813199527 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2344", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.772197458002289 ], [ 106.687347036071799, 10.772156892318543 ], [ 106.687347036071799, 10.772837539824708 ], [ 106.686610019094189, 10.772878107949664 ], [ 106.686610019094189, 10.772197458002289 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f234c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.772156892318543 ], [ 106.688084055525565, 10.772116324746849 ], [ 106.688084055525565, 10.772796969811695 ], [ 106.687347036071799, 10.772837539824708 ], [ 106.687347036071799, 10.772156892318543 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2354", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.772837539824708 ], [ 106.688084055525565, 10.772796969811695 ], [ 106.688084055525565, 10.773477620020882 ], [ 106.687347036071799, 10.773518192475214 ], [ 106.687347036071799, 10.772837539824708 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f235c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.772878107949664 ], [ 106.687347036071799, 10.772837539824708 ], [ 106.687347036071799, 10.773518192475214 ], [ 106.686610019094246, 10.773558763041368 ], [ 106.686610019094189, 10.772878107949664 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2364", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.772918674186544 ], [ 106.686610019094189, 10.772878107949664 ], [ 106.686610019094246, 10.773558763041368 ], [ 106.685873004593077, 10.773599331719337 ], [ 106.685873004593077, 10.772918674186544 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f236c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.772238021798078 ], [ 106.686610019094189, 10.772197458002289 ], [ 106.686610019094189, 10.772878107949664 ], [ 106.685873004593077, 10.772918674186544 ], [ 106.685873004593077, 10.772238021798078 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2374", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.772278583705907 ], [ 106.685873004593077, 10.772238021798078 ], [ 106.685873004593077, 10.772918674186544 ], [ 106.68513599256886, 10.772959238535348 ], [ 106.68513599256886, 10.772278583705907 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f237c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.772959238535348 ], [ 106.685873004593077, 10.772918674186544 ], [ 106.685873004593077, 10.773599331719337 ], [ 106.68513599256886, 10.773639898509115 ], [ 106.68513599256886, 10.772959238535348 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2384", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.772999800996066 ], [ 106.68513599256886, 10.772959238535348 ], [ 106.68513599256886, 10.773639898509115 ], [ 106.684398983021822, 10.77368046341069 ], [ 106.684398983021822, 10.772999800996066 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f238c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.773040361568693 ], [ 106.684398983021822, 10.772999800996066 ], [ 106.684398983021822, 10.77368046341069 ], [ 106.683661975952418, 10.773721026424059 ], [ 106.683661975952418, 10.773040361568693 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2394", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.772359701857642 ], [ 106.684398983021822, 10.772319143725763 ], [ 106.684398983021822, 10.772999800996066 ], [ 106.683661975952418, 10.773040361568693 ], [ 106.683661975952418, 10.772359701857642 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f239c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.772319143725763 ], [ 106.68513599256886, 10.772278583705907 ], [ 106.68513599256886, 10.772959238535348 ], [ 106.684398983021822, 10.772999800996066 ], [ 106.684398983021822, 10.772319143725763 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.771638491600063 ], [ 106.68513599256886, 10.771597934021068 ], [ 106.68513599256886, 10.772278583705907 ], [ 106.684398983021822, 10.772319143725763 ], [ 106.684398983021822, 10.771638491600063 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.770957844619243 ], [ 106.68513599256886, 10.770917289481122 ], [ 106.68513599256886, 10.771597934021068 ], [ 106.684398983021822, 10.771638491600063 ], [ 106.684398983021822, 10.770957844619243 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.770998397869615 ], [ 106.684398983021822, 10.770957844619243 ], [ 106.684398983021822, 10.771638491600063 ], [ 106.683661975952418, 10.77167904729119 ], [ 106.683661975952418, 10.770998397869615 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.77167904729119 ], [ 106.684398983021822, 10.771638491600063 ], [ 106.684398983021822, 10.772319143725763 ], [ 106.683661975952418, 10.772359701857642 ], [ 106.683661975952418, 10.77167904729119 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.771719601094444 ], [ 106.683661975952418, 10.77167904729119 ], [ 106.683661975952418, 10.772359701857642 ], [ 106.682924971360933, 10.772400258101534 ], [ 106.682924971360933, 10.771719601094444 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.77103894923223 ], [ 106.683661975952418, 10.770998397869615 ], [ 106.683661975952418, 10.77167904729119 ], [ 106.682924971360933, 10.771719601094444 ], [ 106.682924971360933, 10.77103894923223 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.771079498707081 ], [ 106.682924971360933, 10.77103894923223 ], [ 106.682924971360933, 10.771719601094444 ], [ 106.682187969247821, 10.77176015300982 ], [ 106.682187969247821, 10.771079498707081 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.77176015300982 ], [ 106.682924971360933, 10.771719601094444 ], [ 106.682924971360933, 10.772400258101534 ], [ 106.682187969247821, 10.772440812457431 ], [ 106.682187969247821, 10.77176015300982 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.772440812457431 ], [ 106.682924971360933, 10.772400258101534 ], [ 106.682924971360933, 10.773080920253214 ], [ 106.682187969247821, 10.773121477049628 ], [ 106.682187969247821, 10.772440812457431 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.772400258101534 ], [ 106.683661975952418, 10.772359701857642 ], [ 106.683661975952418, 10.773040361568693 ], [ 106.682924971360933, 10.773080920253214 ], [ 106.682924971360933, 10.772400258101534 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.773080920253214 ], [ 106.683661975952418, 10.773040361568693 ], [ 106.683661975952418, 10.773721026424059 ], [ 106.682924971360933, 10.77376158754921 ], [ 106.682924971360933, 10.773080920253214 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.773121477049628 ], [ 106.682924971360933, 10.773080920253214 ], [ 106.682924971360933, 10.77376158754921 ], [ 106.682187969247821, 10.773802146786133 ], [ 106.682187969247821, 10.773121477049628 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2404", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.773802146786133 ], [ 106.682924971360933, 10.77376158754921 ], [ 106.682924971360933, 10.774442259989229 ], [ 106.682187969247821, 10.774482821666663 ], [ 106.682187969247821, 10.773802146786133 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f240c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.77376158754921 ], [ 106.683661975952418, 10.773721026424059 ], [ 106.683661975952418, 10.774401696423455 ], [ 106.682924971360933, 10.774442259989229 ], [ 106.682924971360933, 10.77376158754921 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2414", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.774442259989229 ], [ 106.683661975952418, 10.774401696423455 ], [ 106.683661975952418, 10.775082371566599 ], [ 106.682924971360933, 10.775122937572993 ], [ 106.682924971360933, 10.774442259989229 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f241c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.774482821666663 ], [ 106.682924971360933, 10.774442259989229 ], [ 106.682924971360933, 10.775122937572993 ], [ 106.682187969247821, 10.775163501690932 ], [ 106.682187969247821, 10.774482821666663 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2424", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.775163501690932 ], [ 106.682924971360933, 10.775122937572993 ], [ 106.682924971360933, 10.77580362030022 ], [ 106.682187969247821, 10.775844186858654 ], [ 106.682187969247821, 10.775163501690932 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f242c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.775844186858654 ], [ 106.682924971360933, 10.77580362030022 ], [ 106.682924971360933, 10.776484308170627 ], [ 106.682187969247821, 10.776524877169557 ], [ 106.682187969247821, 10.775844186858654 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2434", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.77580362030022 ], [ 106.683661975952418, 10.775763051853215 ], [ 106.683661975952418, 10.77644373728301 ], [ 106.682924971360933, 10.776484308170627 ], [ 106.682924971360933, 10.77580362030022 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f243c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.775122937572993 ], [ 106.683661975952418, 10.775082371566599 ], [ 106.683661975952418, 10.775763051853215 ], [ 106.682924971360933, 10.77580362030022 ], [ 106.682924971360933, 10.775122937572993 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2444", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.775082371566599 ], [ 106.684398983021822, 10.775041803671764 ], [ 106.684398983021822, 10.775722481517647 ], [ 106.683661975952418, 10.775763051853215 ], [ 106.683661975952418, 10.775082371566599 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f244c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.775763051853215 ], [ 106.684398983021822, 10.775722481517647 ], [ 106.684398983021822, 10.77640316450672 ], [ 106.683661975952418, 10.77644373728301 ], [ 106.683661975952418, 10.775763051853215 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2454", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.775722481517647 ], [ 106.68513599256886, 10.77568190929353 ], [ 106.68513599256886, 10.77636258984176 ], [ 106.684398983021822, 10.77640316450672 ], [ 106.684398983021822, 10.775722481517647 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f245c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.775041803671764 ], [ 106.68513599256886, 10.775001233888487 ], [ 106.68513599256886, 10.77568190929353 ], [ 106.684398983021822, 10.775722481517647 ], [ 106.684398983021822, 10.775041803671764 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2464", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.774361130969352 ], [ 106.68513599256886, 10.774320563626922 ], [ 106.68513599256886, 10.775001233888487 ], [ 106.684398983021822, 10.775041803671764 ], [ 106.684398983021822, 10.774361130969352 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f246c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.774401696423455 ], [ 106.684398983021822, 10.774361130969352 ], [ 106.684398983021822, 10.775041803671764 ], [ 106.683661975952418, 10.775082371566599 ], [ 106.683661975952418, 10.774401696423455 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2474", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.773721026424059 ], [ 106.684398983021822, 10.77368046341069 ], [ 106.684398983021822, 10.774361130969352 ], [ 106.683661975952418, 10.774401696423455 ], [ 106.683661975952418, 10.773721026424059 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f247c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.77368046341069 ], [ 106.68513599256886, 10.773639898509115 ], [ 106.68513599256886, 10.774320563626922 ], [ 106.684398983021822, 10.774361130969352 ], [ 106.684398983021822, 10.77368046341069 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2484", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.773639898509115 ], [ 106.685873004593077, 10.773599331719337 ], [ 106.685873004593077, 10.774279994396178 ], [ 106.68513599256886, 10.774320563626922 ], [ 106.68513599256886, 10.773639898509115 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f248c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.774320563626922 ], [ 106.685873004593077, 10.774279994396178 ], [ 106.685873004593077, 10.774960662216779 ], [ 106.68513599256886, 10.775001233888487 ], [ 106.68513599256886, 10.774320563626922 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2494", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.774279994396178 ], [ 106.686610019094189, 10.774239423277127 ], [ 106.686610019094246, 10.774920088656648 ], [ 106.685873004593077, 10.774960662216779 ], [ 106.685873004593077, 10.774279994396178 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f249c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.773599331719337 ], [ 106.686610019094246, 10.773558763041368 ], [ 106.686610019094189, 10.774239423277127 ], [ 106.685873004593077, 10.774279994396178 ], [ 106.685873004593077, 10.773599331719337 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.773558763041368 ], [ 106.687347036071799, 10.773518192475214 ], [ 106.687347036071799, 10.774198850269775 ], [ 106.686610019094189, 10.774239423277127 ], [ 106.686610019094246, 10.773558763041368 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.773518192475214 ], [ 106.688084055525565, 10.773477620020882 ], [ 106.688084055525565, 10.774158275374129 ], [ 106.687347036071799, 10.774198850269775 ], [ 106.687347036071799, 10.773518192475214 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.774198850269775 ], [ 106.688084055525565, 10.774158275374129 ], [ 106.688084055525565, 10.774838935871152 ], [ 106.687347036071799, 10.774879513208102 ], [ 106.687347036071799, 10.774198850269775 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.774239423277127 ], [ 106.687347036071799, 10.774198850269775 ], [ 106.687347036071799, 10.774879513208102 ], [ 106.686610019094246, 10.774920088656648 ], [ 106.686610019094189, 10.774239423277127 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.774920088656648 ], [ 106.687347036071799, 10.774879513208102 ], [ 106.687347036071799, 10.775560181289924 ], [ 106.686610019094246, 10.775600759179657 ], [ 106.686610019094246, 10.774920088656648 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.774879513208102 ], [ 106.688084055525565, 10.774838935871152 ], [ 106.688084055525565, 10.775519601511666 ], [ 106.687347036071799, 10.775560181289924 ], [ 106.687347036071799, 10.774879513208102 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.775560181289924 ], [ 106.688084055525565, 10.775519601511666 ], [ 106.688084055525565, 10.776200272295393 ], [ 106.687347036071799, 10.776240854514947 ], [ 106.687347036071799, 10.775560181289924 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.775600759179657 ], [ 106.687347036071799, 10.775560181289924 ], [ 106.687347036071799, 10.776240854514947 ], [ 106.686610019094189, 10.776281434845867 ], [ 106.686610019094246, 10.775600759179657 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.775641335180861 ], [ 106.686610019094246, 10.775600759179657 ], [ 106.686610019094189, 10.776281434845867 ], [ 106.685873004593077, 10.776322013288139 ], [ 106.685873004593077, 10.775641335180861 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.774960662216779 ], [ 106.686610019094246, 10.774920088656648 ], [ 106.686610019094246, 10.775600759179657 ], [ 106.685873004593077, 10.775641335180861 ], [ 106.685873004593077, 10.774960662216779 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.775001233888487 ], [ 106.685873004593077, 10.774960662216779 ], [ 106.685873004593077, 10.775641335180861 ], [ 106.68513599256886, 10.77568190929353 ], [ 106.68513599256886, 10.775001233888487 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f24fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.77568190929353 ], [ 106.685873004593077, 10.775641335180861 ], [ 106.685873004593077, 10.776322013288139 ], [ 106.68513599256886, 10.77636258984176 ], [ 106.68513599256886, 10.77568190929353 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2504", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.77636258984176 ], [ 106.685873004593077, 10.776322013288139 ], [ 106.685873004593077, 10.777002696538332 ], [ 106.68513599256886, 10.777043275532902 ], [ 106.68513599256886, 10.77636258984176 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f250c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.777043275532902 ], [ 106.685873004593077, 10.777002696538332 ], [ 106.685873004593077, 10.777683384931155 ], [ 106.68513599256886, 10.777723966366672 ], [ 106.68513599256886, 10.777043275532902 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2514", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.777002696538332 ], [ 106.686610019094246, 10.776962115654992 ], [ 106.686610019094189, 10.777642801606758 ], [ 106.685873004593077, 10.777683384931155 ], [ 106.685873004593077, 10.777002696538332 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f251c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.776322013288139 ], [ 106.686610019094189, 10.776281434845867 ], [ 106.686610019094246, 10.776962115654992 ], [ 106.685873004593077, 10.777002696538332 ], [ 106.685873004593077, 10.776322013288139 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2524", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.776281434845867 ], [ 106.687347036071799, 10.776240854514947 ], [ 106.687347036071799, 10.776921532882897 ], [ 106.686610019094246, 10.776962115654992 ], [ 106.686610019094189, 10.776281434845867 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f252c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.776240854514947 ], [ 106.688084055525565, 10.776200272295393 ], [ 106.688084055525565, 10.776880948222047 ], [ 106.687347036071799, 10.776921532882897 ], [ 106.687347036071799, 10.776240854514947 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2534", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.776921532882897 ], [ 106.688084055525565, 10.776880948222047 ], [ 106.688084055525565, 10.777561629291343 ], [ 106.687347036071799, 10.777602216393484 ], [ 106.687347036071799, 10.776921532882897 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f253c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.776962115654992 ], [ 106.687347036071799, 10.776921532882897 ], [ 106.687347036071799, 10.777602216393484 ], [ 106.686610019094189, 10.777642801606758 ], [ 106.686610019094246, 10.776962115654992 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2544", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.777642801606758 ], [ 106.687347036071799, 10.777602216393484 ], [ 106.687347036071799, 10.778282905046431 ], [ 106.686610019094189, 10.778323492700874 ], [ 106.686610019094189, 10.777642801606758 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f254c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.777602216393484 ], [ 106.688084055525565, 10.777561629291343 ], [ 106.688084055525565, 10.778242315503004 ], [ 106.687347036071799, 10.778282905046431 ], [ 106.687347036071799, 10.777602216393484 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2554", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.778282905046431 ], [ 106.688084055525565, 10.778242315503004 ], [ 106.688084055525565, 10.778923006856742 ], [ 106.687347036071799, 10.778963598841449 ], [ 106.687347036071799, 10.778282905046431 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f255c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.778323492700874 ], [ 106.687347036071799, 10.778282905046431 ], [ 106.687347036071799, 10.778963598841449 ], [ 106.686610019094189, 10.77900418893706 ], [ 106.686610019094189, 10.778323492700874 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2564", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.778364078466328 ], [ 106.686610019094189, 10.778323492700874 ], [ 106.686610019094189, 10.77900418893706 ], [ 106.685873004593077, 10.779044777143568 ], [ 106.685873004593077, 10.778364078466328 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f256c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.777683384931155 ], [ 106.686610019094189, 10.777642801606758 ], [ 106.686610019094189, 10.778323492700874 ], [ 106.685873004593077, 10.778364078466328 ], [ 106.685873004593077, 10.777683384931155 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2574", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.777723966366672 ], [ 106.685873004593077, 10.777683384931155 ], [ 106.685873004593077, 10.778364078466328 ], [ 106.68513599256886, 10.778404662342785 ], [ 106.68513599256886, 10.777723966366672 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f257c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.778404662342785 ], [ 106.685873004593077, 10.778364078466328 ], [ 106.685873004593077, 10.779044777143568 ], [ 106.68513599256886, 10.779085363460961 ], [ 106.68513599256886, 10.778404662342785 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2584", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.778445244330237 ], [ 106.68513599256886, 10.778404662342785 ], [ 106.68513599256886, 10.779085363460961 ], [ 106.684398983021822, 10.779125947889233 ], [ 106.684398983021822, 10.778445244330237 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f258c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.778485824428675 ], [ 106.684398983021822, 10.778445244330237 ], [ 106.684398983021822, 10.779125947889233 ], [ 106.683661975952418, 10.779166530428377 ], [ 106.683661975952418, 10.778485824428675 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2594", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.777805123571023 ], [ 106.684398983021822, 10.777764545913298 ], [ 106.684398983021822, 10.778445244330237 ], [ 106.683661975952418, 10.778485824428675 ], [ 106.683661975952418, 10.777805123571023 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f259c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.777764545913298 ], [ 106.68513599256886, 10.777723966366672 ], [ 106.68513599256886, 10.778404662342785 ], [ 106.684398983021822, 10.778445244330237 ], [ 106.684398983021822, 10.777764545913298 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.777083852638697 ], [ 106.68513599256886, 10.777043275532902 ], [ 106.68513599256886, 10.777723966366672 ], [ 106.684398983021822, 10.777764545913298 ], [ 106.684398983021822, 10.777083852638697 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.77640316450672 ], [ 106.68513599256886, 10.77636258984176 ], [ 106.68513599256886, 10.777043275532902 ], [ 106.684398983021822, 10.777083852638697 ], [ 106.684398983021822, 10.77640316450672 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.77644373728301 ], [ 106.684398983021822, 10.77640316450672 ], [ 106.684398983021822, 10.777083852638697 ], [ 106.683661975952418, 10.777124427855709 ], [ 106.683661975952418, 10.77644373728301 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.777124427855709 ], [ 106.684398983021822, 10.777083852638697 ], [ 106.684398983021822, 10.777764545913298 ], [ 106.683661975952418, 10.777805123571023 ], [ 106.683661975952418, 10.777124427855709 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.777165001183926 ], [ 106.683661975952418, 10.777124427855709 ], [ 106.683661975952418, 10.777805123571023 ], [ 106.682924971360933, 10.777845699339842 ], [ 106.682924971360933, 10.777165001183926 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.776484308170627 ], [ 106.683661975952418, 10.77644373728301 ], [ 106.683661975952418, 10.777124427855709 ], [ 106.682924971360933, 10.777165001183926 ], [ 106.682924971360933, 10.776484308170627 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.776524877169557 ], [ 106.682924971360933, 10.776484308170627 ], [ 106.682924971360933, 10.777165001183926 ], [ 106.682187969247821, 10.777205572623346 ], [ 106.682187969247821, 10.776524877169557 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.777205572623346 ], [ 106.682924971360933, 10.777165001183926 ], [ 106.682924971360933, 10.777845699339842 ], [ 106.682187969247821, 10.77788627321975 ], [ 106.682187969247821, 10.777205572623346 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.77788627321975 ], [ 106.682924971360933, 10.777845699339842 ], [ 106.682924971360933, 10.778526402638091 ], [ 106.682187969247821, 10.778566978958477 ], [ 106.682187969247821, 10.77788627321975 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.777845699339842 ], [ 106.683661975952418, 10.777805123571023 ], [ 106.683661975952418, 10.778485824428675 ], [ 106.682924971360933, 10.778526402638091 ], [ 106.682924971360933, 10.777845699339842 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.778526402638091 ], [ 106.683661975952418, 10.778485824428675 ], [ 106.683661975952418, 10.779166530428377 ], [ 106.682924971360933, 10.779207111078383 ], [ 106.682924971360933, 10.778526402638091 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.778566978958477 ], [ 106.682924971360933, 10.778526402638091 ], [ 106.682924971360933, 10.779207111078383 ], [ 106.682187969247821, 10.779247689839247 ], [ 106.682187969247821, 10.778566978958477 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2604", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.778607553389826 ], [ 106.682187969247821, 10.778566978958477 ], [ 106.682187969247821, 10.779247689839247 ], [ 106.681450969613365, 10.779288266710957 ], [ 106.681450969613365, 10.778607553389826 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f260c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.777926845210732 ], [ 106.682187969247821, 10.77788627321975 ], [ 106.682187969247821, 10.778566978958477 ], [ 106.681450969613365, 10.778607553389826 ], [ 106.681450969613365, 10.777926845210732 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2614", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.777967415312785 ], [ 106.681450969613365, 10.777926845210732 ], [ 106.681450969613365, 10.778607553389826 ], [ 106.680713972457966, 10.778648125932131 ], [ 106.680713972457966, 10.777967415312785 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f261c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.778648125932131 ], [ 106.681450969613365, 10.778607553389826 ], [ 106.681450969613365, 10.779288266710957 ], [ 106.680713972457966, 10.779328841693507 ], [ 106.680713972457966, 10.778648125932131 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2624", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.778688696585382 ], [ 106.680713972457966, 10.778648125932131 ], [ 106.680713972457966, 10.779328841693507 ], [ 106.679976977782019, 10.77936941478689 ], [ 106.679976977782019, 10.778688696585382 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f262c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.778729265349572 ], [ 106.679976977782019, 10.778688696585382 ], [ 106.679976977782019, 10.77936941478689 ], [ 106.679239985585866, 10.779409985991098 ], [ 106.679239985585866, 10.778729265349572 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f263c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.778007983525901 ], [ 106.680713972457966, 10.777967415312785 ], [ 106.680713972457966, 10.778648125932131 ], [ 106.679976977782019, 10.778688696585382 ], [ 106.679976977782019, 10.778007983525901 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f266c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.777286709835757 ], [ 106.681450969613365, 10.77724614217396 ], [ 106.681450969613365, 10.777926845210732 ], [ 106.680713972457966, 10.777967415312785 ], [ 106.680713972457966, 10.777286709835757 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2674", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.77724614217396 ], [ 106.682187969247821, 10.777205572623346 ], [ 106.682187969247821, 10.77788627321975 ], [ 106.681450969613365, 10.777926845210732 ], [ 106.681450969613365, 10.77724614217396 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f267c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.776565444279795 ], [ 106.682187969247821, 10.776524877169557 ], [ 106.682187969247821, 10.777205572623346 ], [ 106.681450969613365, 10.77724614217396 ], [ 106.681450969613365, 10.776565444279795 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f269c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.775204063920405 ], [ 106.682187969247821, 10.775163501690932 ], [ 106.682187969247821, 10.775844186858654 ], [ 106.681450969613365, 10.775884751528514 ], [ 106.681450969613365, 10.775204063920405 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f26a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.774523381455749 ], [ 106.682187969247821, 10.774482821666663 ], [ 106.682187969247821, 10.775163501690932 ], [ 106.681450969613365, 10.775204063920405 ], [ 106.681450969613365, 10.774523381455749 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f26ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.773842704134827 ], [ 106.682187969247821, 10.773802146786133 ], [ 106.682187969247821, 10.774482821666663 ], [ 106.681450969613365, 10.774523381455749 ], [ 106.681450969613365, 10.773842704134827 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f26b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.77388325959528 ], [ 106.681450969613365, 10.773842704134827 ], [ 106.681450969613365, 10.774523381455749 ], [ 106.680713972457966, 10.774563939356479 ], [ 106.680713972457966, 10.77388325959528 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f26bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.774563939356479 ], [ 106.681450969613365, 10.774523381455749 ], [ 106.681450969613365, 10.775204063920405 ], [ 106.680713972457966, 10.775244624261408 ], [ 106.680713972457966, 10.774563939356479 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f26cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.773923813167487 ], [ 106.680713972457966, 10.77388325959528 ], [ 106.680713972457966, 10.774563939356479 ], [ 106.679976977782019, 10.774604495368846 ], [ 106.679976977782019, 10.773923813167487 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f27d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.778769832224693 ], [ 106.679239985585866, 10.778729265349572 ], [ 106.679239985585866, 10.779409985991098 ], [ 106.678502995869849, 10.77945055530612 ], [ 106.678502995869849, 10.778769832224693 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f280c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.779531688268587 ], [ 106.677766008634421, 10.779491122731955 ], [ 106.677766008634421, 10.780171853394904 ], [ 106.677029023879811, 10.780212421371203 ], [ 106.677029023879811, 10.779531688268587 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2814", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.780212421371203 ], [ 106.677766008634421, 10.780171853394904 ], [ 106.677766008634421, 10.780852589199302 ], [ 106.677029023879811, 10.780893159615264 ], [ 106.677029023879811, 10.780212421371203 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f281c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.780252987458182 ], [ 106.677029023879811, 10.780212421371203 ], [ 106.677029023879811, 10.780893159615264 ], [ 106.67629204160653, 10.780933728141791 ], [ 106.67629204160653, 10.780252987458182 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2824", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.780933728141791 ], [ 106.677029023879811, 10.780893159615264 ], [ 106.677029023879811, 10.78157390300049 ], [ 106.67629204160653, 10.78161447396656 ], [ 106.67629204160653, 10.780933728141791 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f282c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.78161447396656 ], [ 106.677029023879811, 10.78157390300049 ], [ 106.677029023879811, 10.782254651526598 ], [ 106.67629204160653, 10.782295224932202 ], [ 106.67629204160653, 10.78161447396656 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2834", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.78157390300049 ], [ 106.677766008634421, 10.781533330144871 ], [ 106.677766008634421, 10.782214076231321 ], [ 106.677029023879811, 10.782254651526598 ], [ 106.677029023879811, 10.78157390300049 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f283c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.780893159615264 ], [ 106.677766008634421, 10.780852589199302 ], [ 106.677766008634421, 10.781533330144871 ], [ 106.677029023879811, 10.78157390300049 ], [ 106.677029023879811, 10.780893159615264 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2844", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.780852589199302 ], [ 106.678502995869849, 10.780812016893909 ], [ 106.678502995869849, 10.781492755399704 ], [ 106.677766008634421, 10.781533330144871 ], [ 106.677766008634421, 10.780852589199302 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f284c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.781533330144871 ], [ 106.678502995869849, 10.781492755399704 ], [ 106.678502995869849, 10.782173499046388 ], [ 106.677766008634421, 10.782214076231321 ], [ 106.677766008634421, 10.781533330144871 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2854", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.781492755399704 ], [ 106.679239985585866, 10.781452178765003 ], [ 106.679239985585866, 10.782132919971803 ], [ 106.678502995869849, 10.782173499046388 ], [ 106.678502995869849, 10.781492755399704 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f285c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.780812016893909 ], [ 106.679239985585866, 10.780771442699097 ], [ 106.679239985585866, 10.781452178765003 ], [ 106.678502995869849, 10.781492755399704 ], [ 106.678502995869849, 10.780812016893909 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2864", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.780131283529288 ], [ 106.679239985585866, 10.780090711774367 ], [ 106.679239985585866, 10.780771442699097 ], [ 106.678502995869849, 10.780812016893909 ], [ 106.678502995869849, 10.780131283529288 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f286c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.780171853394904 ], [ 106.678502995869849, 10.780131283529288 ], [ 106.678502995869849, 10.780812016893909 ], [ 106.677766008634421, 10.780852589199302 ], [ 106.677766008634421, 10.780171853394904 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2874", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.779491122731955 ], [ 106.678502995869849, 10.77945055530612 ], [ 106.678502995869849, 10.780131283529288 ], [ 106.677766008634421, 10.780171853394904 ], [ 106.677766008634421, 10.779491122731955 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f287c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.77945055530612 ], [ 106.679239985585866, 10.779409985991098 ], [ 106.679239985585866, 10.780090711774367 ], [ 106.678502995869849, 10.780131283529288 ], [ 106.678502995869849, 10.77945055530612 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2884", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.779409985991098 ], [ 106.679976977782019, 10.77936941478689 ], [ 106.679976977782019, 10.780050138130148 ], [ 106.679239985585866, 10.780090711774367 ], [ 106.679239985585866, 10.779409985991098 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f288c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.780090711774367 ], [ 106.679976977782019, 10.780050138130148 ], [ 106.679976977782019, 10.780730866614869 ], [ 106.679239985585866, 10.780771442699097 ], [ 106.679239985585866, 10.780090711774367 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2894", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.780050138130148 ], [ 106.680713972457966, 10.780009562596639 ], [ 106.680713972457966, 10.780690288641237 ], [ 106.679976977782019, 10.780730866614869 ], [ 106.679976977782019, 10.780050138130148 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f289c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.77936941478689 ], [ 106.680713972457966, 10.779328841693507 ], [ 106.680713972457966, 10.780009562596639 ], [ 106.679976977782019, 10.780050138130148 ], [ 106.679976977782019, 10.77936941478689 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.779328841693507 ], [ 106.681450969613365, 10.779288266710957 ], [ 106.681450969613365, 10.779968985173845 ], [ 106.680713972457966, 10.780009562596639 ], [ 106.680713972457966, 10.779328841693507 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.779288266710957 ], [ 106.682187969247821, 10.779247689839247 ], [ 106.682187969247821, 10.779928405861778 ], [ 106.681450969613365, 10.779968985173845 ], [ 106.681450969613365, 10.779288266710957 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.779968985173845 ], [ 106.682187969247821, 10.779928405861778 ], [ 106.682187969247821, 10.780609127025786 ], [ 106.681450969613365, 10.780649708778206 ], [ 106.681450969613365, 10.779968985173845 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.780009562596639 ], [ 106.681450969613365, 10.779968985173845 ], [ 106.681450969613365, 10.780649708778206 ], [ 106.680713972457966, 10.780690288641237 ], [ 106.680713972457966, 10.780009562596639 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.780690288641237 ], [ 106.681450969613365, 10.780649708778206 ], [ 106.681450969613365, 10.781330437523758 ], [ 106.680713972457966, 10.781371019827022 ], [ 106.680713972457966, 10.780690288641237 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.780649708778206 ], [ 106.682187969247821, 10.780609127025786 ], [ 106.682187969247821, 10.781289853330989 ], [ 106.681450969613365, 10.781330437523758 ], [ 106.681450969613365, 10.780649708778206 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.781330437523758 ], [ 106.682187969247821, 10.781289853330989 ], [ 106.682187969247821, 10.781970584777106 ], [ 106.681450969613365, 10.782011171410218 ], [ 106.681450969613365, 10.781330437523758 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.781371019827022 ], [ 106.681450969613365, 10.781330437523758 ], [ 106.681450969613365, 10.782011171410218 ], [ 106.680713972457966, 10.782051756153709 ], [ 106.680713972457966, 10.781371019827022 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.781411600240771 ], [ 106.680713972457966, 10.781371019827022 ], [ 106.680713972457966, 10.782051756153709 ], [ 106.679976977782019, 10.782092339007574 ], [ 106.679976977782019, 10.781411600240771 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.780730866614869 ], [ 106.680713972457966, 10.780690288641237 ], [ 106.680713972457966, 10.781371019827022 ], [ 106.679976977782019, 10.781411600240771 ], [ 106.679976977782019, 10.780730866614869 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.780771442699097 ], [ 106.679976977782019, 10.780730866614869 ], [ 106.679976977782019, 10.781411600240771 ], [ 106.679239985585866, 10.781452178765003 ], [ 106.679239985585866, 10.780771442699097 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.781452178765003 ], [ 106.679976977782019, 10.781411600240771 ], [ 106.679976977782019, 10.782092339007574 ], [ 106.679239985585866, 10.782132919971803 ], [ 106.679239985585866, 10.781452178765003 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2904", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.782132919971803 ], [ 106.679976977782019, 10.782092339007574 ], [ 106.679976977782019, 10.782773082914991 ], [ 106.679239985585866, 10.782813666319214 ], [ 106.679239985585866, 10.782132919971803 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f290c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.782813666319214 ], [ 106.679976977782019, 10.782773082914991 ], [ 106.679976977782019, 10.78345383196274 ], [ 106.679239985585866, 10.783494417806953 ], [ 106.679239985585866, 10.782813666319214 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2914", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.782773082914991 ], [ 106.680713972457966, 10.782732497621017 ], [ 106.680713972457966, 10.783413244228662 ], [ 106.679976977782019, 10.78345383196274 ], [ 106.679976977782019, 10.782773082914991 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f291c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.782092339007574 ], [ 106.680713972457966, 10.782051756153709 ], [ 106.680713972457966, 10.782732497621017 ], [ 106.679976977782019, 10.782773082914991 ], [ 106.679976977782019, 10.782092339007574 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2924", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.782051756153709 ], [ 106.681450969613365, 10.782011171410218 ], [ 106.681450969613365, 10.782691910437302 ], [ 106.680713972457966, 10.782732497621017 ], [ 106.680713972457966, 10.782051756153709 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f292c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.782011171410218 ], [ 106.682187969247821, 10.781970584777106 ], [ 106.682187969247821, 10.782651321363851 ], [ 106.681450969613365, 10.782691910437302 ], [ 106.681450969613365, 10.782011171410218 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2934", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.782691910437302 ], [ 106.682187969247821, 10.782651321363851 ], [ 106.682187969247821, 10.78333206309094 ], [ 106.681450969613365, 10.783372654604726 ], [ 106.681450969613365, 10.782691910437302 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f293c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.782732497621017 ], [ 106.681450969613365, 10.782691910437302 ], [ 106.681450969613365, 10.783372654604726 ], [ 106.680713972457966, 10.783413244228662 ], [ 106.680713972457966, 10.782732497621017 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2944", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.783413244228662 ], [ 106.681450969613365, 10.783372654604726 ], [ 106.681450969613365, 10.78405340391221 ], [ 106.680713972457966, 10.784093995976361 ], [ 106.680713972457966, 10.783413244228662 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f294c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.783372654604726 ], [ 106.682187969247821, 10.78333206309094 ], [ 106.682187969247821, 10.784012809958096 ], [ 106.681450969613365, 10.78405340391221 ], [ 106.681450969613365, 10.783372654604726 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2954", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.78405340391221 ], [ 106.682187969247821, 10.784012809958096 ], [ 106.682187969247821, 10.784693561965032 ], [ 106.681450969613365, 10.78473415835947 ], [ 106.681450969613365, 10.78405340391221 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f295c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.784093995976361 ], [ 106.681450969613365, 10.78405340391221 ], [ 106.681450969613365, 10.78473415835947 ], [ 106.680713972457966, 10.784774752863832 ], [ 106.680713972457966, 10.784093995976361 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2964", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.78413458615054 ], [ 106.680713972457966, 10.784093995976361 ], [ 106.680713972457966, 10.784774752863832 ], [ 106.679976977782019, 10.784815345478108 ], [ 106.679976977782019, 10.78413458615054 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f296c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.78345383196274 ], [ 106.680713972457966, 10.783413244228662 ], [ 106.680713972457966, 10.784093995976361 ], [ 106.679976977782019, 10.78413458615054 ], [ 106.679976977782019, 10.78345383196274 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2974", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.783494417806953 ], [ 106.679976977782019, 10.78345383196274 ], [ 106.679976977782019, 10.78413458615054 ], [ 106.679239985585866, 10.784175174434738 ], [ 106.679239985585866, 10.783494417806953 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f297c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.784175174434738 ], [ 106.679976977782019, 10.78413458615054 ], [ 106.679976977782019, 10.784815345478108 ], [ 106.679239985585866, 10.784855936202286 ], [ 106.679239985585866, 10.784175174434738 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2984", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.784215760828948 ], [ 106.679239985585866, 10.784175174434738 ], [ 106.679239985585866, 10.784855936202286 ], [ 106.678502995869849, 10.784896525036361 ], [ 106.678502995869849, 10.784215760828948 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f298c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.78425634533316 ], [ 106.678502995869849, 10.784215760828948 ], [ 106.678502995869849, 10.784896525036361 ], [ 106.677766008634421, 10.784937111980325 ], [ 106.677766008634421, 10.78425634533316 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2994", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.783575583825751 ], [ 106.678502995869849, 10.783535001761292 ], [ 106.678502995869849, 10.784215760828948 ], [ 106.677766008634421, 10.78425634533316 ], [ 106.677766008634421, 10.783575583825751 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f299c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.783535001761292 ], [ 106.679239985585866, 10.783494417806953 ], [ 106.679239985585866, 10.784175174434738 ], [ 106.678502995869849, 10.784215760828948 ], [ 106.678502995869849, 10.783535001761292 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.782854247833676 ], [ 106.679239985585866, 10.782813666319214 ], [ 106.679239985585866, 10.783494417806953 ], [ 106.678502995869849, 10.783535001761292 ], [ 106.678502995869849, 10.782854247833676 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.782173499046388 ], [ 106.679239985585866, 10.782132919971803 ], [ 106.679239985585866, 10.782813666319214 ], [ 106.678502995869849, 10.782854247833676 ], [ 106.678502995869849, 10.782173499046388 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.782214076231321 ], [ 106.678502995869849, 10.782173499046388 ], [ 106.678502995869849, 10.782854247833676 ], [ 106.677766008634421, 10.782894827458376 ], [ 106.677766008634421, 10.782214076231321 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.782894827458376 ], [ 106.678502995869849, 10.782854247833676 ], [ 106.678502995869849, 10.783535001761292 ], [ 106.677766008634421, 10.783575583825751 ], [ 106.677766008634421, 10.782894827458376 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.7829354051933 ], [ 106.677766008634421, 10.782894827458376 ], [ 106.677766008634421, 10.783575583825751 ], [ 106.677029023879811, 10.783616164000318 ], [ 106.677029023879811, 10.7829354051933 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.782254651526598 ], [ 106.677766008634421, 10.782214076231321 ], [ 106.677766008634421, 10.782894827458376 ], [ 106.677029023879811, 10.7829354051933 ], [ 106.677029023879811, 10.782254651526598 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.782295224932202 ], [ 106.677029023879811, 10.782254651526598 ], [ 106.677029023879811, 10.7829354051933 ], [ 106.67629204160653, 10.782975981038442 ], [ 106.67629204160653, 10.782295224932202 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.782975981038442 ], [ 106.677029023879811, 10.7829354051933 ], [ 106.677029023879811, 10.783616164000318 ], [ 106.67629204160653, 10.783656742284991 ], [ 106.67629204160653, 10.782975981038442 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.783656742284991 ], [ 106.677029023879811, 10.783616164000318 ], [ 106.677029023879811, 10.78429692794737 ], [ 106.67629204160653, 10.784337508671566 ], [ 106.67629204160653, 10.783656742284991 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.783616164000318 ], [ 106.677766008634421, 10.783575583825751 ], [ 106.677766008634421, 10.78425634533316 ], [ 106.677029023879811, 10.78429692794737 ], [ 106.677029023879811, 10.783616164000318 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.78429692794737 ], [ 106.677766008634421, 10.78425634533316 ], [ 106.677766008634421, 10.784937111980325 ], [ 106.677029023879811, 10.784977697034169 ], [ 106.677029023879811, 10.78429692794737 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.784337508671566 ], [ 106.677029023879811, 10.78429692794737 ], [ 106.677029023879811, 10.784977697034169 ], [ 106.67629204160653, 10.785018280197887 ], [ 106.67629204160653, 10.784337508671566 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a04", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.785018280197887 ], [ 106.677029023879811, 10.784977697034169 ], [ 106.677029023879811, 10.785658471260433 ], [ 106.67629204160653, 10.785699056863669 ], [ 106.67629204160653, 10.785018280197887 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a0c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.785699056863669 ], [ 106.677029023879811, 10.785658471260433 ], [ 106.677029023879811, 10.786339250625883 ], [ 106.67629204160653, 10.786379838668628 ], [ 106.67629204160653, 10.785699056863669 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a14", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.785658471260433 ], [ 106.677766008634421, 10.785617883766957 ], [ 106.677766008634421, 10.786298660692781 ], [ 106.677029023879811, 10.786339250625883 ], [ 106.677029023879811, 10.785658471260433 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a1c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.784977697034169 ], [ 106.677766008634421, 10.784937111980325 ], [ 106.677766008634421, 10.785617883766957 ], [ 106.677029023879811, 10.785658471260433 ], [ 106.677029023879811, 10.784977697034169 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a24", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.784937111980325 ], [ 106.678502995869849, 10.784896525036361 ], [ 106.678502995869849, 10.785577294383248 ], [ 106.677766008634421, 10.785617883766957 ], [ 106.677766008634421, 10.784937111980325 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a2c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.784896525036361 ], [ 106.679239985585866, 10.784855936202286 ], [ 106.679239985585866, 10.785536703109312 ], [ 106.678502995869849, 10.785577294383248 ], [ 106.678502995869849, 10.784896525036361 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a34", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.785577294383248 ], [ 106.679239985585866, 10.785536703109312 ], [ 106.679239985585866, 10.786217475155537 ], [ 106.678502995869849, 10.786258068869328 ], [ 106.678502995869849, 10.785577294383248 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a3c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.785617883766957 ], [ 106.678502995869849, 10.785577294383248 ], [ 106.678502995869849, 10.786258068869328 ], [ 106.677766008634421, 10.786298660692781 ], [ 106.677766008634421, 10.785617883766957 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a44", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.786298660692781 ], [ 106.678502995869849, 10.786258068869328 ], [ 106.678502995869849, 10.786938848494318 ], [ 106.677766008634421, 10.786979442757506 ], [ 106.677766008634421, 10.786298660692781 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.786258068869328 ], [ 106.679239985585866, 10.786217475155537 ], [ 106.679239985585866, 10.786898252340674 ], [ 106.678502995869849, 10.786938848494318 ], [ 106.678502995869849, 10.786258068869328 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.786938848494318 ], [ 106.679239985585866, 10.786898252340674 ], [ 106.679239985585866, 10.787579034664439 ], [ 106.678502995869849, 10.787619633257931 ], [ 106.678502995869849, 10.786938848494318 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.786979442757506 ], [ 106.678502995869849, 10.786938848494318 ], [ 106.678502995869849, 10.787619633257931 ], [ 106.677766008634421, 10.787660229960855 ], [ 106.677766008634421, 10.786979442757506 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a64", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.787020035130231 ], [ 106.677766008634421, 10.786979442757506 ], [ 106.677766008634421, 10.787660229960855 ], [ 106.677029023879811, 10.787700824773196 ], [ 106.677029023879811, 10.787020035130231 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a6c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.786339250625883 ], [ 106.677766008634421, 10.786298660692781 ], [ 106.677766008634421, 10.786979442757506 ], [ 106.677029023879811, 10.787020035130231 ], [ 106.677029023879811, 10.786339250625883 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a74", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.786379838668628 ], [ 106.677029023879811, 10.786339250625883 ], [ 106.677029023879811, 10.787020035130231 ], [ 106.67629204160653, 10.787060625612483 ], [ 106.67629204160653, 10.786379838668628 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a7c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.787060625612483 ], [ 106.677029023879811, 10.787020035130231 ], [ 106.677029023879811, 10.787700824773196 ], [ 106.67629204160653, 10.787741417694953 ], [ 106.67629204160653, 10.787060625612483 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a84", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.787741417694953 ], [ 106.677029023879811, 10.787700824773196 ], [ 106.677029023879811, 10.788381619554496 ], [ 106.67629204160653, 10.78842221491575 ], [ 106.67629204160653, 10.787741417694953 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.787700824773196 ], [ 106.677766008634421, 10.787660229960855 ], [ 106.677766008634421, 10.788341022302541 ], [ 106.677029023879811, 10.788381619554496 ], [ 106.677029023879811, 10.787700824773196 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a94", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.788381619554496 ], [ 106.677766008634421, 10.788341022302541 ], [ 106.677766008634421, 10.789021819782283 ], [ 106.677029023879811, 10.789062419473849 ], [ 106.677029023879811, 10.788381619554496 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2a9c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.78842221491575 ], [ 106.677029023879811, 10.788381619554496 ], [ 106.677029023879811, 10.789062419473849 ], [ 106.67629204160653, 10.789103017274595 ], [ 106.67629204160653, 10.78842221491575 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2aa4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.789103017274595 ], [ 106.677029023879811, 10.789062419473849 ], [ 106.677029023879811, 10.789743224530968 ], [ 106.67629204160653, 10.789783824771202 ], [ 106.67629204160653, 10.789103017274595 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2aac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.789783824771202 ], [ 106.677029023879811, 10.789743224530968 ], [ 106.677029023879811, 10.790424034725573 ], [ 106.67629204160653, 10.790464637405295 ], [ 106.67629204160653, 10.789783824771202 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ab4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.789743224530968 ], [ 106.677766008634421, 10.789702622399801 ], [ 106.677766008634421, 10.790383430154805 ], [ 106.677029023879811, 10.790424034725573 ], [ 106.677029023879811, 10.789743224530968 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2abc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.789062419473849 ], [ 106.677766008634421, 10.789021819782283 ], [ 106.677766008634421, 10.789702622399801 ], [ 106.677029023879811, 10.789743224530968 ], [ 106.677029023879811, 10.789062419473849 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ac4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.789021819782283 ], [ 106.678502995869849, 10.788981218199909 ], [ 106.678502995869849, 10.789662018377706 ], [ 106.677766008634421, 10.789702622399801 ], [ 106.677766008634421, 10.789021819782283 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2acc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.789702622399801 ], [ 106.678502995869849, 10.789662018377706 ], [ 106.678502995869849, 10.790342823692997 ], [ 106.677766008634421, 10.790383430154805 ], [ 106.677766008634421, 10.789702622399801 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ad4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.789662018377706 ], [ 106.679239985585866, 10.789621412464697 ], [ 106.679239985585866, 10.790302215340159 ], [ 106.678502995869849, 10.790342823692997 ], [ 106.678502995869849, 10.789662018377706 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2adc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.788981218199909 ], [ 106.679239985585866, 10.788940614726737 ], [ 106.679239985585866, 10.789621412464697 ], [ 106.678502995869849, 10.789662018377706 ], [ 106.678502995869849, 10.788981218199909 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ae4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.788300423159892 ], [ 106.679239985585866, 10.788259822126555 ], [ 106.679239985585866, 10.788940614726737 ], [ 106.678502995869849, 10.788981218199909 ], [ 106.678502995869849, 10.788300423159892 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2aec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.788341022302541 ], [ 106.678502995869849, 10.788300423159892 ], [ 106.678502995869849, 10.788981218199909 ], [ 106.677766008634421, 10.789021819782283 ], [ 106.677766008634421, 10.788341022302541 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2af4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.787660229960855 ], [ 106.678502995869849, 10.787619633257931 ], [ 106.678502995869849, 10.788300423159892 ], [ 106.677766008634421, 10.788341022302541 ], [ 106.677766008634421, 10.787660229960855 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2afc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.787619633257931 ], [ 106.679239985585866, 10.787579034664439 ], [ 106.679239985585866, 10.788259822126555 ], [ 106.678502995869849, 10.788300423159892 ], [ 106.678502995869849, 10.787619633257931 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b04", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.787579034664439 ], [ 106.679976977782019, 10.787538434180384 ], [ 106.679976977782019, 10.788219219202542 ], [ 106.679239985585866, 10.788259822126555 ], [ 106.679239985585866, 10.787579034664439 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b0c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.787538434180384 ], [ 106.680713972457966, 10.787497831805773 ], [ 106.680713972457966, 10.788178614387856 ], [ 106.679976977782019, 10.788219219202542 ], [ 106.679976977782019, 10.787538434180384 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b14", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.788219219202542 ], [ 106.680713972457966, 10.788178614387856 ], [ 106.680713972457966, 10.788859402108015 ], [ 106.679976977782019, 10.788900009362766 ], [ 106.679976977782019, 10.788219219202542 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b1c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.788259822126555 ], [ 106.679976977782019, 10.788219219202542 ], [ 106.679976977782019, 10.788900009362766 ], [ 106.679239985585866, 10.788940614726737 ], [ 106.679239985585866, 10.788259822126555 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b24", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.788940614726737 ], [ 106.679976977782019, 10.788900009362766 ], [ 106.679976977782019, 10.789580804660782 ], [ 106.679239985585866, 10.789621412464697 ], [ 106.679239985585866, 10.788940614726737 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b2c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.789621412464697 ], [ 106.679976977782019, 10.789580804660782 ], [ 106.679976977782019, 10.790261605096298 ], [ 106.679239985585866, 10.790302215340159 ], [ 106.679239985585866, 10.789621412464697 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b34", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.789580804660782 ], [ 106.680713972457966, 10.789540194965966 ], [ 106.680713972457966, 10.790220992961421 ], [ 106.679976977782019, 10.790261605096298 ], [ 106.679976977782019, 10.789580804660782 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b3c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.788900009362766 ], [ 106.680713972457966, 10.788859402108015 ], [ 106.680713972457966, 10.789540194965966 ], [ 106.679976977782019, 10.789580804660782 ], [ 106.679976977782019, 10.788900009362766 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b44", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.788859402108015 ], [ 106.681450969613365, 10.788818792962484 ], [ 106.681450969613365, 10.789499583380255 ], [ 106.680713972457966, 10.789540194965966 ], [ 106.680713972457966, 10.788859402108015 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.789540194965966 ], [ 106.681450969613365, 10.789499583380255 ], [ 106.681450969613365, 10.790180378935537 ], [ 106.680713972457966, 10.790220992961421 ], [ 106.680713972457966, 10.789540194965966 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.789499583380255 ], [ 106.682187969247821, 10.789458969903661 ], [ 106.682187969247821, 10.790139763018653 ], [ 106.681450969613365, 10.790180378935537 ], [ 106.681450969613365, 10.789499583380255 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.788818792962484 ], [ 106.682187969247821, 10.788778181926185 ], [ 106.682187969247821, 10.789458969903661 ], [ 106.681450969613365, 10.789499583380255 ], [ 106.681450969613365, 10.788818792962484 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b64", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.788138007682509 ], [ 106.682187969247821, 10.788097399086508 ], [ 106.682187969247821, 10.788778181926185 ], [ 106.681450969613365, 10.788818792962484 ], [ 106.681450969613365, 10.788138007682509 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b6c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.788178614387856 ], [ 106.681450969613365, 10.788138007682509 ], [ 106.681450969613365, 10.788818792962484 ], [ 106.680713972457966, 10.788859402108015 ], [ 106.680713972457966, 10.788178614387856 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b74", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.787497831805773 ], [ 106.681450969613365, 10.787457227540614 ], [ 106.681450969613365, 10.788138007682509 ], [ 106.680713972457966, 10.788178614387856 ], [ 106.680713972457966, 10.787497831805773 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b7c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.787457227540614 ], [ 106.682187969247821, 10.787416621384914 ], [ 106.682187969247821, 10.788097399086508 ], [ 106.681450969613365, 10.788138007682509 ], [ 106.681450969613365, 10.787457227540614 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b84", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.786776452537081 ], [ 106.682187969247821, 10.78673584882169 ], [ 106.682187969247821, 10.787416621384914 ], [ 106.681450969613365, 10.787457227540614 ], [ 106.681450969613365, 10.786776452537081 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.786095682672189 ], [ 106.682187969247821, 10.786055081397111 ], [ 106.682187969247821, 10.78673584882169 ], [ 106.681450969613365, 10.786776452537081 ], [ 106.681450969613365, 10.786095682672189 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b94", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.786136282056958 ], [ 106.681450969613365, 10.786095682672189 ], [ 106.681450969613365, 10.786776452537081 ], [ 106.680713972457966, 10.786817054362047 ], [ 106.680713972457966, 10.786136282056958 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b9c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.786817054362047 ], [ 106.681450969613365, 10.786776452537081 ], [ 106.681450969613365, 10.787457227540614 ], [ 106.680713972457966, 10.787497831805773 ], [ 106.680713972457966, 10.786817054362047 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ba4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.786857654296581 ], [ 106.680713972457966, 10.786817054362047 ], [ 106.680713972457966, 10.787497831805773 ], [ 106.679976977782019, 10.787538434180384 ], [ 106.679976977782019, 10.786857654296581 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.786898252340674 ], [ 106.679976977782019, 10.786857654296581 ], [ 106.679976977782019, 10.787538434180384 ], [ 106.679239985585866, 10.787579034664439 ], [ 106.679239985585866, 10.786898252340674 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bb4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.786217475155537 ], [ 106.679976977782019, 10.786176879551411 ], [ 106.679976977782019, 10.786857654296581 ], [ 106.679239985585866, 10.786898252340674 ], [ 106.679239985585866, 10.786217475155537 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bbc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.786176879551411 ], [ 106.680713972457966, 10.786136282056958 ], [ 106.680713972457966, 10.786817054362047 ], [ 106.679976977782019, 10.786857654296581 ], [ 106.679976977782019, 10.786176879551411 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bc4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.785496109945157 ], [ 106.680713972457966, 10.785455514890792 ], [ 106.680713972457966, 10.786136282056958 ], [ 106.679976977782019, 10.786176879551411 ], [ 106.679976977782019, 10.785496109945157 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bcc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.785536703109312 ], [ 106.679976977782019, 10.785496109945157 ], [ 106.679976977782019, 10.786176879551411 ], [ 106.679239985585866, 10.786217475155537 ], [ 106.679239985585866, 10.785536703109312 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bd4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.784855936202286 ], [ 106.679976977782019, 10.784815345478108 ], [ 106.679976977782019, 10.785496109945157 ], [ 106.679239985585866, 10.785536703109312 ], [ 106.679239985585866, 10.784855936202286 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bdc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.784815345478108 ], [ 106.680713972457966, 10.784774752863832 ], [ 106.680713972457966, 10.785455514890792 ], [ 106.679976977782019, 10.785496109945157 ], [ 106.679976977782019, 10.784815345478108 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2be4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.784774752863832 ], [ 106.681450969613365, 10.78473415835947 ], [ 106.681450969613365, 10.785414917946227 ], [ 106.680713972457966, 10.785455514890792 ], [ 106.680713972457966, 10.784774752863832 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.785455514890792 ], [ 106.681450969613365, 10.785414917946227 ], [ 106.681450969613365, 10.786095682672189 ], [ 106.680713972457966, 10.786136282056958 ], [ 106.680713972457966, 10.785455514890792 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bf4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.785414917946227 ], [ 106.682187969247821, 10.785374319111463 ], [ 106.682187969247821, 10.786055081397111 ], [ 106.681450969613365, 10.786095682672189 ], [ 106.681450969613365, 10.785414917946227 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2bfc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.78473415835947 ], [ 106.682187969247821, 10.784693561965032 ], [ 106.682187969247821, 10.785374319111463 ], [ 106.681450969613365, 10.785414917946227 ], [ 106.681450969613365, 10.78473415835947 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c04", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.784693561965032 ], [ 106.682924971360933, 10.784652963680518 ], [ 106.682924971360933, 10.785333718386513 ], [ 106.682187969247821, 10.785374319111463 ], [ 106.682187969247821, 10.784693561965032 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c0c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.785374319111463 ], [ 106.682924971360933, 10.785333718386513 ], [ 106.682924971360933, 10.786014478231728 ], [ 106.682187969247821, 10.786055081397111 ], [ 106.682187969247821, 10.785374319111463 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c14", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.785333718386513 ], [ 106.683661975952418, 10.785293115771385 ], [ 106.683661975952418, 10.785973873176054 ], [ 106.682924971360933, 10.786014478231728 ], [ 106.682924971360933, 10.785333718386513 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c1c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.784652963680518 ], [ 106.683661975952418, 10.784612363505941 ], [ 106.683661975952418, 10.785293115771385 ], [ 106.682924971360933, 10.785333718386513 ], [ 106.682924971360933, 10.784652963680518 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c24", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.784612363505941 ], [ 106.684398983021822, 10.784571761441306 ], [ 106.684398983021822, 10.785252511266084 ], [ 106.683661975952418, 10.785293115771385 ], [ 106.683661975952418, 10.784612363505941 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c2c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.784571761441306 ], [ 106.68513599256886, 10.784531157486622 ], [ 106.68513599256886, 10.785211904870621 ], [ 106.684398983021822, 10.785252511266084 ], [ 106.684398983021822, 10.784571761441306 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c34", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.785252511266084 ], [ 106.68513599256886, 10.785211904870621 ], [ 106.68513599256886, 10.785892657393852 ], [ 106.684398983021822, 10.785933266230092 ], [ 106.684398983021822, 10.785252511266084 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c3c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.785293115771385 ], [ 106.684398983021822, 10.785252511266084 ], [ 106.684398983021822, 10.785933266230092 ], [ 106.683661975952418, 10.785973873176054 ], [ 106.683661975952418, 10.785293115771385 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c44", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.785973873176054 ], [ 106.684398983021822, 10.785933266230092 ], [ 106.684398983021822, 10.786614026333044 ], [ 106.683661975952418, 10.786654635719662 ], [ 106.683661975952418, 10.785973873176054 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.785933266230092 ], [ 106.68513599256886, 10.785892657393852 ], [ 106.68513599256886, 10.786573415056031 ], [ 106.684398983021822, 10.786614026333044 ], [ 106.684398983021822, 10.785933266230092 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.786614026333044 ], [ 106.68513599256886, 10.786573415056031 ], [ 106.68513599256886, 10.787254177856878 ], [ 106.684398983021822, 10.787294791574658 ], [ 106.684398983021822, 10.786614026333044 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.786654635719662 ], [ 106.684398983021822, 10.786614026333044 ], [ 106.684398983021822, 10.787294791574658 ], [ 106.683661975952418, 10.787335403401928 ], [ 106.683661975952418, 10.786654635719662 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c64", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.786695243215879 ], [ 106.683661975952418, 10.786654635719662 ], [ 106.683661975952418, 10.787335403401928 ], [ 106.682924971360933, 10.787376013338685 ], [ 106.682924971360933, 10.786695243215879 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c6c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.786014478231728 ], [ 106.683661975952418, 10.785973873176054 ], [ 106.683661975952418, 10.786654635719662 ], [ 106.682924971360933, 10.786695243215879 ], [ 106.682924971360933, 10.786014478231728 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c74", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.786055081397111 ], [ 106.682924971360933, 10.786014478231728 ], [ 106.682924971360933, 10.786695243215879 ], [ 106.682187969247821, 10.78673584882169 ], [ 106.682187969247821, 10.786055081397111 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c7c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.78673584882169 ], [ 106.682924971360933, 10.786695243215879 ], [ 106.682924971360933, 10.787376013338685 ], [ 106.682187969247821, 10.787416621384914 ], [ 106.682187969247821, 10.78673584882169 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c84", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.787416621384914 ], [ 106.682924971360933, 10.787376013338685 ], [ 106.682924971360933, 10.78805678859986 ], [ 106.682187969247821, 10.788097399086508 ], [ 106.682187969247821, 10.787416621384914 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.787376013338685 ], [ 106.683661975952418, 10.787335403401928 ], [ 106.683661975952418, 10.788016176222571 ], [ 106.682924971360933, 10.78805678859986 ], [ 106.682924971360933, 10.787376013338685 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c94", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.78805678859986 ], [ 106.683661975952418, 10.788016176222571 ], [ 106.683661975952418, 10.788696954181304 ], [ 106.682924971360933, 10.788737568999121 ], [ 106.682924971360933, 10.78805678859986 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2c9c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.788097399086508 ], [ 106.682924971360933, 10.78805678859986 ], [ 106.682924971360933, 10.788737568999121 ], [ 106.682187969247821, 10.788778181926185 ], [ 106.682187969247821, 10.788097399086508 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ca4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.788778181926185 ], [ 106.682924971360933, 10.788737568999121 ], [ 106.682924971360933, 10.789418354536188 ], [ 106.682187969247821, 10.789458969903661 ], [ 106.682187969247821, 10.788778181926185 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2cac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.789458969903661 ], [ 106.682924971360933, 10.789418354536188 ], [ 106.682924971360933, 10.790099145210778 ], [ 106.682187969247821, 10.790139763018653 ], [ 106.682187969247821, 10.789458969903661 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2cb4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.789418354536188 ], [ 106.683661975952418, 10.789377737277848 ], [ 106.683661975952418, 10.790058525511915 ], [ 106.682924971360933, 10.790099145210778 ], [ 106.682924971360933, 10.789418354536188 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2cbc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.788737568999121 ], [ 106.683661975952418, 10.788696954181304 ], [ 106.683661975952418, 10.789377737277848 ], [ 106.682924971360933, 10.789418354536188 ], [ 106.682924971360933, 10.788737568999121 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2cc4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.788696954181304 ], [ 106.684398983021822, 10.788656337472743 ], [ 106.684398983021822, 10.789337118128644 ], [ 106.683661975952418, 10.789377737277848 ], [ 106.683661975952418, 10.788696954181304 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ccc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.789377737277848 ], [ 106.684398983021822, 10.789337118128644 ], [ 106.684398983021822, 10.790017903922081 ], [ 106.683661975952418, 10.790058525511915 ], [ 106.683661975952418, 10.789377737277848 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2cd4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.789337118128644 ], [ 106.68513599256886, 10.789296497088591 ], [ 106.68513599256886, 10.789977280441276 ], [ 106.684398983021822, 10.790017903922081 ], [ 106.684398983021822, 10.789337118128644 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2cdc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.788656337472743 ], [ 106.68513599256886, 10.788615718873441 ], [ 106.68513599256886, 10.789296497088591 ], [ 106.684398983021822, 10.789337118128644 ], [ 106.684398983021822, 10.788656337472743 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ce4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.787975561954651 ], [ 106.68513599256886, 10.78793494579611 ], [ 106.68513599256886, 10.788615718873441 ], [ 106.684398983021822, 10.788656337472743 ], [ 106.684398983021822, 10.787975561954651 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2cec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.788016176222571 ], [ 106.684398983021822, 10.787975561954651 ], [ 106.684398983021822, 10.788656337472743 ], [ 106.683661975952418, 10.788696954181304 ], [ 106.683661975952418, 10.788016176222571 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2cf4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.787335403401928 ], [ 106.684398983021822, 10.787294791574658 ], [ 106.684398983021822, 10.787975561954651 ], [ 106.683661975952418, 10.788016176222571 ], [ 106.683661975952418, 10.787335403401928 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2cfc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.787294791574658 ], [ 106.68513599256886, 10.787254177856878 ], [ 106.68513599256886, 10.78793494579611 ], [ 106.684398983021822, 10.787975561954651 ], [ 106.684398983021822, 10.787294791574658 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d04", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.787254177856878 ], [ 106.685873004593077, 10.787213562248599 ], [ 106.685873004593077, 10.787894327746951 ], [ 106.68513599256886, 10.78793494579611 ], [ 106.68513599256886, 10.787254177856878 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d0c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.787213562248599 ], [ 106.686610019094246, 10.787172944749825 ], [ 106.686610019094189, 10.787853707807185 ], [ 106.685873004593077, 10.787894327746951 ], [ 106.685873004593077, 10.787213562248599 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d14", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.787894327746951 ], [ 106.686610019094189, 10.787853707807185 ], [ 106.686610019094246, 10.788534476002656 ], [ 106.685873004593077, 10.78857509838341 ], [ 106.685873004593077, 10.787894327746951 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d1c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.78793494579611 ], [ 106.685873004593077, 10.787894327746951 ], [ 106.685873004593077, 10.78857509838341 ], [ 106.68513599256886, 10.788615718873441 ], [ 106.68513599256886, 10.78793494579611 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d24", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.788615718873441 ], [ 106.685873004593077, 10.78857509838341 ], [ 106.685873004593077, 10.789255874157691 ], [ 106.68513599256886, 10.789296497088591 ], [ 106.68513599256886, 10.788615718873441 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d2c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.789296497088591 ], [ 106.685873004593077, 10.789255874157691 ], [ 106.685873004593077, 10.789936655069512 ], [ 106.68513599256886, 10.789977280441276 ], [ 106.68513599256886, 10.789296497088591 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d34", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.789255874157691 ], [ 106.686610019094246, 10.789215249335953 ], [ 106.686610019094246, 10.789896027806794 ], [ 106.685873004593077, 10.789936655069512 ], [ 106.685873004593077, 10.789255874157691 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d3c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.78857509838341 ], [ 106.686610019094246, 10.788534476002656 ], [ 106.686610019094246, 10.789215249335953 ], [ 106.685873004593077, 10.789255874157691 ], [ 106.685873004593077, 10.78857509838341 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d44", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.788534476002656 ], [ 106.687347036071799, 10.788493851731186 ], [ 106.687347036071799, 10.789174622623385 ], [ 106.686610019094246, 10.789215249335953 ], [ 106.686610019094246, 10.788534476002656 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.789215249335953 ], [ 106.687347036071799, 10.789174622623385 ], [ 106.687347036071799, 10.789855398653133 ], [ 106.686610019094246, 10.789896027806794 ], [ 106.686610019094246, 10.789215249335953 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.789174622623385 ], [ 106.688084055525565, 10.789133994019995 ], [ 106.688084055525565, 10.789814767608535 ], [ 106.687347036071799, 10.789855398653133 ], [ 106.687347036071799, 10.789174622623385 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.788493851731186 ], [ 106.688084055525565, 10.788453225569011 ], [ 106.688084055525565, 10.789133994019995 ], [ 106.687347036071799, 10.789174622623385 ], [ 106.687347036071799, 10.788493851731186 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d64", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.787813085976824 ], [ 106.688084055525565, 10.787772462255866 ], [ 106.688084055525565, 10.788453225569011 ], [ 106.687347036071799, 10.788493851731186 ], [ 106.687347036071799, 10.787813085976824 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d6c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.787853707807185 ], [ 106.687347036071799, 10.787813085976824 ], [ 106.687347036071799, 10.788493851731186 ], [ 106.686610019094246, 10.788534476002656 ], [ 106.686610019094189, 10.787853707807185 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d74", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.787172944749825 ], [ 106.687347036071799, 10.78713232536057 ], [ 106.687347036071799, 10.787813085976824 ], [ 106.686610019094189, 10.787853707807185 ], [ 106.686610019094246, 10.787172944749825 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d7c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.78713232536057 ], [ 106.688084055525565, 10.787091704080835 ], [ 106.688084055525565, 10.787772462255866 ], [ 106.687347036071799, 10.787813085976824 ], [ 106.687347036071799, 10.78713232536057 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d84", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.786451569882715 ], [ 106.688084055525565, 10.786410951044209 ], [ 106.688084055525565, 10.787091704080835 ], [ 106.687347036071799, 10.78713232536057 ], [ 106.687347036071799, 10.786451569882715 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.78577081954354 ], [ 106.688084055525565, 10.785730203146265 ], [ 106.688084055525565, 10.786410951044209 ], [ 106.687347036071799, 10.786451569882715 ], [ 106.687347036071799, 10.78577081954354 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d94", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.785811434050569 ], [ 106.687347036071799, 10.78577081954354 ], [ 106.687347036071799, 10.786451569882715 ], [ 106.686610019094246, 10.786492186830861 ], [ 106.686610019094246, 10.785811434050569 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d9c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.786492186830861 ], [ 106.687347036071799, 10.786451569882715 ], [ 106.687347036071799, 10.78713232536057 ], [ 106.686610019094246, 10.787172944749825 ], [ 106.686610019094246, 10.786492186830861 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2da4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.786532801888635 ], [ 106.686610019094246, 10.786492186830861 ], [ 106.686610019094246, 10.787172944749825 ], [ 106.685873004593077, 10.787213562248599 ], [ 106.685873004593077, 10.786532801888635 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2dac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.786573415056031 ], [ 106.685873004593077, 10.786532801888635 ], [ 106.685873004593077, 10.787213562248599 ], [ 106.68513599256886, 10.787254177856878 ], [ 106.68513599256886, 10.786573415056031 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2db4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.785892657393852 ], [ 106.685873004593077, 10.785852046667342 ], [ 106.685873004593077, 10.786532801888635 ], [ 106.68513599256886, 10.786573415056031 ], [ 106.68513599256886, 10.785892657393852 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2dbc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.785852046667342 ], [ 106.686610019094246, 10.785811434050569 ], [ 106.686610019094246, 10.786492186830861 ], [ 106.685873004593077, 10.786532801888635 ], [ 106.685873004593077, 10.785852046667342 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2dc4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.785171296585002 ], [ 106.686610019094246, 10.785130686409236 ], [ 106.686610019094246, 10.785811434050569 ], [ 106.685873004593077, 10.785852046667342 ], [ 106.685873004593077, 10.785171296585002 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2dcc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.785211904870621 ], [ 106.685873004593077, 10.785171296585002 ], [ 106.685873004593077, 10.785852046667342 ], [ 106.68513599256886, 10.785892657393852 ], [ 106.68513599256886, 10.785211904870621 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2dd4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.784531157486622 ], [ 106.685873004593077, 10.7844905516419 ], [ 106.685873004593077, 10.785171296585002 ], [ 106.68513599256886, 10.785211904870621 ], [ 106.68513599256886, 10.784531157486622 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ddc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.7844905516419 ], [ 106.686610019094189, 10.784449943907143 ], [ 106.686610019094246, 10.785130686409236 ], [ 106.685873004593077, 10.785171296585002 ], [ 106.685873004593077, 10.7844905516419 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2de4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.784449943907143 ], [ 106.687347036071799, 10.784409334282364 ], [ 106.687347036071799, 10.78509007434333 ], [ 106.686610019094246, 10.785130686409236 ], [ 106.686610019094189, 10.784449943907143 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2dec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.785130686409236 ], [ 106.687347036071799, 10.78509007434333 ], [ 106.687347036071799, 10.78577081954354 ], [ 106.686610019094246, 10.785811434050569 ], [ 106.686610019094246, 10.785130686409236 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2df4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.78509007434333 ], [ 106.688084055525565, 10.78504946038729 ], [ 106.688084055525565, 10.785730203146265 ], [ 106.687347036071799, 10.78577081954354 ], [ 106.687347036071799, 10.78509007434333 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2dfc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.784409334282364 ], [ 106.688084055525565, 10.784368722767567 ], [ 106.688084055525565, 10.78504946038729 ], [ 106.687347036071799, 10.78509007434333 ], [ 106.687347036071799, 10.784409334282364 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e04", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.783728599360927 ], [ 106.688084055525565, 10.783687990287376 ], [ 106.688084055525565, 10.784368722767567 ], [ 106.687347036071799, 10.784409334282364 ], [ 106.687347036071799, 10.783728599360927 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e0c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.783769206544578 ], [ 106.687347036071799, 10.783728599360927 ], [ 106.687347036071799, 10.784409334282364 ], [ 106.686610019094189, 10.784449943907143 ], [ 106.686610019094246, 10.783769206544578 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e14", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.78308847432182 ], [ 106.687347036071799, 10.783047869579303 ], [ 106.687347036071799, 10.783728599360927 ], [ 106.686610019094246, 10.783769206544578 ], [ 106.686610019094189, 10.78308847432182 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e1c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.783047869579303 ], [ 106.688084055525565, 10.783007262947002 ], [ 106.688084055525565, 10.783687990287376 ], [ 106.687347036071799, 10.783728599360927 ], [ 106.687347036071799, 10.783047869579303 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e24", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.782367144937776 ], [ 106.688084055525565, 10.782326540746729 ], [ 106.688084055525565, 10.783007262947002 ], [ 106.687347036071799, 10.783047869579303 ], [ 106.687347036071799, 10.782367144937776 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e2c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.781686425436623 ], [ 106.688084055525565, 10.781645823686835 ], [ 106.688084055525565, 10.782326540746729 ], [ 106.687347036071799, 10.782367144937776 ], [ 106.687347036071799, 10.781686425436623 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e34", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.781727025296856 ], [ 106.687347036071799, 10.781686425436623 ], [ 106.687347036071799, 10.782367144937776 ], [ 106.686610019094246, 10.782407747239152 ], [ 106.686610019094189, 10.781727025296856 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e3c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.782407747239152 ], [ 106.687347036071799, 10.782367144937776 ], [ 106.687347036071799, 10.783047869579303 ], [ 106.686610019094189, 10.78308847432182 ], [ 106.686610019094246, 10.782407747239152 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e44", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.782448347650845 ], [ 106.686610019094246, 10.782407747239152 ], [ 106.686610019094189, 10.78308847432182 ], [ 106.685873004593077, 10.783129077174539 ], [ 106.685873004593077, 10.782448347650845 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.78176762326752 ], [ 106.686610019094189, 10.781727025296856 ], [ 106.686610019094246, 10.782407747239152 ], [ 106.685873004593077, 10.782448347650845 ], [ 106.685873004593077, 10.78176762326752 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.781808219348616 ], [ 106.685873004593077, 10.78176762326752 ], [ 106.685873004593077, 10.782448347650845 ], [ 106.68513599256886, 10.782488946172855 ], [ 106.68513599256886, 10.781808219348616 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.782488946172855 ], [ 106.685873004593077, 10.782448347650845 ], [ 106.685873004593077, 10.783129077174539 ], [ 106.68513599256886, 10.783169678137456 ], [ 106.68513599256886, 10.782488946172855 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e64", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.783169678137456 ], [ 106.685873004593077, 10.783129077174539 ], [ 106.685873004593077, 10.783809811838319 ], [ 106.68513599256886, 10.78385041524214 ], [ 106.68513599256886, 10.783169678137456 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e6c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.783129077174539 ], [ 106.686610019094189, 10.78308847432182 ], [ 106.686610019094246, 10.783769206544578 ], [ 106.685873004593077, 10.783809811838319 ], [ 106.685873004593077, 10.783129077174539 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e74", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.783809811838319 ], [ 106.686610019094246, 10.783769206544578 ], [ 106.686610019094189, 10.784449943907143 ], [ 106.685873004593077, 10.7844905516419 ], [ 106.685873004593077, 10.783809811838319 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e7c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.78385041524214 ], [ 106.685873004593077, 10.783809811838319 ], [ 106.685873004593077, 10.7844905516419 ], [ 106.68513599256886, 10.784531157486622 ], [ 106.68513599256886, 10.78385041524214 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e84", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.783891016756037 ], [ 106.68513599256886, 10.78385041524214 ], [ 106.68513599256886, 10.784531157486622 ], [ 106.684398983021822, 10.784571761441306 ], [ 106.684398983021822, 10.783891016756037 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.783210277210562 ], [ 106.68513599256886, 10.783169678137456 ], [ 106.68513599256886, 10.78385041524214 ], [ 106.684398983021822, 10.783891016756037 ], [ 106.684398983021822, 10.783210277210562 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e94", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.783250874393852 ], [ 106.684398983021822, 10.783210277210562 ], [ 106.684398983021822, 10.783891016756037 ], [ 106.683661975952418, 10.783931616380002 ], [ 106.683661975952418, 10.783250874393852 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2e9c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.783931616380002 ], [ 106.684398983021822, 10.783891016756037 ], [ 106.684398983021822, 10.784571761441306 ], [ 106.683661975952418, 10.784612363505941 ], [ 106.683661975952418, 10.783931616380002 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ea4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.783972214114023 ], [ 106.683661975952418, 10.783931616380002 ], [ 106.683661975952418, 10.784612363505941 ], [ 106.682924971360933, 10.784652963680518 ], [ 106.682924971360933, 10.783972214114023 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2eac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.784012809958096 ], [ 106.682924971360933, 10.783972214114023 ], [ 106.682924971360933, 10.784652963680518 ], [ 106.682187969247821, 10.784693561965032 ], [ 106.682187969247821, 10.784012809958096 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2eb4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.78333206309094 ], [ 106.682924971360933, 10.783291469687311 ], [ 106.682924971360933, 10.783972214114023 ], [ 106.682187969247821, 10.784012809958096 ], [ 106.682187969247821, 10.78333206309094 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ebc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.783291469687311 ], [ 106.683661975952418, 10.783250874393852 ], [ 106.683661975952418, 10.783931616380002 ], [ 106.682924971360933, 10.783972214114023 ], [ 106.682924971360933, 10.783291469687311 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ec4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.782610730400673 ], [ 106.683661975952418, 10.782570137547774 ], [ 106.683661975952418, 10.783250874393852 ], [ 106.682924971360933, 10.783291469687311 ], [ 106.682924971360933, 10.782610730400673 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ecc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.782651321363851 ], [ 106.682924971360933, 10.782610730400673 ], [ 106.682924971360933, 10.783291469687311 ], [ 106.682187969247821, 10.78333206309094 ], [ 106.682187969247821, 10.782651321363851 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ed4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.781970584777106 ], [ 106.682924971360933, 10.781929996254382 ], [ 106.682924971360933, 10.782610730400673 ], [ 106.682187969247821, 10.782651321363851 ], [ 106.682187969247821, 10.781970584777106 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2edc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.781929996254382 ], [ 106.683661975952418, 10.781889405842053 ], [ 106.683661975952418, 10.782570137547774 ], [ 106.682924971360933, 10.782610730400673 ], [ 106.682924971360933, 10.781929996254382 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ee4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.781889405842053 ], [ 106.684398983021822, 10.781848813540128 ], [ 106.684398983021822, 10.782529542805166 ], [ 106.683661975952418, 10.782570137547774 ], [ 106.683661975952418, 10.781889405842053 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2eec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.782570137547774 ], [ 106.684398983021822, 10.782529542805166 ], [ 106.684398983021822, 10.783210277210562 ], [ 106.683661975952418, 10.783250874393852 ], [ 106.683661975952418, 10.782570137547774 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ef4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.782529542805166 ], [ 106.68513599256886, 10.782488946172855 ], [ 106.68513599256886, 10.783169678137456 ], [ 106.684398983021822, 10.783210277210562 ], [ 106.684398983021822, 10.782529542805166 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2efc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.781848813540128 ], [ 106.68513599256886, 10.781808219348616 ], [ 106.68513599256886, 10.782488946172855 ], [ 106.684398983021822, 10.782529542805166 ], [ 106.684398983021822, 10.781848813540128 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f04", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.781168089415733 ], [ 106.68513599256886, 10.781127497665022 ], [ 106.68513599256886, 10.781808219348616 ], [ 106.684398983021822, 10.781848813540128 ], [ 106.684398983021822, 10.781168089415733 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f0c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.780487370432262 ], [ 106.68513599256886, 10.780446781122363 ], [ 106.68513599256886, 10.781127497665022 ], [ 106.684398983021822, 10.781168089415733 ], [ 106.684398983021822, 10.780487370432262 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f14", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.780527957852806 ], [ 106.684398983021822, 10.780487370432262 ], [ 106.684398983021822, 10.781168089415733 ], [ 106.683661975952418, 10.78120867927697 ], [ 106.683661975952418, 10.780527957852806 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f1c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.78120867927697 ], [ 106.684398983021822, 10.781168089415733 ], [ 106.684398983021822, 10.781848813540128 ], [ 106.683661975952418, 10.781889405842053 ], [ 106.683661975952418, 10.78120867927697 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f24", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.781249267248725 ], [ 106.683661975952418, 10.78120867927697 ], [ 106.683661975952418, 10.781889405842053 ], [ 106.682924971360933, 10.781929996254382 ], [ 106.682924971360933, 10.781249267248725 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f2c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.781289853330989 ], [ 106.682924971360933, 10.781249267248725 ], [ 106.682924971360933, 10.781929996254382 ], [ 106.682187969247821, 10.781970584777106 ], [ 106.682187969247821, 10.781289853330989 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f34", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.780609127025786 ], [ 106.682924971360933, 10.780568543383984 ], [ 106.682924971360933, 10.781249267248725 ], [ 106.682187969247821, 10.781289853330989 ], [ 106.682187969247821, 10.780609127025786 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f3c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.780568543383984 ], [ 106.683661975952418, 10.780527957852806 ], [ 106.683661975952418, 10.78120867927697 ], [ 106.682924971360933, 10.781249267248725 ], [ 106.682924971360933, 10.780568543383984 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f44", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.779887824660443 ], [ 106.683661975952418, 10.779847241569849 ], [ 106.683661975952418, 10.780527957852806 ], [ 106.682924971360933, 10.780568543383984 ], [ 106.682924971360933, 10.779887824660443 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.779928405861778 ], [ 106.682924971360933, 10.779887824660443 ], [ 106.682924971360933, 10.780568543383984 ], [ 106.682187969247821, 10.780609127025786 ], [ 106.682187969247821, 10.779928405861778 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.779247689839247 ], [ 106.682924971360933, 10.779207111078383 ], [ 106.682924971360933, 10.779887824660443 ], [ 106.682187969247821, 10.779928405861778 ], [ 106.682187969247821, 10.779247689839247 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.779207111078383 ], [ 106.683661975952418, 10.779166530428377 ], [ 106.683661975952418, 10.779847241569849 ], [ 106.682924971360933, 10.779887824660443 ], [ 106.682924971360933, 10.779207111078383 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f64", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.779166530428377 ], [ 106.684398983021822, 10.779125947889233 ], [ 106.684398983021822, 10.779806656590003 ], [ 106.683661975952418, 10.779847241569849 ], [ 106.683661975952418, 10.779166530428377 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f6c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.779847241569849 ], [ 106.684398983021822, 10.779806656590003 ], [ 106.684398983021822, 10.780487370432262 ], [ 106.683661975952418, 10.780527957852806 ], [ 106.683661975952418, 10.779847241569849 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f74", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.779806656590003 ], [ 106.68513599256886, 10.779766069720914 ], [ 106.68513599256886, 10.780446781122363 ], [ 106.684398983021822, 10.780487370432262 ], [ 106.684398983021822, 10.779806656590003 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f7c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.684398983021822, 10.779125947889233 ], [ 106.68513599256886, 10.779085363460961 ], [ 106.68513599256886, 10.779766069720914 ], [ 106.684398983021822, 10.779806656590003 ], [ 106.684398983021822, 10.779125947889233 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f84", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.779085363460961 ], [ 106.685873004593077, 10.779044777143568 ], [ 106.685873004593077, 10.77972548096259 ], [ 106.68513599256886, 10.779766069720914 ], [ 106.68513599256886, 10.779085363460961 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.779044777143568 ], [ 106.686610019094189, 10.77900418893706 ], [ 106.686610019094189, 10.779684890315036 ], [ 106.685873004593077, 10.77972548096259 ], [ 106.685873004593077, 10.779044777143568 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f94", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.77972548096259 ], [ 106.686610019094189, 10.779684890315036 ], [ 106.686610019094246, 10.780365596834514 ], [ 106.685873004593077, 10.780406189923109 ], [ 106.685873004593077, 10.77972548096259 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f9c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.779766069720914 ], [ 106.685873004593077, 10.77972548096259 ], [ 106.685873004593077, 10.780406189923109 ], [ 106.68513599256886, 10.780446781122363 ], [ 106.68513599256886, 10.779766069720914 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fa4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.780446781122363 ], [ 106.685873004593077, 10.780406189923109 ], [ 106.685873004593077, 10.781086904024848 ], [ 106.68513599256886, 10.781127497665022 ], [ 106.68513599256886, 10.780446781122363 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.781127497665022 ], [ 106.685873004593077, 10.781086904024848 ], [ 106.685873004593077, 10.78176762326752 ], [ 106.68513599256886, 10.781808219348616 ], [ 106.68513599256886, 10.781127497665022 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fb4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.781086904024848 ], [ 106.686610019094189, 10.781046308495213 ], [ 106.686610019094189, 10.781727025296856 ], [ 106.685873004593077, 10.78176762326752 ], [ 106.685873004593077, 10.781086904024848 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fbc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.685873004593077, 10.780406189923109 ], [ 106.686610019094246, 10.780365596834514 ], [ 106.686610019094189, 10.781046308495213 ], [ 106.685873004593077, 10.781086904024848 ], [ 106.685873004593077, 10.780406189923109 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fc4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.780365596834514 ], [ 106.687347036071799, 10.780325001856585 ], [ 106.687347036071799, 10.781005711076133 ], [ 106.686610019094189, 10.781046308495213 ], [ 106.686610019094246, 10.780365596834514 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fcc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.781046308495213 ], [ 106.687347036071799, 10.781005711076133 ], [ 106.687347036071799, 10.781686425436623 ], [ 106.686610019094189, 10.781727025296856 ], [ 106.686610019094189, 10.781046308495213 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fd4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.781005711076133 ], [ 106.688084055525565, 10.780965111767607 ], [ 106.688084055525565, 10.781645823686835 ], [ 106.687347036071799, 10.781686425436623 ], [ 106.687347036071799, 10.781005711076133 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fdc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.780325001856585 ], [ 106.688084055525565, 10.780284404989329 ], [ 106.688084055525565, 10.780965111767607 ], [ 106.687347036071799, 10.781005711076133 ], [ 106.687347036071799, 10.780325001856585 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fe4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.779644297778264 ], [ 106.688084055525565, 10.779603703352279 ], [ 106.688084055525565, 10.780284404989329 ], [ 106.687347036071799, 10.780325001856585 ], [ 106.687347036071799, 10.779644297778264 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2fec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.779684890315036 ], [ 106.687347036071799, 10.779644297778264 ], [ 106.687347036071799, 10.780325001856585 ], [ 106.686610019094246, 10.780365596834514 ], [ 106.686610019094189, 10.779684890315036 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ff4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094189, 10.77900418893706 ], [ 106.687347036071799, 10.778963598841449 ], [ 106.687347036071799, 10.779644297778264 ], [ 106.686610019094189, 10.779684890315036 ], [ 106.686610019094189, 10.77900418893706 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2ffc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.778963598841449 ], [ 106.688084055525565, 10.778923006856742 ], [ 106.688084055525565, 10.779603703352279 ], [ 106.687347036071799, 10.779644297778264 ], [ 106.687347036071799, 10.778963598841449 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3004", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.778923006856742 ], [ 106.688821077455032, 10.778882412982945 ], [ 106.688821077455032, 10.779563107037092 ], [ 106.688084055525565, 10.779603703352279 ], [ 106.688084055525565, 10.778923006856742 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f300c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.778882412982945 ], [ 106.689558101859916, 10.778841817220068 ], [ 106.689558101859916, 10.779522508832706 ], [ 106.688821077455032, 10.779563107037092 ], [ 106.688821077455032, 10.778882412982945 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3014", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.779563107037092 ], [ 106.689558101859916, 10.779522508832706 ], [ 106.689558101859916, 10.780203205586865 ], [ 106.688821077455032, 10.780243806232752 ], [ 106.688821077455032, 10.779563107037092 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f301c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.779603703352279 ], [ 106.688821077455032, 10.779563107037092 ], [ 106.688821077455032, 10.780243806232752 ], [ 106.688084055525565, 10.780284404989329 ], [ 106.688084055525565, 10.779603703352279 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3024", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.780284404989329 ], [ 106.688821077455032, 10.780243806232752 ], [ 106.688821077455032, 10.780924510569648 ], [ 106.688084055525565, 10.780965111767607 ], [ 106.688084055525565, 10.780284404989329 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f302c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.780965111767607 ], [ 106.688821077455032, 10.780924510569648 ], [ 106.688821077455032, 10.781605220047497 ], [ 106.688084055525565, 10.781645823686835 ], [ 106.688084055525565, 10.780965111767607 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3034", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.780924510569648 ], [ 106.689558101859916, 10.780883907482265 ], [ 106.689558101859916, 10.78156461451862 ], [ 106.688821077455032, 10.781605220047497 ], [ 106.688821077455032, 10.780924510569648 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f303c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.780243806232752 ], [ 106.689558101859916, 10.780203205586865 ], [ 106.689558101859916, 10.780883907482265 ], [ 106.688821077455032, 10.780924510569648 ], [ 106.688821077455032, 10.780243806232752 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3044", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.780203205586865 ], [ 106.690295128739763, 10.780162603051673 ], [ 106.690295128739763, 10.78084330250546 ], [ 106.689558101859916, 10.780883907482265 ], [ 106.689558101859916, 10.780203205586865 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f304c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.780883907482265 ], [ 106.690295128739763, 10.78084330250546 ], [ 106.690295128739763, 10.781524007100206 ], [ 106.689558101859916, 10.78156461451862 ], [ 106.689558101859916, 10.780883907482265 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3054", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.78084330250546 ], [ 106.691032158094345, 10.780802695639245 ], [ 106.691032158094345, 10.78148339779227 ], [ 106.690295128739763, 10.781524007100206 ], [ 106.690295128739763, 10.78084330250546 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f305c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.780162603051673 ], [ 106.691032158094345, 10.780121998627187 ], [ 106.691032158094345, 10.780802695639245 ], [ 106.690295128739763, 10.78084330250546 ], [ 106.690295128739763, 10.780162603051673 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3064", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.779481908739132 ], [ 106.691032158094345, 10.779441306756379 ], [ 106.691032158094345, 10.780121998627187 ], [ 106.690295128739763, 10.780162603051673 ], [ 106.690295128739763, 10.779481908739132 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f306c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.779522508832706 ], [ 106.690295128739763, 10.779481908739132 ], [ 106.690295128739763, 10.780162603051673 ], [ 106.689558101859916, 10.780203205586865 ], [ 106.689558101859916, 10.779522508832706 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3074", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.778841817220068 ], [ 106.690295128739763, 10.778801219568116 ], [ 106.690295128739763, 10.779481908739132 ], [ 106.689558101859916, 10.779522508832706 ], [ 106.689558101859916, 10.778841817220068 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f307c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.778801219568116 ], [ 106.691032158094345, 10.778760620027098 ], [ 106.691032158094345, 10.779441306756379 ], [ 106.690295128739763, 10.779481908739132 ], [ 106.690295128739763, 10.778801219568116 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3084", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.778760620027098 ], [ 106.691769189923207, 10.778720018597026 ], [ 106.691769189923207, 10.779400702884452 ], [ 106.691032158094345, 10.779441306756379 ], [ 106.691032158094345, 10.778760620027098 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f308c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.779441306756379 ], [ 106.691769189923207, 10.779400702884452 ], [ 106.691769189923207, 10.780081392313415 ], [ 106.691032158094345, 10.780121998627187 ], [ 106.691032158094345, 10.779441306756379 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3094", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.779400702884452 ], [ 106.692506224225951, 10.779360097123361 ], [ 106.692506224225951, 10.780040784110362 ], [ 106.691769189923207, 10.780081392313415 ], [ 106.691769189923207, 10.779400702884452 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f309c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.778720018597026 ], [ 106.692506224225951, 10.778679415277901 ], [ 106.692506224225951, 10.779360097123361 ], [ 106.691769189923207, 10.779400702884452 ], [ 106.691769189923207, 10.778720018597026 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.778679415277901 ], [ 106.693243261002294, 10.778638810069735 ], [ 106.693243261002294, 10.779319489473114 ], [ 106.692506224225951, 10.779360097123361 ], [ 106.692506224225951, 10.778679415277901 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.778638810069735 ], [ 106.693980300251837, 10.778598202972537 ], [ 106.693980300251837, 10.779278879933718 ], [ 106.693243261002294, 10.779319489473114 ], [ 106.693243261002294, 10.778638810069735 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.779319489473114 ], [ 106.693980300251837, 10.779278879933718 ], [ 106.693980300251837, 10.779959562036447 ], [ 106.693243261002294, 10.780000174018037 ], [ 106.693243261002294, 10.779319489473114 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.779360097123361 ], [ 106.693243261002294, 10.779319489473114 ], [ 106.693243261002294, 10.780000174018037 ], [ 106.692506224225951, 10.780040784110362 ], [ 106.692506224225951, 10.779360097123361 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.780040784110362 ], [ 106.693243261002294, 10.780000174018037 ], [ 106.693243261002294, 10.780680863704221 ], [ 106.692506224225951, 10.780721476238618 ], [ 106.692506224225951, 10.780040784110362 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.780000174018037 ], [ 106.693980300251837, 10.779959562036447 ], [ 106.693980300251837, 10.780640249280445 ], [ 106.693243261002294, 10.780680863704221 ], [ 106.693243261002294, 10.780000174018037 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.780680863704221 ], [ 106.693980300251837, 10.780640249280445 ], [ 106.693980300251837, 10.781320941665426 ], [ 106.693243261002294, 10.781361558531385 ], [ 106.693243261002294, 10.780680863704221 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.780721476238618 ], [ 106.693243261002294, 10.780680863704221 ], [ 106.693243261002294, 10.781361558531385 ], [ 106.692506224225951, 10.781402173507852 ], [ 106.692506224225951, 10.780721476238618 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.780762086883628 ], [ 106.692506224225951, 10.780721476238618 ], [ 106.692506224225951, 10.781402173507852 ], [ 106.691769189923207, 10.781442786594816 ], [ 106.691769189923207, 10.780762086883628 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.780081392313415 ], [ 106.692506224225951, 10.780040784110362 ], [ 106.692506224225951, 10.780721476238618 ], [ 106.691769189923207, 10.780762086883628 ], [ 106.691769189923207, 10.780081392313415 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.780121998627187 ], [ 106.691769189923207, 10.780081392313415 ], [ 106.691769189923207, 10.780762086883628 ], [ 106.691032158094345, 10.780802695639245 ], [ 106.691032158094345, 10.780121998627187 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f30fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.780802695639245 ], [ 106.691769189923207, 10.780762086883628 ], [ 106.691769189923207, 10.781442786594816 ], [ 106.691032158094345, 10.78148339779227 ], [ 106.691032158094345, 10.780802695639245 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3104", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.78148339779227 ], [ 106.691769189923207, 10.781442786594816 ], [ 106.691769189923207, 10.782123491446688 ], [ 106.691032158094345, 10.782164105085977 ], [ 106.691032158094345, 10.78148339779227 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f310c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.782164105085977 ], [ 106.691769189923207, 10.782123491446688 ], [ 106.691769189923207, 10.782804201438966 ], [ 106.691032158094345, 10.782844817520083 ], [ 106.691032158094345, 10.782164105085977 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3114", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.782123491446688 ], [ 106.692506224225951, 10.782082875917775 ], [ 106.692506224225951, 10.782763583468109 ], [ 106.691769189923207, 10.782804201438966 ], [ 106.691769189923207, 10.782123491446688 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f311c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.781442786594816 ], [ 106.692506224225951, 10.781402173507852 ], [ 106.692506224225951, 10.782082875917775 ], [ 106.691769189923207, 10.782123491446688 ], [ 106.691769189923207, 10.781442786594816 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3124", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.781402173507852 ], [ 106.693243261002294, 10.781361558531385 ], [ 106.693243261002294, 10.782042258499246 ], [ 106.692506224225951, 10.782082875917775 ], [ 106.692506224225951, 10.781402173507852 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f312c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.781361558531385 ], [ 106.693980300251837, 10.781320941665426 ], [ 106.693980300251837, 10.782001639191106 ], [ 106.693243261002294, 10.782042258499246 ], [ 106.693243261002294, 10.781361558531385 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3134", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.782042258499246 ], [ 106.693980300251837, 10.782001639191106 ], [ 106.693980300251837, 10.782682341857207 ], [ 106.693243261002294, 10.782722963607521 ], [ 106.693243261002294, 10.782042258499246 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f313c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.782082875917775 ], [ 106.693243261002294, 10.782042258499246 ], [ 106.693243261002294, 10.782722963607521 ], [ 106.692506224225951, 10.782763583468109 ], [ 106.692506224225951, 10.782082875917775 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3144", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.782763583468109 ], [ 106.693243261002294, 10.782722963607521 ], [ 106.693243261002294, 10.783403673855926 ], [ 106.692506224225951, 10.783444296158569 ], [ 106.692506224225951, 10.782763583468109 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f314c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.782722963607521 ], [ 106.693980300251837, 10.782682341857207 ], [ 106.693980300251837, 10.783363049663444 ], [ 106.693243261002294, 10.783403673855926 ], [ 106.693243261002294, 10.782722963607521 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3154", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.783403673855926 ], [ 106.693980300251837, 10.783363049663444 ], [ 106.693980300251837, 10.78404376260953 ], [ 106.693243261002294, 10.784084389244176 ], [ 106.693243261002294, 10.783403673855926 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f315c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.783444296158569 ], [ 106.693243261002294, 10.783403673855926 ], [ 106.693243261002294, 10.784084389244176 ], [ 106.692506224225951, 10.784125013988872 ], [ 106.692506224225951, 10.783444296158569 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3164", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.783484916571366 ], [ 106.692506224225951, 10.783444296158569 ], [ 106.692506224225951, 10.784125013988872 ], [ 106.691769189923207, 10.784165636843605 ], [ 106.691769189923207, 10.783484916571366 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f316c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.782804201438966 ], [ 106.692506224225951, 10.782763583468109 ], [ 106.692506224225951, 10.783444296158569 ], [ 106.691769189923207, 10.783484916571366 ], [ 106.691769189923207, 10.782804201438966 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3174", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.782844817520083 ], [ 106.691769189923207, 10.782804201438966 ], [ 106.691769189923207, 10.783484916571366 ], [ 106.691032158094345, 10.783525535094308 ], [ 106.691032158094345, 10.782844817520083 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f317c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.783525535094308 ], [ 106.691769189923207, 10.783484916571366 ], [ 106.691769189923207, 10.784165636843605 ], [ 106.691032158094345, 10.784206257808368 ], [ 106.691032158094345, 10.783525535094308 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3184", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.78356615172739 ], [ 106.691032158094345, 10.783525535094308 ], [ 106.691032158094345, 10.784206257808368 ], [ 106.690295128739763, 10.784246876883154 ], [ 106.690295128739763, 10.78356615172739 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f318c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.783606766470598 ], [ 106.690295128739763, 10.78356615172739 ], [ 106.690295128739763, 10.784246876883154 ], [ 106.689558101859916, 10.784287494067954 ], [ 106.689558101859916, 10.783606766470598 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3194", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.78292604401307 ], [ 106.690295128739763, 10.782885431711453 ], [ 106.690295128739763, 10.78356615172739 ], [ 106.689558101859916, 10.783606766470598 ], [ 106.689558101859916, 10.78292604401307 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f319c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.782885431711453 ], [ 106.691032158094345, 10.782844817520083 ], [ 106.691032158094345, 10.783525535094308 ], [ 106.690295128739763, 10.78356615172739 ], [ 106.690295128739763, 10.782885431711453 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.782204716835635 ], [ 106.691032158094345, 10.782164105085977 ], [ 106.691032158094345, 10.782844817520083 ], [ 106.690295128739763, 10.782885431711453 ], [ 106.690295128739763, 10.782204716835635 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.781524007100206 ], [ 106.691032158094345, 10.78148339779227 ], [ 106.691032158094345, 10.782164105085977 ], [ 106.690295128739763, 10.782204716835635 ], [ 106.690295128739763, 10.781524007100206 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.78156461451862 ], [ 106.690295128739763, 10.781524007100206 ], [ 106.690295128739763, 10.782204716835635 ], [ 106.689558101859916, 10.78224532669565 ], [ 106.689558101859916, 10.78156461451862 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.78224532669565 ], [ 106.690295128739763, 10.782204716835635 ], [ 106.690295128739763, 10.782885431711453 ], [ 106.689558101859916, 10.78292604401307 ], [ 106.689558101859916, 10.78224532669565 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.782285934666017 ], [ 106.689558101859916, 10.78224532669565 ], [ 106.689558101859916, 10.78292604401307 ], [ 106.688821077455032, 10.78296665442492 ], [ 106.688821077455032, 10.782285934666017 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.781605220047497 ], [ 106.689558101859916, 10.78156461451862 ], [ 106.689558101859916, 10.78224532669565 ], [ 106.688821077455032, 10.782285934666017 ], [ 106.688821077455032, 10.781605220047497 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.781645823686835 ], [ 106.688821077455032, 10.781605220047497 ], [ 106.688821077455032, 10.782285934666017 ], [ 106.688084055525565, 10.782326540746729 ], [ 106.688084055525565, 10.781645823686835 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.782326540746729 ], [ 106.688821077455032, 10.782285934666017 ], [ 106.688821077455032, 10.78296665442492 ], [ 106.688084055525565, 10.783007262947002 ], [ 106.688084055525565, 10.782326540746729 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.783007262947002 ], [ 106.688821077455032, 10.78296665442492 ], [ 106.688821077455032, 10.783647379323931 ], [ 106.688084055525565, 10.783687990287376 ], [ 106.688084055525565, 10.783007262947002 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.78296665442492 ], [ 106.689558101859916, 10.78292604401307 ], [ 106.689558101859916, 10.783606766470598 ], [ 106.688821077455032, 10.783647379323931 ], [ 106.688821077455032, 10.78296665442492 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.783647379323931 ], [ 106.689558101859916, 10.783606766470598 ], [ 106.689558101859916, 10.784287494067954 ], [ 106.688821077455032, 10.784328109362761 ], [ 106.688821077455032, 10.783647379323931 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.783687990287376 ], [ 106.688821077455032, 10.783647379323931 ], [ 106.688821077455032, 10.784328109362761 ], [ 106.688084055525565, 10.784368722767567 ], [ 106.688084055525565, 10.783687990287376 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3204", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.784368722767567 ], [ 106.688821077455032, 10.784328109362761 ], [ 106.688821077455032, 10.78500884454113 ], [ 106.688084055525565, 10.78504946038729 ], [ 106.688084055525565, 10.784368722767567 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f320c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.78504946038729 ], [ 106.688821077455032, 10.78500884454113 ], [ 106.688821077455032, 10.785689584858753 ], [ 106.688084055525565, 10.785730203146265 ], [ 106.688084055525565, 10.78504946038729 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3214", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.78500884454113 ], [ 106.689558101859916, 10.784968226804851 ], [ 106.689558101859916, 10.785648964681009 ], [ 106.688821077455032, 10.785689584858753 ], [ 106.688821077455032, 10.78500884454113 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f321c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.784328109362761 ], [ 106.689558101859916, 10.784287494067954 ], [ 106.689558101859916, 10.784968226804851 ], [ 106.688821077455032, 10.78500884454113 ], [ 106.688821077455032, 10.784328109362761 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3224", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.784287494067954 ], [ 106.690295128739763, 10.784246876883154 ], [ 106.690295128739763, 10.784927607178465 ], [ 106.689558101859916, 10.784968226804851 ], [ 106.689558101859916, 10.784287494067954 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f322c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.784246876883154 ], [ 106.691032158094345, 10.784206257808368 ], [ 106.691032158094345, 10.784886985661979 ], [ 106.690295128739763, 10.784927607178465 ], [ 106.690295128739763, 10.784246876883154 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3234", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.784927607178465 ], [ 106.691032158094345, 10.784886985661979 ], [ 106.691032158094345, 10.785567718654857 ], [ 106.690295128739763, 10.78560834261304 ], [ 106.690295128739763, 10.784927607178465 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f323c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.784968226804851 ], [ 106.690295128739763, 10.784927607178465 ], [ 106.690295128739763, 10.78560834261304 ], [ 106.689558101859916, 10.785648964681009 ], [ 106.689558101859916, 10.784968226804851 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3244", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.785648964681009 ], [ 106.690295128739763, 10.78560834261304 ], [ 106.690295128739763, 10.786289083186599 ], [ 106.689558101859916, 10.786329707696142 ], [ 106.689558101859916, 10.785648964681009 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f324c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.78560834261304 ], [ 106.691032158094345, 10.785567718654857 ], [ 106.691032158094345, 10.786248456786721 ], [ 106.690295128739763, 10.786289083186599 ], [ 106.690295128739763, 10.78560834261304 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3254", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.786289083186599 ], [ 106.691032158094345, 10.786248456786721 ], [ 106.691032158094345, 10.786929200057287 ], [ 106.690295128739763, 10.786969828898853 ], [ 106.690295128739763, 10.786289083186599 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f325c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.786329707696142 ], [ 106.690295128739763, 10.786289083186599 ], [ 106.690295128739763, 10.786969828898853 ], [ 106.689558101859916, 10.787010455849968 ], [ 106.689558101859916, 10.786329707696142 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3264", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.786370330315348 ], [ 106.689558101859916, 10.786329707696142 ], [ 106.689558101859916, 10.787010455849968 ], [ 106.688821077455032, 10.787051080910633 ], [ 106.688821077455032, 10.786370330315348 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f326c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.785689584858753 ], [ 106.689558101859916, 10.785648964681009 ], [ 106.689558101859916, 10.786329707696142 ], [ 106.688821077455032, 10.786370330315348 ], [ 106.688821077455032, 10.785689584858753 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3274", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.785730203146265 ], [ 106.688821077455032, 10.785689584858753 ], [ 106.688821077455032, 10.786370330315348 ], [ 106.688084055525565, 10.786410951044209 ], [ 106.688084055525565, 10.785730203146265 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f327c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.786410951044209 ], [ 106.688821077455032, 10.786370330315348 ], [ 106.688821077455032, 10.787051080910633 ], [ 106.688084055525565, 10.787091704080835 ], [ 106.688084055525565, 10.786410951044209 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3284", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.787091704080835 ], [ 106.688821077455032, 10.787051080910633 ], [ 106.688821077455032, 10.787731836644324 ], [ 106.688084055525565, 10.787772462255866 ], [ 106.688084055525565, 10.787091704080835 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f328c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.787051080910633 ], [ 106.689558101859916, 10.787010455849968 ], [ 106.689558101859916, 10.787691209142208 ], [ 106.688821077455032, 10.787731836644324 ], [ 106.688821077455032, 10.787051080910633 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3294", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.787731836644324 ], [ 106.689558101859916, 10.787691209142208 ], [ 106.689558101859916, 10.788371967572573 ], [ 106.688821077455032, 10.788412597516139 ], [ 106.688821077455032, 10.787731836644324 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f329c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.787772462255866 ], [ 106.688821077455032, 10.787731836644324 ], [ 106.688821077455032, 10.788412597516139 ], [ 106.688084055525565, 10.788453225569011 ], [ 106.688084055525565, 10.787772462255866 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.788453225569011 ], [ 106.688821077455032, 10.788412597516139 ], [ 106.688821077455032, 10.789093363525794 ], [ 106.688084055525565, 10.789133994019995 ], [ 106.688084055525565, 10.788453225569011 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.789133994019995 ], [ 106.688821077455032, 10.789093363525794 ], [ 106.688821077455032, 10.789774134673007 ], [ 106.688084055525565, 10.789814767608535 ], [ 106.688084055525565, 10.789133994019995 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.789093363525794 ], [ 106.689558101859916, 10.789052731140783 ], [ 106.689558101859916, 10.789733499846557 ], [ 106.688821077455032, 10.789774134673007 ], [ 106.688821077455032, 10.789093363525794 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.788412597516139 ], [ 106.689558101859916, 10.788371967572573 ], [ 106.689558101859916, 10.789052731140783 ], [ 106.688821077455032, 10.789093363525794 ], [ 106.688821077455032, 10.788412597516139 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.788371967572573 ], [ 106.690295128739763, 10.788331335738325 ], [ 106.690295128739763, 10.789012096864976 ], [ 106.689558101859916, 10.789052731140783 ], [ 106.689558101859916, 10.788371967572573 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.789052731140783 ], [ 106.690295128739763, 10.789012096864976 ], [ 106.690295128739763, 10.789692863129195 ], [ 106.689558101859916, 10.789733499846557 ], [ 106.689558101859916, 10.789052731140783 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.789012096864976 ], [ 106.691032158094345, 10.788971460698379 ], [ 106.691032158094345, 10.78965222452093 ], [ 106.690295128739763, 10.789692863129195 ], [ 106.690295128739763, 10.789012096864976 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.788331335738325 ], [ 106.691032158094345, 10.7882907020134 ], [ 106.691032158094345, 10.788971460698379 ], [ 106.690295128739763, 10.789012096864976 ], [ 106.690295128739763, 10.788331335738325 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.787650579749522 ], [ 106.691032158094345, 10.787609948466276 ], [ 106.691032158094345, 10.7882907020134 ], [ 106.690295128739763, 10.788331335738325 ], [ 106.690295128739763, 10.787650579749522 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.787691209142208 ], [ 106.690295128739763, 10.787650579749522 ], [ 106.690295128739763, 10.788331335738325 ], [ 106.689558101859916, 10.788371967572573 ], [ 106.689558101859916, 10.787691209142208 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.787010455849968 ], [ 106.690295128739763, 10.786969828898853 ], [ 106.690295128739763, 10.787650579749522 ], [ 106.689558101859916, 10.787691209142208 ], [ 106.689558101859916, 10.787010455849968 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.786969828898853 ], [ 106.691032158094345, 10.786929200057287 ], [ 106.691032158094345, 10.787609948466276 ], [ 106.690295128739763, 10.787650579749522 ], [ 106.690295128739763, 10.786969828898853 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3304", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.786929200057287 ], [ 106.691769189923207, 10.786888569325289 ], [ 106.691769189923207, 10.787569315292478 ], [ 106.691032158094345, 10.787609948466276 ], [ 106.691032158094345, 10.786929200057287 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f330c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.786888569325289 ], [ 106.692506224225951, 10.786847936702861 ], [ 106.692506224225951, 10.787528680228139 ], [ 106.691769189923207, 10.787569315292478 ], [ 106.691769189923207, 10.786888569325289 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3314", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.787569315292478 ], [ 106.692506224225951, 10.787528680228139 ], [ 106.692506224225951, 10.788209428891561 ], [ 106.691769189923207, 10.788250066397811 ], [ 106.691769189923207, 10.787569315292478 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f331c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.787609948466276 ], [ 106.691769189923207, 10.787569315292478 ], [ 106.691769189923207, 10.788250066397811 ], [ 106.691032158094345, 10.7882907020134 ], [ 106.691032158094345, 10.787609948466276 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3324", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.7882907020134 ], [ 106.691769189923207, 10.788250066397811 ], [ 106.691769189923207, 10.788930822641001 ], [ 106.691032158094345, 10.788971460698379 ], [ 106.691032158094345, 10.7882907020134 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3374", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.786847936702861 ], [ 106.693243261002294, 10.786807302190011 ], [ 106.693243261002294, 10.78748804327326 ], [ 106.692506224225951, 10.787528680228139 ], [ 106.692506224225951, 10.786847936702861 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f337c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.786807302190011 ], [ 106.693980300251837, 10.786766665786745 ], [ 106.693980300251837, 10.787447404427851 ], [ 106.693243261002294, 10.78748804327326 ], [ 106.693243261002294, 10.786807302190011 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3384", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.786126566245196 ], [ 106.693980300251837, 10.786085932284077 ], [ 106.693980300251837, 10.786766665786745 ], [ 106.693243261002294, 10.786807302190011 ], [ 106.693243261002294, 10.786126566245196 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f338c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.785445835439097 ], [ 106.693980300251837, 10.785405203920131 ], [ 106.693980300251837, 10.786085932284077 ], [ 106.693243261002294, 10.786126566245196 ], [ 106.693243261002294, 10.785445835439097 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3394", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.785486465067878 ], [ 106.693243261002294, 10.785445835439097 ], [ 106.693243261002294, 10.786126566245196 ], [ 106.692506224225951, 10.786167198316013 ], [ 106.692506224225951, 10.785486465067878 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f339c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.786167198316013 ], [ 106.693243261002294, 10.786126566245196 ], [ 106.693243261002294, 10.786807302190011 ], [ 106.692506224225951, 10.786847936702861 ], [ 106.692506224225951, 10.786167198316013 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.786207828496526 ], [ 106.692506224225951, 10.786167198316013 ], [ 106.692506224225951, 10.786847936702861 ], [ 106.691769189923207, 10.786888569325289 ], [ 106.691769189923207, 10.786207828496526 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.786248456786721 ], [ 106.691769189923207, 10.786207828496526 ], [ 106.691769189923207, 10.786888569325289 ], [ 106.691032158094345, 10.786929200057287 ], [ 106.691032158094345, 10.786248456786721 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.785567718654857 ], [ 106.691769189923207, 10.785527092806468 ], [ 106.691769189923207, 10.786207828496526 ], [ 106.691032158094345, 10.786248456786721 ], [ 106.691032158094345, 10.785567718654857 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.785527092806468 ], [ 106.692506224225951, 10.785486465067878 ], [ 106.692506224225951, 10.786167198316013 ], [ 106.691769189923207, 10.786207828496526 ], [ 106.691769189923207, 10.785527092806468 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.784846362255399 ], [ 106.692506224225951, 10.784805736958736 ], [ 106.692506224225951, 10.785486465067878 ], [ 106.691769189923207, 10.785527092806468 ], [ 106.691769189923207, 10.784846362255399 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.784886985661979 ], [ 106.691769189923207, 10.784846362255399 ], [ 106.691769189923207, 10.785527092806468 ], [ 106.691032158094345, 10.785567718654857 ], [ 106.691032158094345, 10.784886985661979 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.784206257808368 ], [ 106.691769189923207, 10.784165636843605 ], [ 106.691769189923207, 10.784846362255399 ], [ 106.691032158094345, 10.784886985661979 ], [ 106.691032158094345, 10.784206257808368 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.784165636843605 ], [ 106.692506224225951, 10.784125013988872 ], [ 106.692506224225951, 10.784805736958736 ], [ 106.691769189923207, 10.784846362255399 ], [ 106.691769189923207, 10.784165636843605 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.784125013988872 ], [ 106.693243261002294, 10.784084389244176 ], [ 106.693243261002294, 10.784765109771996 ], [ 106.692506224225951, 10.784805736958736 ], [ 106.692506224225951, 10.784125013988872 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.784805736958736 ], [ 106.693243261002294, 10.784765109771996 ], [ 106.693243261002294, 10.785445835439097 ], [ 106.692506224225951, 10.785486465067878 ], [ 106.692506224225951, 10.784805736958736 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.784765109771996 ], [ 106.693980300251837, 10.784724480695187 ], [ 106.693980300251837, 10.785405203920131 ], [ 106.693243261002294, 10.785445835439097 ], [ 106.693243261002294, 10.784765109771996 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.784084389244176 ], [ 106.693980300251837, 10.78404376260953 ], [ 106.693980300251837, 10.784724480695187 ], [ 106.693243261002294, 10.784765109771996 ], [ 106.693243261002294, 10.784084389244176 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3404", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.78404376260953 ], [ 106.694717341974183, 10.784003134084935 ], [ 106.694717341974183, 10.784683849728317 ], [ 106.693980300251837, 10.784724480695187 ], [ 106.693980300251837, 10.78404376260953 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f340c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.784724480695187 ], [ 106.694717341974183, 10.784683849728317 ], [ 106.694717341974183, 10.785364570510989 ], [ 106.693980300251837, 10.785405203920131 ], [ 106.693980300251837, 10.784724480695187 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3414", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.784683849728317 ], [ 106.695454386168933, 10.784643216871395 ], [ 106.695454386168933, 10.785323935211682 ], [ 106.694717341974183, 10.785364570510989 ], [ 106.694717341974183, 10.784683849728317 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f341c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.784003134084935 ], [ 106.695454386168933, 10.783962503670406 ], [ 106.695454386168933, 10.784643216871395 ], [ 106.694717341974183, 10.784683849728317 ], [ 106.694717341974183, 10.784003134084935 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3424", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.783962503670406 ], [ 106.69619143283586, 10.783921871365944 ], [ 106.69619143283586, 10.784602582124428 ], [ 106.695454386168933, 10.784643216871395 ], [ 106.695454386168933, 10.783962503670406 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f342c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.69619143283586, 10.783921871365944 ], [ 106.696928481974453, 10.783881237171563 ], [ 106.696928481974453, 10.784561945487425 ], [ 106.69619143283586, 10.784602582124428 ], [ 106.69619143283586, 10.783921871365944 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f343c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.784643216871395 ], [ 106.69619143283586, 10.784602582124428 ], [ 106.69619143283586, 10.785283298022215 ], [ 106.695454386168933, 10.785323935211682 ], [ 106.695454386168933, 10.784643216871395 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f346c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.785364570510989 ], [ 106.695454386168933, 10.785323935211682 ], [ 106.695454386168933, 10.786004658690983 ], [ 106.694717341974183, 10.78604529643267 ], [ 106.694717341974183, 10.785364570510989 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3474", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.785405203920131 ], [ 106.694717341974183, 10.785364570510989 ], [ 106.694717341974183, 10.78604529643267 ], [ 106.693980300251837, 10.786085932284077 ], [ 106.693980300251837, 10.785405203920131 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f347c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.786085932284077 ], [ 106.694717341974183, 10.78604529643267 ], [ 106.694717341974183, 10.786726027493076 ], [ 106.693980300251837, 10.786766665786745 ], [ 106.693980300251837, 10.786085932284077 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f35d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.696928481974453, 10.783881237171563 ], [ 106.697665533584427, 10.783840601087265 ], [ 106.697665533584427, 10.784521306960389 ], [ 106.696928481974453, 10.784561945487425 ], [ 106.696928481974453, 10.783881237171563 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3644", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.697665533584427, 10.781798514306562 ], [ 106.698402587665328, 10.781757883660939 ], [ 106.698402587665328, 10.782438571671577 ], [ 106.697665533584427, 10.782479204760067 ], [ 106.697665533584427, 10.781798514306562 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3654", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.696928481974453, 10.781158455306807 ], [ 106.697665533584427, 10.7811178289935 ], [ 106.697665533584427, 10.781798514306562 ], [ 106.696928481974453, 10.781839143062625 ], [ 106.696928481974453, 10.781158455306807 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f365c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.696928481974453, 10.781839143062625 ], [ 106.697665533584427, 10.781798514306562 ], [ 106.697665533584427, 10.782479204760067 ], [ 106.696928481974453, 10.782519835958878 ], [ 106.696928481974453, 10.781839143062625 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3664", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.696928481974453, 10.782519835958878 ], [ 106.697665533584427, 10.782479204760067 ], [ 106.697665533584427, 10.783159900353727 ], [ 106.696928481974453, 10.783200533995286 ], [ 106.696928481974453, 10.782519835958878 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f366c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.697665533584427, 10.782479204760067 ], [ 106.698402587665328, 10.782438571671577 ], [ 106.698402587665385, 10.783119264822378 ], [ 106.697665533584427, 10.783159900353727 ], [ 106.697665533584427, 10.782479204760067 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3674", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.697665533584427, 10.783159900353727 ], [ 106.698402587665385, 10.783119264822378 ], [ 106.698402587665385, 10.78379996311306 ], [ 106.697665533584427, 10.783840601087265 ], [ 106.697665533584427, 10.783159900353727 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f367c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.696928481974453, 10.783200533995286 ], [ 106.697665533584427, 10.783159900353727 ], [ 106.697665533584427, 10.783840601087265 ], [ 106.696928481974453, 10.783881237171563 ], [ 106.696928481974453, 10.783200533995286 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3684", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.69619143283586, 10.783241165747043 ], [ 106.696928481974453, 10.783200533995286 ], [ 106.696928481974453, 10.783881237171563 ], [ 106.69619143283586, 10.783921871365944 ], [ 106.69619143283586, 10.783241165747043 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f368c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.69619143283586, 10.782560465268007 ], [ 106.696928481974453, 10.782519835958878 ], [ 106.696928481974453, 10.783200533995286 ], [ 106.69619143283586, 10.783241165747043 ], [ 106.69619143283586, 10.782560465268007 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3694", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.78260109268744 ], [ 106.69619143283586, 10.782560465268007 ], [ 106.69619143283586, 10.783241165747043 ], [ 106.695454386168933, 10.783281795608993 ], [ 106.695454386168933, 10.78260109268744 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f369c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.783281795608993 ], [ 106.69619143283586, 10.783241165747043 ], [ 106.69619143283586, 10.783921871365944 ], [ 106.695454386168933, 10.783962503670406 ], [ 106.695454386168933, 10.783281795608993 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36a4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.783322423581129 ], [ 106.695454386168933, 10.783281795608993 ], [ 106.695454386168933, 10.783962503670406 ], [ 106.694717341974183, 10.784003134084935 ], [ 106.694717341974183, 10.783322423581129 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36ac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.783363049663444 ], [ 106.694717341974183, 10.783322423581129 ], [ 106.694717341974183, 10.784003134084935 ], [ 106.693980300251837, 10.78404376260953 ], [ 106.693980300251837, 10.783363049663444 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36b4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.782682341857207 ], [ 106.694717341974183, 10.78264171821718 ], [ 106.694717341974183, 10.783322423581129 ], [ 106.693980300251837, 10.783363049663444 ], [ 106.693980300251837, 10.782682341857207 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36bc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.78264171821718 ], [ 106.695454386168933, 10.78260109268744 ], [ 106.695454386168933, 10.783281795608993 ], [ 106.694717341974183, 10.783322423581129 ], [ 106.694717341974183, 10.78264171821718 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36c4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.781961017993369 ], [ 106.695454386168933, 10.781920394906036 ], [ 106.695454386168933, 10.78260109268744 ], [ 106.694717341974183, 10.78264171821718 ], [ 106.694717341974183, 10.781961017993369 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36cc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.782001639191106 ], [ 106.694717341974183, 10.781961017993369 ], [ 106.694717341974183, 10.78264171821718 ], [ 106.693980300251837, 10.782682341857207 ], [ 106.693980300251837, 10.782001639191106 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36d4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.781320941665426 ], [ 106.694717341974183, 10.781280322909979 ], [ 106.694717341974183, 10.781961017993369 ], [ 106.693980300251837, 10.782001639191106 ], [ 106.693980300251837, 10.781320941665426 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36dc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.781280322909979 ], [ 106.695454386168933, 10.781239702265056 ], [ 106.695454386168933, 10.781920394906036 ], [ 106.694717341974183, 10.781961017993369 ], [ 106.694717341974183, 10.781280322909979 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36e4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.781239702265056 ], [ 106.69619143283586, 10.781199079730662 ], [ 106.69619143283586, 10.78187976992912 ], [ 106.695454386168933, 10.781920394906036 ], [ 106.695454386168933, 10.781239702265056 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36ec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.781920394906036 ], [ 106.69619143283586, 10.78187976992912 ], [ 106.69619143283586, 10.782560465268007 ], [ 106.695454386168933, 10.78260109268744 ], [ 106.695454386168933, 10.781920394906036 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36f4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.69619143283586, 10.78187976992912 ], [ 106.696928481974453, 10.781839143062625 ], [ 106.696928481974453, 10.782519835958878 ], [ 106.69619143283586, 10.782560465268007 ], [ 106.69619143283586, 10.78187976992912 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.69619143283586, 10.781199079730662 ], [ 106.696928481974453, 10.781158455306807 ], [ 106.696928481974453, 10.781839143062625 ], [ 106.69619143283586, 10.78187976992912 ], [ 106.69619143283586, 10.781199079730662 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3704", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.69619143283586, 10.780518394672923 ], [ 106.696928481974453, 10.780477772691709 ], [ 106.696928481974453, 10.781158455306807 ], [ 106.69619143283586, 10.781199079730662 ], [ 106.69619143283586, 10.780518394672923 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3714", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.779878332405509 ], [ 106.69619143283586, 10.779837714756178 ], [ 106.69619143283586, 10.780518394672923 ], [ 106.695454386168933, 10.780559014764787 ], [ 106.695454386168933, 10.779878332405509 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f371c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.780559014764787 ], [ 106.69619143283586, 10.780518394672923 ], [ 106.69619143283586, 10.781199079730662 ], [ 106.695454386168933, 10.781239702265056 ], [ 106.695454386168933, 10.780559014764787 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3724", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.780599632967299 ], [ 106.695454386168933, 10.780559014764787 ], [ 106.695454386168933, 10.781239702265056 ], [ 106.694717341974183, 10.781280322909979 ], [ 106.694717341974183, 10.780599632967299 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f372c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.780640249280445 ], [ 106.694717341974183, 10.780599632967299 ], [ 106.694717341974183, 10.781280322909979 ], [ 106.693980300251837, 10.781320941665426 ], [ 106.693980300251837, 10.780640249280445 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3734", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.779959562036447 ], [ 106.694717341974183, 10.779918948165603 ], [ 106.694717341974183, 10.780599632967299 ], [ 106.693980300251837, 10.780640249280445 ], [ 106.693980300251837, 10.779959562036447 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f373c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.779918948165603 ], [ 106.695454386168933, 10.779878332405509 ], [ 106.695454386168933, 10.780559014764787 ], [ 106.694717341974183, 10.780599632967299 ], [ 106.694717341974183, 10.779918948165603 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3744", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.77923826850518 ], [ 106.695454386168933, 10.779197655187509 ], [ 106.695454386168933, 10.779878332405509 ], [ 106.694717341974183, 10.779918948165603 ], [ 106.694717341974183, 10.77923826850518 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f374c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.779278879933718 ], [ 106.694717341974183, 10.77923826850518 ], [ 106.694717341974183, 10.779918948165603 ], [ 106.693980300251837, 10.779959562036447 ], [ 106.693980300251837, 10.779278879933718 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3754", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.778598202972537 ], [ 106.694717341974183, 10.77855759398631 ], [ 106.694717341974183, 10.77923826850518 ], [ 106.693980300251837, 10.779278879933718 ], [ 106.693980300251837, 10.778598202972537 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f39fc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.777917531153186 ], [ 106.694717341974183, 10.777876924609277 ], [ 106.694717341974183, 10.77855759398631 ], [ 106.693980300251837, 10.778598202972537 ], [ 106.693980300251837, 10.777917531153186 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a04", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.777958135808186 ], [ 106.693980300251837, 10.777917531153186 ], [ 106.693980300251837, 10.778598202972537 ], [ 106.693243261002294, 10.778638810069735 ], [ 106.693243261002294, 10.777958135808186 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a0c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.777998738574265 ], [ 106.693243261002294, 10.777958135808186 ], [ 106.693243261002294, 10.778638810069735 ], [ 106.692506224225951, 10.778679415277901 ], [ 106.692506224225951, 10.777998738574265 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a14", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.777318067012734 ], [ 106.693243261002294, 10.777277466688746 ], [ 106.693243261002294, 10.777958135808186 ], [ 106.692506224225951, 10.777998738574265 ], [ 106.692506224225951, 10.777318067012734 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a1c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.777277466688746 ], [ 106.693980300251837, 10.777236864475952 ], [ 106.693980300251837, 10.777917531153186 ], [ 106.693243261002294, 10.777958135808186 ], [ 106.693243261002294, 10.777277466688746 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a3c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.776637400593593 ], [ 106.693243261002294, 10.776596802711698 ], [ 106.693243261002294, 10.777277466688746 ], [ 106.692506224225951, 10.777318067012734 ], [ 106.692506224225951, 10.776637400593593 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a44", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.776677996586789 ], [ 106.692506224225951, 10.776637400593593 ], [ 106.692506224225951, 10.777318067012734 ], [ 106.691769189923207, 10.77735866544791 ], [ 106.691769189923207, 10.776677996586789 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.775997332868334 ], [ 106.692506224225951, 10.775956739317124 ], [ 106.692506224225951, 10.776637400593593 ], [ 106.691769189923207, 10.776677996586789 ], [ 106.691769189923207, 10.775997332868334 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.776037924530955 ], [ 106.691769189923207, 10.775997332868334 ], [ 106.691769189923207, 10.776677996586789 ], [ 106.691032158094345, 10.776718590691281 ], [ 106.691032158094345, 10.776037924530955 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.776718590691281 ], [ 106.691769189923207, 10.776677996586789 ], [ 106.691769189923207, 10.77735866544791 ], [ 106.691032158094345, 10.777399261994269 ], [ 106.691032158094345, 10.776718590691281 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a64", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.777399261994269 ], [ 106.691769189923207, 10.77735866544791 ], [ 106.691769189923207, 10.778039339451416 ], [ 106.691032158094345, 10.778079938439635 ], [ 106.691032158094345, 10.777399261994269 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a6c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.77735866544791 ], [ 106.692506224225951, 10.777318067012734 ], [ 106.692506224225951, 10.777998738574265 ], [ 106.691769189923207, 10.778039339451416 ], [ 106.691769189923207, 10.77735866544791 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a74", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691769189923207, 10.778039339451416 ], [ 106.692506224225951, 10.777998738574265 ], [ 106.692506224225951, 10.778679415277901 ], [ 106.691769189923207, 10.778720018597026 ], [ 106.691769189923207, 10.778039339451416 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a7c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.778079938439635 ], [ 106.691769189923207, 10.778039339451416 ], [ 106.691769189923207, 10.778720018597026 ], [ 106.691032158094345, 10.778760620027098 ], [ 106.691032158094345, 10.778079938439635 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a84", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.778120535538912 ], [ 106.691032158094345, 10.778079938439635 ], [ 106.691032158094345, 10.778760620027098 ], [ 106.690295128739763, 10.778801219568116 ], [ 106.690295128739763, 10.778120535538912 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a8c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.777439856651798 ], [ 106.691032158094345, 10.777399261994269 ], [ 106.691032158094345, 10.778079938439635 ], [ 106.690295128739763, 10.778120535538912 ], [ 106.690295128739763, 10.777439856651798 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a94", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.777480449420493 ], [ 106.690295128739763, 10.777439856651798 ], [ 106.690295128739763, 10.778120535538912 ], [ 106.689558101859916, 10.778161130749236 ], [ 106.689558101859916, 10.777480449420493 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a9c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.778161130749236 ], [ 106.690295128739763, 10.778120535538912 ], [ 106.690295128739763, 10.778801219568116 ], [ 106.689558101859916, 10.778841817220068 ], [ 106.689558101859916, 10.778161130749236 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3aa4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.778201724070604 ], [ 106.689558101859916, 10.778161130749236 ], [ 106.689558101859916, 10.778841817220068 ], [ 106.688821077455032, 10.778882412982945 ], [ 106.688821077455032, 10.778201724070604 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3aac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.778242315503004 ], [ 106.688821077455032, 10.778201724070604 ], [ 106.688821077455032, 10.778882412982945 ], [ 106.688084055525565, 10.778923006856742 ], [ 106.688084055525565, 10.778242315503004 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3ab4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.777561629291343 ], [ 106.688821077455032, 10.777521040300343 ], [ 106.688821077455032, 10.778201724070604 ], [ 106.688084055525565, 10.778242315503004 ], [ 106.688084055525565, 10.777561629291343 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3abc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.777521040300343 ], [ 106.689558101859916, 10.777480449420493 ], [ 106.689558101859916, 10.778161130749236 ], [ 106.688821077455032, 10.778201724070604 ], [ 106.688821077455032, 10.777521040300343 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3ac4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.77684036167245 ], [ 106.689558101859916, 10.776799773234119 ], [ 106.689558101859916, 10.777480449420493 ], [ 106.688821077455032, 10.777521040300343 ], [ 106.688821077455032, 10.77684036167245 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3acc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.776880948222047 ], [ 106.688821077455032, 10.77684036167245 ], [ 106.688821077455032, 10.777521040300343 ], [ 106.688084055525565, 10.777561629291343 ], [ 106.688084055525565, 10.776880948222047 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3ad4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.776200272295393 ], [ 106.688821077455032, 10.776159688187208 ], [ 106.688821077455032, 10.77684036167245 ], [ 106.688084055525565, 10.776880948222047 ], [ 106.688084055525565, 10.776200272295393 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3adc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.776159688187208 ], [ 106.689558101859916, 10.776119102190403 ], [ 106.689558101859916, 10.776799773234119 ], [ 106.688821077455032, 10.77684036167245 ], [ 106.688821077455032, 10.776159688187208 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3ae4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.776119102190403 ], [ 106.690295128739763, 10.776078514304983 ], [ 106.690295128739763, 10.77675918290706 ], [ 106.689558101859916, 10.776799773234119 ], [ 106.689558101859916, 10.776119102190403 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3aec", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.776799773234119 ], [ 106.690295128739763, 10.77675918290706 ], [ 106.690295128739763, 10.777439856651798 ], [ 106.689558101859916, 10.777480449420493 ], [ 106.689558101859916, 10.776799773234119 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3af4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.77675918290706 ], [ 106.691032158094345, 10.776718590691281 ], [ 106.691032158094345, 10.777399261994269 ], [ 106.690295128739763, 10.777439856651798 ], [ 106.690295128739763, 10.77675918290706 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3afc", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.776078514304983 ], [ 106.691032158094345, 10.776037924530955 ], [ 106.691032158094345, 10.776718590691281 ], [ 106.690295128739763, 10.77675918290706 ], [ 106.690295128739763, 10.776078514304983 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b04", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.775397850845843 ], [ 106.691032158094345, 10.775357263513579 ], [ 106.691032158094345, 10.776037924530955 ], [ 106.690295128739763, 10.776078514304983 ], [ 106.690295128739763, 10.775397850845843 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b0c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.774717192529929 ], [ 106.691032158094345, 10.774676607639426 ], [ 106.691032158094345, 10.775357263513579 ], [ 106.690295128739763, 10.775397850845843 ], [ 106.690295128739763, 10.774717192529929 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b14", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.774757775532056 ], [ 106.690295128739763, 10.774717192529929 ], [ 106.690295128739763, 10.775397850845843 ], [ 106.689558101859916, 10.775438436289621 ], [ 106.689558101859916, 10.774757775532056 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b1c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.775438436289621 ], [ 106.690295128739763, 10.775397850845843 ], [ 106.690295128739763, 10.776078514304983 ], [ 106.689558101859916, 10.776119102190403 ], [ 106.689558101859916, 10.775438436289621 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b24", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.775479019844896 ], [ 106.689558101859916, 10.775438436289621 ], [ 106.689558101859916, 10.776119102190403 ], [ 106.688821077455032, 10.776159688187208 ], [ 106.688821077455032, 10.775479019844896 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b2c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.775519601511666 ], [ 106.688821077455032, 10.775479019844896 ], [ 106.688821077455032, 10.776159688187208 ], [ 106.688084055525565, 10.776200272295393 ], [ 106.688084055525565, 10.775519601511666 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b34", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.774838935871152 ], [ 106.688821077455032, 10.774798356645798 ], [ 106.688821077455032, 10.775479019844896 ], [ 106.688084055525565, 10.775519601511666 ], [ 106.688084055525565, 10.774838935871152 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b3c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.774798356645798 ], [ 106.689558101859916, 10.774757775532056 ], [ 106.689558101859916, 10.775438436289621 ], [ 106.688821077455032, 10.775479019844896 ], [ 106.688821077455032, 10.774798356645798 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b44", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.774117698590199 ], [ 106.689558101859916, 10.774077119917996 ], [ 106.689558101859916, 10.774757775532056 ], [ 106.688821077455032, 10.774798356645798 ], [ 106.688821077455032, 10.774117698590199 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b4c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.774158275374129 ], [ 106.688821077455032, 10.774117698590199 ], [ 106.688821077455032, 10.774798356645798 ], [ 106.688084055525565, 10.774838935871152 ], [ 106.688084055525565, 10.774158275374129 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b54", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.773477620020882 ], [ 106.688821077455032, 10.773437045678383 ], [ 106.688821077455032, 10.774117698590199 ], [ 106.688084055525565, 10.774158275374129 ], [ 106.688084055525565, 10.773477620020882 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b5c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.773437045678383 ], [ 106.689558101859916, 10.773396469447718 ], [ 106.689558101859916, 10.774077119917996 ], [ 106.688821077455032, 10.774117698590199 ], [ 106.688821077455032, 10.773437045678383 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b64", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.773396469447718 ], [ 106.690295128739763, 10.773355891328903 ], [ 106.690295128739763, 10.774036539357521 ], [ 106.689558101859916, 10.774077119917996 ], [ 106.689558101859916, 10.773396469447718 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b6c", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.774077119917996 ], [ 106.690295128739763, 10.774036539357521 ], [ 106.690295128739763, 10.774717192529929 ], [ 106.689558101859916, 10.774757775532056 ], [ 106.689558101859916, 10.774077119917996 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b74", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.774036539357521 ], [ 106.691032158094345, 10.773995956908788 ], [ 106.691032158094345, 10.774676607639426 ], [ 106.690295128739763, 10.774717192529929 ], [ 106.690295128739763, 10.774036539357521 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3ba4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.774676607639426 ], [ 106.691769189923207, 10.774636020860557 ], [ 106.691769189923207, 10.77531667429283 ], [ 106.691032158094345, 10.775357263513579 ], [ 106.691032158094345, 10.774676607639426 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3bac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.775357263513579 ], [ 106.691769189923207, 10.77531667429283 ], [ 106.691769189923207, 10.775997332868334 ], [ 106.691032158094345, 10.776037924530955 ], [ 106.691032158094345, 10.775357263513579 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3ca4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.772756397910626 ], [ 106.689558101859916, 10.772715824121512 ], [ 106.689558101859916, 10.773396469447718 ], [ 106.688821077455032, 10.773437045678383 ], [ 106.688821077455032, 10.772756397910626 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3cac", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.772796969811695 ], [ 106.688821077455032, 10.772756397910626 ], [ 106.688821077455032, 10.773437045678383 ], [ 106.688084055525565, 10.773477620020882 ], [ 106.688084055525565, 10.772796969811695 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3cb4", "resolution": 17 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.772116324746849 ], [ 106.688821077455032, 10.772075755287215 ], [ 106.688821077455032, 10.772756397910626 ], [ 106.688084055525565, 10.772796969811695 ], [ 106.688084055525565, 10.772116324746849 ] ] ] } }
]
}
```

- **Compact:**
```geojson
{
"type": "FeatureCollection",
"name": "s2_17_compact",
"crs": { "type": "name", "properties": { "name": "urn:ogc:def:crs:OGC:1.3:CRS84" } },
"features": [
{ "type": "Feature", "properties": { "s2": "317528cd34", "resolution": 17, "center_lat": 10.7900536, "center_lon": 106.6899266, "avg_edge_len": 78.016, "cell_area": 6069.703 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.789733499846557 ], [ 106.690295128739763, 10.789692863129195 ], [ 106.690295128739763, 10.790373634530699 ], [ 106.689558101859916, 10.790414273689612 ], [ 106.689558101859916, 10.789733499846557 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528cd4c", "resolution": 17, "center_lat": 10.7900942, "center_lon": 106.6891896, "avg_edge_len": 78.016, "cell_area": 6069.703 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.789774134673007 ], [ 106.689558101859916, 10.789733499846557 ], [ 106.689558101859916, 10.790414273689612 ], [ 106.688821077455032, 10.790454910957493 ], [ 106.688821077455032, 10.789774134673007 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528cd54", "resolution": 17, "center_lat": 10.7901348, "center_lon": 106.6884526, "avg_edge_len": 78.016, "cell_area": 6069.704 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.789814767608535 ], [ 106.688821077455032, 10.789774134673007 ], [ 106.688821077455032, 10.790454910957493 ], [ 106.688084055525565, 10.790495546334341 ], [ 106.688084055525565, 10.789814767608535 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528cd5c", "resolution": 17, "center_lat": 10.7908156, "center_lon": 106.6884526, "avg_edge_len": 78.016, "cell_area": 6069.736 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.790495546334341 ], [ 106.688821077455032, 10.790454910957493 ], [ 106.688821077455032, 10.791135692378973 ], [ 106.688084055525565, 10.791176330197136 ], [ 106.688084055525565, 10.790495546334341 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d28c", "resolution": 17, "center_lat": 10.7922585, "center_lon": 106.6869785, "avg_edge_len": 78.017, "cell_area": 6069.802 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.791938394041752 ], [ 106.687347036071799, 10.791897757564833 ], [ 106.687347036071799, 10.792578554141945 ], [ 106.686610019094246, 10.79261919305994 ], [ 106.686610019094246, 10.791938394041752 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d294", "resolution": 17, "center_lat": 10.7915777, "center_lon": 106.6869785, "avg_edge_len": 78.016, "cell_area": 6069.77 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.791257600159977 ], [ 106.687347036071799, 10.791216966124139 ], [ 106.687347036071799, 10.791897757564833 ], [ 106.686610019094246, 10.791938394041752 ], [ 106.686610019094246, 10.791257600159977 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d29c", "resolution": 17, "center_lat": 10.791537, "center_lon": 106.6877155, "avg_edge_len": 78.016, "cell_area": 6069.769 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.687347036071799, 10.791216966124139 ], [ 106.688084055525565, 10.791176330197136 ], [ 106.688084055525565, 10.791857119196633 ], [ 106.687347036071799, 10.791897757564833 ], [ 106.687347036071799, 10.791216966124139 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2b", "resolution": 16, "center_lat": 10.7905362, "center_lon": 106.687347, "avg_edge_len": 156.032, "cell_area": 24278.884 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.789896027806794 ], [ 106.688084055525565, 10.789814767608535 ], [ 106.688084055525565, 10.791176330197136 ], [ 106.686610019094246, 10.791257600159977 ], [ 106.686610019094246, 10.789896027806794 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2d", "resolution": 16, "center_lat": 10.7906174, "center_lon": 106.685873, "avg_edge_len": 156.032, "cell_area": 24278.888 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.789977280441276 ], [ 106.686610019094246, 10.789896027806794 ], [ 106.686610019094246, 10.791257600159977 ], [ 106.68513599256886, 10.791338862558119 ], [ 106.68513599256886, 10.789977280441276 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d2f", "resolution": 16, "center_lat": 10.791979, "center_lon": 106.685873, "avg_edge_len": 156.033, "cell_area": 24279.148 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.791338862558119 ], [ 106.686610019094246, 10.791257600159977 ], [ 106.686610019094246, 10.79261919305994 ], [ 106.68513599256886, 10.792700465221705 ], [ 106.68513599256886, 10.791338862558119 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d34", "resolution": 15, "center_lat": 10.7914201, "center_lon": 106.683662, "avg_edge_len": 312.065, "cell_area": 97116.099 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.790139763018653 ], [ 106.68513599256886, 10.789977280441276 ], [ 106.68513599256886, 10.792700465221705 ], [ 106.682187969247821, 10.792862986848132 ], [ 106.682187969247821, 10.790139763018653 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d49c", "resolution": 17, "center_lat": 10.7918621, "center_lon": 106.6818195, "avg_edge_len": 78.016, "cell_area": 6069.774 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.791541985457505 ], [ 106.682187969247821, 10.791501364660057 ], [ 106.682187969247821, 10.792182173185902 ], [ 106.681450969613365, 10.792222796423626 ], [ 106.681450969613365, 10.791541985457505 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4b", "resolution": 16, "center_lat": 10.7908612, "center_lon": 106.681451, "avg_edge_len": 156.032, "cell_area": 24278.902 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.790220992961421 ], [ 106.682187969247821, 10.790139763018653 ], [ 106.682187969247821, 10.791501364660057 ], [ 106.680713972457966, 10.791582604363727 ], [ 106.680713972457966, 10.790220992961421 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d4d", "resolution": 16, "center_lat": 10.7909424, "center_lon": 106.679977, "avg_edge_len": 156.032, "cell_area": 24278.906 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.790302215340159 ], [ 106.680713972457966, 10.790220992961421 ], [ 106.680713972457966, 10.791582604363727 ], [ 106.679239985585866, 10.791663836502444 ], [ 106.679239985585866, 10.790302215340159 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d524", "resolution": 17, "center_lat": 10.7913437, "center_lon": 106.6788715, "avg_edge_len": 78.016, "cell_area": 6069.743 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.7910236341455 ], [ 106.679239985585866, 10.790983023352835 ], [ 106.679239985585866, 10.791663836502444 ], [ 106.678502995869849, 10.791704449734929 ], [ 106.678502995869849, 10.7910236341455 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d52c", "resolution": 17, "center_lat": 10.7906629, "center_lon": 106.6788715, "avg_edge_len": 78.016, "cell_area": 6069.711 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.790342823692997 ], [ 106.679239985585866, 10.790302215340159 ], [ 106.679239985585866, 10.790983023352835 ], [ 106.678502995869849, 10.7910236341455 ], [ 106.678502995869849, 10.790342823692997 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d534", "resolution": 17, "center_lat": 10.7907035, "center_lon": 106.6781345, "avg_edge_len": 78.016, "cell_area": 6069.712 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.790383430154805 ], [ 106.678502995869849, 10.790342823692997 ], [ 106.678502995869849, 10.7910236341455 ], [ 106.677766008634421, 10.791064243047016 ], [ 106.677766008634421, 10.790383430154805 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d54c", "resolution": 17, "center_lat": 10.7907441, "center_lon": 106.6773975, "avg_edge_len": 78.016, "cell_area": 6069.712 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.790424034725573 ], [ 106.677766008634421, 10.790383430154805 ], [ 106.677766008634421, 10.791064243047016 ], [ 106.677029023879811, 10.79110485005738 ], [ 106.677029023879811, 10.790424034725573 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "317528d554", "resolution": 17, "center_lat": 10.7907847, "center_lon": 106.6766605, "avg_edge_len": 78.016, "cell_area": 6069.713 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.790464637405295 ], [ 106.677029023879811, 10.790424034725573 ], [ 106.677029023879811, 10.79110485005738 ], [ 106.67629204160653, 10.791145455176579 ], [ 106.67629204160653, 10.790464637405295 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "3175292aac", "resolution": 17, "center_lat": 10.7908253, "center_lon": 106.6759236, "avg_edge_len": 78.016, "cell_area": 6069.713 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.790505238193962 ], [ 106.67629204160653, 10.790464637405295 ], [ 106.67629204160653, 10.791145455176579 ], [ 106.675555061814862, 10.791186058404611 ], [ 106.675555061814862, 10.790505238193962 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ecd8c", "resolution": 17, "center_lat": 10.7873897, "center_lon": 106.6641322, "avg_edge_len": 78.014, "cell_area": 6069.526 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.663763722948886, 10.787069546163119 ], [ 106.664500662984608, 10.787028990257635 ], [ 106.664500662984608, 10.787709816217969 ], [ 106.663763722948886, 10.787750374561005 ], [ 106.663763722948886, 10.787069546163119 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ecdf4", "resolution": 17, "center_lat": 10.7867089, "center_lon": 106.6641322, "avg_edge_len": 78.014, "cell_area": 6069.494 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.663763722948886, 10.786388722904052 ], [ 106.664500662984608, 10.786348169436126 ], [ 106.664500662984608, 10.787028990257635 ], [ 106.663763722948886, 10.787069546163119 ], [ 106.663763722948886, 10.786388722904052 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed134", "resolution": 17, "center_lat": 10.7836612, "center_lon": 106.6700278, "avg_edge_len": 78.013, "cell_area": 6069.36 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.783341078603238 ], [ 106.670396272789503, 10.783300519766486 ], [ 106.670396272789503, 10.783981300525131 ], [ 106.669659312864042, 10.78402186180038 ], [ 106.669659312864042, 10.783341078603238 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed15", "resolution": 16, "center_lat": 10.7847027, "center_lon": 106.6696593, "avg_edge_len": 156.028, "cell_area": 24277.637 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.668922355423547, 10.784062421185652 ], [ 106.670396272789503, 10.783981300525131 ], [ 106.670396272789503, 10.785342877462115 ], [ 106.668922355423547, 10.78542400787636 ], [ 106.668922355423547, 10.784062421185652 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed164", "resolution": 17, "center_lat": 10.7851039, "center_lon": 106.6685539, "avg_edge_len": 78.014, "cell_area": 6069.426 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.784783771894704 ], [ 106.668922355423547, 10.784743211961157 ], [ 106.668922355423547, 10.78542400787636 ], [ 106.66818540046836, 10.785464570248163 ], [ 106.66818540046836, 10.784783771894704 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed16c", "resolution": 17, "center_lat": 10.7844231, "center_lon": 106.6685539, "avg_edge_len": 78.014, "cell_area": 6069.394 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.78410297868094 ], [ 106.668922355423547, 10.784062421185652 ], [ 106.668922355423547, 10.784743211961157 ], [ 106.66818540046836, 10.784783771894704 ], [ 106.66818540046836, 10.78410297868094 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed17c", "resolution": 17, "center_lat": 10.7851445, "center_lon": 106.6678169, "avg_edge_len": 78.014, "cell_area": 6069.427 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.784824329938147 ], [ 106.66818540046836, 10.784783771894704 ], [ 106.66818540046836, 10.785464570248163 ], [ 106.667448447998822, 10.785505130729748 ], [ 106.667448447998822, 10.784824329938147 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed184", "resolution": 17, "center_lat": 10.785185, "center_lon": 106.66708, "avg_edge_len": 78.014, "cell_area": 6069.427 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.666711498015275, 10.784864886091478 ], [ 106.667448447998822, 10.784824329938147 ], [ 106.667448447998822, 10.785505130729748 ], [ 106.666711498015275, 10.785545689321102 ], [ 106.666711498015275, 10.784864886091478 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed20c", "resolution": 17, "center_lat": 10.7866683, "center_lon": 106.6648691, "avg_edge_len": 78.014, "cell_area": 6069.493 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.786348169436126 ], [ 106.665237605507798, 10.786307614077824 ], [ 106.665237605507798, 10.786988432461662 ], [ 106.664500662984608, 10.787028990257635 ], [ 106.664500662984608, 10.786348169436126 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed214", "resolution": 17, "center_lat": 10.7866277, "center_lon": 106.6656061, "avg_edge_len": 78.014, "cell_area": 6069.493 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.786307614077824 ], [ 106.665974550518172, 10.786267056829157 ], [ 106.665974550518172, 10.786947872775208 ], [ 106.665237605507798, 10.786988432461662 ], [ 106.665237605507798, 10.786307614077824 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed21c", "resolution": 17, "center_lat": 10.7859469, "center_lon": 106.6656061, "avg_edge_len": 78.014, "cell_area": 6069.461 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.785626800833098 ], [ 106.665974550518172, 10.785586246022225 ], [ 106.665974550518172, 10.786267056829157 ], [ 106.665237605507798, 10.786307614077824 ], [ 106.665237605507798, 10.785626800833098 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed23", "resolution": 16, "center_lat": 10.7862265, "center_lon": 106.6667115, "avg_edge_len": 156.028, "cell_area": 24277.904 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665974550518172, 10.785586246022225 ], [ 106.667448447998822, 10.785505130729748 ], [ 106.667448447998822, 10.786866747730887 ], [ 106.665974550518172, 10.786947872775208 ], [ 106.665974550518172, 10.785586246022225 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed25", "resolution": 16, "center_lat": 10.7875881, "center_lon": 106.6667115, "avg_edge_len": 156.029, "cell_area": 24278.164 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665974550518172, 10.786947872775208 ], [ 106.667448447998822, 10.786866747730887 ], [ 106.667448447998822, 10.788228385287391 ], [ 106.665974550518172, 10.788309520083525 ], [ 106.665974550518172, 10.786947872775208 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed27", "resolution": 16, "center_lat": 10.7876693, "center_lon": 106.6652376, "avg_edge_len": 156.029, "cell_area": 24278.167 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.787028990257635 ], [ 106.665974550518172, 10.786947872775208 ], [ 106.665974550518172, 10.788309520083525 ], [ 106.664500662984608, 10.788390647316842 ], [ 106.664500662984608, 10.787028990257635 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed29", "resolution": 16, "center_lat": 10.7890309, "center_lon": 106.6652376, "avg_edge_len": 156.03, "cell_area": 24278.427 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.788390647316842 ], [ 106.665974550518172, 10.788309520083525 ], [ 106.665974550518172, 10.789671187944908 ], [ 106.664500662984608, 10.789752324929079 ], [ 106.664500662984608, 10.788390647316842 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed2a4", "resolution": 17, "center_lat": 10.7900725, "center_lon": 106.6648691, "avg_edge_len": 78.015, "cell_area": 6069.656 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.664500662984608, 10.789752324929079 ], [ 106.665237605507798, 10.789711757382463 ], [ 106.665237605507798, 10.79039260145761 ], [ 106.664500662984608, 10.790433171441876 ], [ 106.664500662984608, 10.789752324929079 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed2bc", "resolution": 17, "center_lat": 10.7900319, "center_lon": 106.6656061, "avg_edge_len": 78.015, "cell_area": 6069.655 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665237605507798, 10.789711757382463 ], [ 106.665974550518172, 10.789671187944908 ], [ 106.665974550518172, 10.790352029582291 ], [ 106.665237605507798, 10.79039260145761 ], [ 106.665237605507798, 10.789711757382463 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed2f", "resolution": 16, "center_lat": 10.7889498, "center_lon": 106.6667115, "avg_edge_len": 156.03, "cell_area": 24278.423 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.665974550518172, 10.788309520083525 ], [ 106.667448447998822, 10.788228385287391 ], [ 106.667448447998822, 10.789590043397 ], [ 106.665974550518172, 10.789671187944908 ], [ 106.665974550518172, 10.788309520083525 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed304", "resolution": 17, "center_lat": 10.7885485, "center_lon": 106.6678169, "avg_edge_len": 78.015, "cell_area": 6069.589 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.788228385287391 ], [ 106.66818540046836, 10.788187815053288 ], [ 106.66818540046836, 10.788868639100979 ], [ 106.667448447998822, 10.7889092117732 ], [ 106.667448447998822, 10.788228385287391 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed30c", "resolution": 17, "center_lat": 10.7885079, "center_lon": 106.6685539, "avg_edge_len": 78.015, "cell_area": 6069.588 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.66818540046836, 10.788187815053288 ], [ 106.668922355423547, 10.788147242928504 ], [ 106.668922355423547, 10.788828064537961 ], [ 106.66818540046836, 10.788868639100979 ], [ 106.66818540046836, 10.788187815053288 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed31c", "resolution": 17, "center_lat": 10.7892293, "center_lon": 106.6678169, "avg_edge_len": 78.015, "cell_area": 6069.621 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.7889092117732 ], [ 106.66818540046836, 10.788868639100979 ], [ 106.66818540046836, 10.789549468286664 ], [ 106.667448447998822, 10.789590043397 ], [ 106.667448447998822, 10.7889092117732 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed374", "resolution": 17, "center_lat": 10.7884674, "center_lon": 106.6692908, "avg_edge_len": 78.015, "cell_area": 6069.588 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.668922355423547, 10.788147242928504 ], [ 106.669659312864042, 10.788106668913048 ], [ 106.669659312864042, 10.788787488084155 ], [ 106.668922355423547, 10.788828064537961 ], [ 106.668922355423547, 10.788147242928504 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed37c", "resolution": 17, "center_lat": 10.7884268, "center_lon": 106.6700278, "avg_edge_len": 78.015, "cell_area": 6069.587 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.669659312864042, 10.788106668913048 ], [ 106.670396272789503, 10.788066093006927 ], [ 106.670396272789503, 10.788746909739572 ], [ 106.669659312864042, 10.788787488084155 ], [ 106.669659312864042, 10.788106668913048 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed3c", "resolution": 15, "center_lat": 10.7867856, "center_lon": 106.6689224, "avg_edge_len": 312.057, "cell_area": 97112.114 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.667448447998822, 10.785505130729748 ], [ 106.670396272789503, 10.785342877462115 ], [ 106.670396272789503, 10.788066093006927 ], [ 106.667448447998822, 10.788228385287391 ], [ 106.667448447998822, 10.785505130729748 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed44", "resolution": 15, "center_lat": 10.7866233, "center_lon": 106.6718702, "avg_edge_len": 312.058, "cell_area": 97112.083 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.785342877462115 ], [ 106.673344137333231, 10.785180593951329 ], [ 106.673344137333231, 10.787903770475946 ], [ 106.670396272789503, 10.788066093006927 ], [ 106.670396272789503, 10.785342877462115 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed49", "resolution": 16, "center_lat": 10.7887063, "center_lon": 106.6711332, "avg_edge_len": 156.03, "cell_area": 24278.412 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.788066093006927 ], [ 106.671870200093736, 10.787984935522722 ], [ 106.671870200093736, 10.789346564371487 ], [ 106.670396272789503, 10.789427731610227 ], [ 106.670396272789503, 10.788066093006927 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4bc", "resolution": 17, "center_lat": 10.7897073, "center_lon": 106.6715017, "avg_edge_len": 78.015, "cell_area": 6069.651 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.789387148936298 ], [ 106.671870200093736, 10.789346564371487 ], [ 106.671870200093736, 10.790027386502617 ], [ 106.671133235199477, 10.790067973506108 ], [ 106.671133235199477, 10.789387148936298 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4c4", "resolution": 17, "center_lat": 10.7896667, "center_lon": 106.6722387, "avg_edge_len": 78.015, "cell_area": 6069.651 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.789346564371487 ], [ 106.67260716747171, 10.789305977915806 ], [ 106.67260716747171, 10.789986797608133 ], [ 106.671870200093736, 10.790027386502617 ], [ 106.671870200093736, 10.789346564371487 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4d4", "resolution": 17, "center_lat": 10.7903069, "center_lon": 106.6729757, "avg_edge_len": 78.016, "cell_area": 6069.683 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.789986797608133 ], [ 106.673344137333231, 10.789946206822675 ], [ 106.673344137333231, 10.790627029213551 ], [ 106.67260716747171, 10.790667622437921 ], [ 106.67260716747171, 10.789986797608133 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4dc", "resolution": 17, "center_lat": 10.7896261, "center_lon": 106.6729757, "avg_edge_len": 78.015, "cell_area": 6069.65 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.789305977915806 ], [ 106.673344137333231, 10.789265389569259 ], [ 106.673344137333231, 10.789946206822675 ], [ 106.67260716747171, 10.789986797608133 ], [ 106.67260716747171, 10.789305977915806 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed4f", "resolution": 16, "center_lat": 10.7886252, "center_lon": 106.6726072, "avg_edge_len": 156.03, "cell_area": 24278.408 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.787984935522722 ], [ 106.673344137333231, 10.787903770475946 ], [ 106.673344137333231, 10.789265389569259 ], [ 106.671870200093736, 10.789346564371487 ], [ 106.671870200093736, 10.787984935522722 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed54", "resolution": 15, "center_lat": 10.7891842, "center_lon": 106.6748181, "avg_edge_len": 312.061, "cell_area": 97114.126 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.787903770475946 ], [ 106.67629204160653, 10.787741417694953 ], [ 106.67629204160653, 10.790464637405295 ], [ 106.673344137333231, 10.790627029213551 ], [ 106.673344137333231, 10.787903770475946 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed5c", "resolution": 15, "center_lat": 10.786461, "center_lon": 106.6748181, "avg_edge_len": 312.058, "cell_area": 97112.051 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.785180593951329 ], [ 106.67629204160653, 10.785018280197887 ], [ 106.67629204160653, 10.787741417694953 ], [ 106.673344137333231, 10.787903770475946 ], [ 106.673344137333231, 10.785180593951329 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed64", "resolution": 15, "center_lat": 10.7837379, "center_lon": 106.6748181, "avg_edge_len": 312.055, "cell_area": 97109.975 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.782457499657808 ], [ 106.67629204160653, 10.782295224932202 ], [ 106.67629204160653, 10.785018280197887 ], [ 106.673344137333231, 10.785180593951329 ], [ 106.673344137333231, 10.782457499657808 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed69", "resolution": 16, "center_lat": 10.7845404, "center_lon": 106.6726072, "avg_edge_len": 156.028, "cell_area": 24277.63 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.783900172304746 ], [ 106.673344137333231, 10.78381903652455 ], [ 106.673344137333231, 10.785180593951329 ], [ 106.671870200093736, 10.785261739487087 ], [ 106.671870200093736, 10.783900172304746 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6b", "resolution": 16, "center_lat": 10.7846215, "center_lon": 106.6711332, "avg_edge_len": 156.028, "cell_area": 24277.633 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.783981300525131 ], [ 106.671870200093736, 10.783900172304746 ], [ 106.671870200093736, 10.785261739487087 ], [ 106.670396272789503, 10.785342877462115 ], [ 106.670396272789503, 10.783981300525131 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6c4", "resolution": 17, "center_lat": 10.7835801, "center_lon": 106.6715017, "avg_edge_len": 78.013, "cell_area": 6069.359 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.783259959039881 ], [ 106.671870200093736, 10.783219396423434 ], [ 106.671870200093736, 10.783900172304746 ], [ 106.671133235199477, 10.78394073735992 ], [ 106.671133235199477, 10.783259959039881 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6cc", "resolution": 17, "center_lat": 10.7836206, "center_lon": 106.6707648, "avg_edge_len": 78.013, "cell_area": 6069.36 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.670396272789503, 10.783300519766486 ], [ 106.671133235199477, 10.783259959039881 ], [ 106.671133235199477, 10.78394073735992 ], [ 106.670396272789503, 10.783981300525131 ], [ 106.670396272789503, 10.783300519766486 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6dc", "resolution": 17, "center_lat": 10.7828993, "center_lon": 106.6715017, "avg_edge_len": 78.013, "cell_area": 6069.327 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671133235199477, 10.782579185860124 ], [ 106.671870200093736, 10.782538625682404 ], [ 106.671870200093736, 10.783219396423434 ], [ 106.671133235199477, 10.783259959039881 ], [ 106.671133235199477, 10.782579185860124 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed6f", "resolution": 16, "center_lat": 10.7831788, "center_lon": 106.6726072, "avg_edge_len": 156.027, "cell_area": 24277.37 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.671870200093736, 10.782538625682404 ], [ 106.673344137333231, 10.782457499657808 ], [ 106.673344137333231, 10.78381903652455 ], [ 106.671870200093736, 10.783900172304746 ], [ 106.671870200093736, 10.782538625682404 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed704", "resolution": 17, "center_lat": 10.7821374, "center_lon": 106.6729757, "avg_edge_len": 78.013, "cell_area": 6069.294 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67260716747171, 10.78181730045335 ], [ 106.673344137333231, 10.781776738935159 ], [ 106.673344137333231, 10.782457499657808 ], [ 106.67260716747171, 10.782498063614963 ], [ 106.67260716747171, 10.78181730045335 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7ac", "resolution": 17, "center_lat": 10.7820968, "center_lon": 106.6737126, "avg_edge_len": 78.013, "cell_area": 6069.293 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.673344137333231, 10.781776738935159 ], [ 106.674081109677786, 10.781736175527376 ], [ 106.674081109677786, 10.782416933810946 ], [ 106.673344137333231, 10.782457499657808 ], [ 106.673344137333231, 10.781776738935159 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7b4", "resolution": 17, "center_lat": 10.7820563, "center_lon": 106.6744496, "avg_edge_len": 78.013, "cell_area": 6069.293 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.781736175527376 ], [ 106.674818084505148, 10.781695610230008 ], [ 106.674818084505148, 10.782376366074388 ], [ 106.674081109677786, 10.782416933810946 ], [ 106.674081109677786, 10.781736175527376 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7bc", "resolution": 17, "center_lat": 10.7813755, "center_lon": 106.6744496, "avg_edge_len": 78.013, "cell_area": 6069.26 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.781055422384666 ], [ 106.674818084505148, 10.781014859526501 ], [ 106.674818084505148, 10.781695610230008 ], [ 106.674081109677786, 10.781736175527376 ], [ 106.674081109677786, 10.781055422384666 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7d", "resolution": 16, "center_lat": 10.781655, "center_lon": 106.6755551, "avg_edge_len": 156.026, "cell_area": 24277.102 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.781014859526501 ], [ 106.67629204160653, 10.780933728141791 ], [ 106.67629204160653, 10.782295224932202 ], [ 106.674818084505148, 10.782376366074388 ], [ 106.674818084505148, 10.781014859526501 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ed7e4", "resolution": 17, "center_lat": 10.7806136, "center_lon": 106.6759236, "avg_edge_len": 78.013, "cell_area": 6069.227 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.780293551655827 ], [ 106.67629204160653, 10.780252987458182 ], [ 106.67629204160653, 10.780933728141791 ], [ 106.675555061814862, 10.780974294778872 ], [ 106.675555061814862, 10.780293551655827 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752edf8c", "resolution": 17, "center_lat": 10.7704031, "center_lon": 106.6759236, "avg_edge_len": 78.01, "cell_area": 6068.739 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.770083021941073 ], [ 106.67629204160653, 10.770042494335451 ], [ 106.67629204160653, 10.770723157867735 ], [ 106.675555061814862, 10.770763687912856 ], [ 106.675555061814862, 10.770083021941073 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752edfe4", "resolution": 17, "center_lat": 10.7690823, "center_lon": 106.6751866, "avg_edge_len": 78.009, "cell_area": 6068.675 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.768762226274051 ], [ 106.675555061814862, 10.768721705434828 ], [ 106.675555061814862, 10.769402361114967 ], [ 106.674818084505148, 10.769442884393589 ], [ 106.674818084505148, 10.768762226274051 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752edff4", "resolution": 17, "center_lat": 10.7697224, "center_lon": 106.6759236, "avg_edge_len": 78.009, "cell_area": 6068.707 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.769402361114967 ], [ 106.67629204160653, 10.76936183594885 ], [ 106.67629204160653, 10.770042494335451 ], [ 106.675555061814862, 10.770083021941073 ], [ 106.675555061814862, 10.769402361114967 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752edffc", "resolution": 17, "center_lat": 10.7690418, "center_lon": 106.6759236, "avg_edge_len": 78.009, "cell_area": 6068.674 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.768721705434828 ], [ 106.67629204160653, 10.768681182708217 ], [ 106.67629204160653, 10.76936183594885 ], [ 106.675555061814862, 10.769402361114967 ], [ 106.675555061814862, 10.768721705434828 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee01", "resolution": 16, "center_lat": 10.7680411, "center_lon": 106.6755551, "avg_edge_len": 156.018, "cell_area": 24274.503 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.767400925473986 ], [ 106.67629204160653, 10.767319891665981 ], [ 106.67629204160653, 10.768681182708217 ], [ 106.674818084505148, 10.768762226274051 ], [ 106.674818084505148, 10.767400925473986 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee024", "resolution": 17, "center_lat": 10.7669998, "center_lon": 106.6759236, "avg_edge_len": 78.009, "cell_area": 6068.577 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.675555061814862, 10.766679769273001 ], [ 106.67629204160653, 10.766639253864945 ], [ 106.67629204160653, 10.767319891665981 ], [ 106.675555061814862, 10.767360409513559 ], [ 106.675555061814862, 10.766679769273001 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee03c", "resolution": 17, "center_lat": 10.7670403, "center_lon": 106.6751866, "avg_edge_len": 78.009, "cell_area": 6068.577 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674818084505148, 10.76672028279402 ], [ 106.675555061814862, 10.766679769273001 ], [ 106.675555061814862, 10.767360409513559 ], [ 106.674818084505148, 10.767400925473986 ], [ 106.674818084505148, 10.76672028279402 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee06c", "resolution": 17, "center_lat": 10.7677615, "center_lon": 106.6744496, "avg_edge_len": 78.009, "cell_area": 6068.61 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.767441439547252 ], [ 106.674818084505148, 10.767400925473986 ], [ 106.674818084505148, 10.768081573300758 ], [ 106.674081109677786, 10.768122089813311 ], [ 106.674081109677786, 10.767441439547252 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752ee074", "resolution": 17, "center_lat": 10.7684422, "center_lon": 106.6744496, "avg_edge_len": 78.009, "cell_area": 6068.643 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.674081109677786, 10.768122089813311 ], [ 106.674818084505148, 10.768081573300758 ], [ 106.674818084505148, 10.768762226274051 ], [ 106.674081109677843, 10.768802745225889 ], [ 106.674081109677786, 10.768122089813311 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1894", "resolution": 17, "center_lat": 10.7672346, "center_lon": 106.6840305, "avg_edge_len": 78.009, "cell_area": 6068.603 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.766914609398494 ], [ 106.684398983021822, 10.766874070792747 ], [ 106.684398983021822, 10.767554686898322 ], [ 106.683661975952418, 10.767595227944849 ], [ 106.683661975952418, 10.766914609398494 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f189c", "resolution": 17, "center_lat": 10.7679153, "center_lon": 106.6840305, "avg_edge_len": 78.009, "cell_area": 6068.636 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.683661975952418, 10.767595227944849 ], [ 106.684398983021822, 10.767554686898322 ], [ 106.684398983021822, 10.768235308150475 ], [ 106.683661975952418, 10.76827585163778 ], [ 106.683661975952418, 10.767595227944849 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18b", "resolution": 16, "center_lat": 10.7676358, "center_lon": 106.682925, "avg_edge_len": 156.018, "cell_area": 24274.481 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.766995680948776 ], [ 106.683661975952418, 10.766914609398494 ], [ 106.683661975952418, 10.76827585163778 ], [ 106.682187969247821, 10.768356932950493 ], [ 106.682187969247821, 10.766995680948776 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18c4", "resolution": 17, "center_lat": 10.7665946, "center_lon": 106.6832935, "avg_edge_len": 78.009, "cell_area": 6068.571 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682924971360933, 10.766274530277006 ], [ 106.683661975952418, 10.766233995998997 ], [ 106.683661975952418, 10.766914609398494 ], [ 106.682924971360933, 10.766955146117171 ], [ 106.682924971360933, 10.766274530277006 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18cc", "resolution": 17, "center_lat": 10.7666351, "center_lon": 106.6825565, "avg_edge_len": 78.009, "cell_area": 6068.572 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.766315062668054 ], [ 106.682924971360933, 10.766274530277006 ], [ 106.682924971360933, 10.766955146117171 ], [ 106.682187969247821, 10.766995680948776 ], [ 106.682187969247821, 10.766315062668054 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f18d4", "resolution": 17, "center_lat": 10.7659545, "center_lon": 106.6825565, "avg_edge_len": 78.009, "cell_area": 6068.539 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.765634449534463 ], [ 106.682924971360933, 10.765593919583974 ], [ 106.682924971360933, 10.766274530277006 ], [ 106.682187969247821, 10.766315062668054 ], [ 106.682187969247821, 10.765634449534463 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1ed4", "resolution": 17, "center_lat": 10.7653144, "center_lon": 106.6818195, "avg_edge_len": 78.008, "cell_area": 6068.507 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.76499436717147 ], [ 106.682187969247821, 10.764953841548287 ], [ 106.682187969247821, 10.765634449534463 ], [ 106.681450969613365, 10.765674977598096 ], [ 106.681450969613365, 10.76499436717147 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1edc", "resolution": 17, "center_lat": 10.7653549, "center_lon": 106.6810825, "avg_edge_len": 78.008, "cell_area": 6068.508 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.765034890907909 ], [ 106.681450969613365, 10.76499436717147 ], [ 106.681450969613365, 10.765674977598096 ], [ 106.680713972457966, 10.765715503774869 ], [ 106.680713972457966, 10.765034890907909 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f4", "resolution": 15, "center_lat": 10.7670768, "center_lon": 106.680714, "avg_edge_len": 312.035, "cell_area": 97097.431 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.765796550467805 ], [ 106.682187969247821, 10.765634449534463 ], [ 106.682187969247821, 10.768356932950493 ], [ 106.679239985585866, 10.768519072928106 ], [ 106.679239985585866, 10.765796550467805 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1f9", "resolution": 16, "center_lat": 10.767879, "center_lon": 106.678503, "avg_edge_len": 156.018, "cell_area": 24274.494 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.767238850309395 ], [ 106.679239985585866, 10.76715780140429 ], [ 106.679239985585866, 10.768519072928106 ], [ 106.677766008634421, 10.76860013159288 ], [ 106.677766008634421, 10.767238850309395 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fb", "resolution": 16, "center_lat": 10.7665177, "center_lon": 106.678503, "avg_edge_len": 156.017, "cell_area": 24274.234 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.765877589613204 ], [ 106.679239985585866, 10.765796550467805 ], [ 106.679239985585866, 10.76715780140429 ], [ 106.677766008634421, 10.767238850309395 ], [ 106.677766008634421, 10.765877589613204 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fc4", "resolution": 17, "center_lat": 10.7669188, "center_lon": 106.6773975, "avg_edge_len": 78.009, "cell_area": 6068.576 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.766598736569856 ], [ 106.677766008634421, 10.766558217387745 ], [ 106.677766008634421, 10.767238850309395 ], [ 106.677029023879811, 10.767279371931256 ], [ 106.677029023879811, 10.766598736569856 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1fdc", "resolution": 17, "center_lat": 10.7669593, "center_lon": 106.6766605, "avg_edge_len": 78.009, "cell_area": 6068.576 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.766639253864945 ], [ 106.677029023879811, 10.766598736569856 ], [ 106.677029023879811, 10.767279371931256 ], [ 106.67629204160653, 10.767319891665981 ], [ 106.67629204160653, 10.766639253864945 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f1ff", "resolution": 16, "center_lat": 10.76796, "center_lon": 106.677029, "avg_edge_len": 156.018, "cell_area": 24274.499 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.767319891665981 ], [ 106.677766008634421, 10.767238850309395 ], [ 106.677766008634421, 10.76860013159288 ], [ 106.67629204160653, 10.768681182708217 ], [ 106.67629204160653, 10.767319891665981 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f204", "resolution": 15, "center_lat": 10.7699614, "center_lon": 106.677766, "avg_edge_len": 312.038, "cell_area": 97099.547 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.768681182708217 ], [ 106.679239985585866, 10.768519072928106 ], [ 106.679239985585866, 10.77124167772868 ], [ 106.67629204160653, 10.77140382654542 ], [ 106.67629204160653, 10.768681182708217 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f208c", "resolution": 17, "center_lat": 10.7716834, "center_lon": 106.6773975, "avg_edge_len": 78.01, "cell_area": 6068.803 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.771363292172968 ], [ 106.677766008634421, 10.771322755912689 ], [ 106.677766008634421, 10.772003424856168 ], [ 106.677029023879811, 10.772043963556166 ], [ 106.677029023879811, 10.771363292172968 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f20dc", "resolution": 17, "center_lat": 10.7729636, "center_lon": 106.6788715, "avg_edge_len": 78.011, "cell_area": 6068.867 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.772643555916725 ], [ 106.679239985585866, 10.772603011000918 ], [ 106.679239985585866, 10.773283685354031 ], [ 106.678502995869849, 10.773324232709779 ], [ 106.678502995869849, 10.772643555916725 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f20f", "resolution": 16, "center_lat": 10.7719629, "center_lon": 106.678503, "avg_edge_len": 156.02, "cell_area": 24275.275 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.771322755912689 ], [ 106.679239985585866, 10.77124167772868 ], [ 106.679239985585866, 10.772603011000918 ], [ 106.677766008634421, 10.77268409894449 ], [ 106.677766008634421, 10.771322755912689 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f214", "resolution": 15, "center_lat": 10.7725219, "center_lon": 106.680714, "avg_edge_len": 312.042, "cell_area": 97101.594 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.77124167772868 ], [ 106.682187969247821, 10.771079498707081 ], [ 106.682187969247821, 10.773802146786133 ], [ 106.679239985585866, 10.773964364851436 ], [ 106.679239985585866, 10.77124167772868 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f21c", "resolution": 15, "center_lat": 10.7697993, "center_lon": 106.680714, "avg_edge_len": 312.039, "cell_area": 97099.513 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.768519072928106 ], [ 106.682187969247821, 10.768356932950493 ], [ 106.682187969247821, 10.771079498707081 ], [ 106.679239985585866, 10.77124167772868 ], [ 106.679239985585866, 10.768519072928106 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f224", "resolution": 15, "center_lat": 10.7696371, "center_lon": 106.683662, "avg_edge_len": 312.039, "cell_area": 97099.478 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.768356932950493 ], [ 106.68513599256886, 10.768194762775879 ], [ 106.68513599256886, 10.770917289481122 ], [ 106.682187969247821, 10.771079498707081 ], [ 106.682187969247821, 10.768356932950493 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f228c", "resolution": 17, "center_lat": 10.7691954, "center_lon": 106.6855045, "avg_edge_len": 78.01, "cell_area": 6068.7 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.768875386733443 ], [ 106.685873004593077, 10.768834837030564 ], [ 106.685873004593077, 10.769515463693152 ], [ 106.68513599256886, 10.769556015837027 ], [ 106.68513599256886, 10.768875386733443 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f22dc", "resolution": 17, "center_lat": 10.7704756, "center_lon": 106.6869785, "avg_edge_len": 78.01, "cell_area": 6068.764 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.686610019094246, 10.770155539028986 ], [ 106.687347036071799, 10.770114980668888 ], [ 106.687347036071799, 10.770795612740349 ], [ 106.686610019094189, 10.770836173541667 ], [ 106.686610019094246, 10.770155539028986 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f22f", "resolution": 16, "center_lat": 10.7701961, "center_lon": 106.685873, "avg_edge_len": 156.02, "cell_area": 24274.993 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.769556015837027 ], [ 106.686610019094246, 10.769474909661769 ], [ 106.686610019094189, 10.770836173541667 ], [ 106.68513599256886, 10.770917289481122 ], [ 106.68513599256886, 10.769556015837027 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f234", "resolution": 15, "center_lat": 10.7721975, "center_lon": 106.68661, "avg_edge_len": 312.042, "cell_area": 97101.522 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.68513599256886, 10.770917289481122 ], [ 106.688084055525565, 10.77075505005131 ], [ 106.688084055525565, 10.773477620020882 ], [ 106.68513599256886, 10.773639898509115 ], [ 106.68513599256886, 10.770917289481122 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f23c", "resolution": 15, "center_lat": 10.7723597, "center_lon": 106.683662, "avg_edge_len": 312.042, "cell_area": 97101.559 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.771079498707081 ], [ 106.68513599256886, 10.770917289481122 ], [ 106.68513599256886, 10.773639898509115 ], [ 106.682187969247821, 10.773802146786133 ], [ 106.682187969247821, 10.771079498707081 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f25", "resolution": 14, "center_lat": 10.7763626, "center_lon": 106.685136, "avg_edge_len": 624.094, "cell_area": 388418.64 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.773802146786133 ], [ 106.688084055525565, 10.773477620020882 ], [ 106.688084055525565, 10.778923006856742 ], [ 106.682187969247821, 10.779247689839247 ], [ 106.682187969247821, 10.773802146786133 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f261", "resolution": 16, "center_lat": 10.7786076, "center_lon": 106.681451, "avg_edge_len": 156.025, "cell_area": 24276.566 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.777967415312785 ], [ 106.682187969247821, 10.77788627321975 ], [ 106.682187969247821, 10.779247689839247 ], [ 106.680713972457966, 10.779328841693507 ], [ 106.680713972457966, 10.777967415312785 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2624", "resolution": 17, "center_lat": 10.7790088, "center_lon": 106.6803455, "avg_edge_len": 78.012, "cell_area": 6069.159 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.778688696585382 ], [ 106.680713972457966, 10.778648125932131 ], [ 106.680713972457966, 10.779328841693507 ], [ 106.679976977782019, 10.77936941478689 ], [ 106.679976977782019, 10.778688696585382 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f262c", "resolution": 17, "center_lat": 10.7790493, "center_lon": 106.6796085, "avg_edge_len": 78.012, "cell_area": 6069.159 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.778729265349572 ], [ 106.679976977782019, 10.778688696585382 ], [ 106.679976977782019, 10.77936941478689 ], [ 106.679239985585866, 10.779409985991098 ], [ 106.679239985585866, 10.778729265349572 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f263c", "resolution": 17, "center_lat": 10.7783281, "center_lon": 106.6803455, "avg_edge_len": 78.012, "cell_area": 6069.126 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.778007983525901 ], [ 106.680713972457966, 10.777967415312785 ], [ 106.680713972457966, 10.778648125932131 ], [ 106.679976977782019, 10.778688696585382 ], [ 106.679976977782019, 10.778007983525901 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f266c", "resolution": 17, "center_lat": 10.7776068, "center_lon": 106.6810825, "avg_edge_len": 78.012, "cell_area": 6069.093 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.777286709835757 ], [ 106.681450969613365, 10.77724614217396 ], [ 106.681450969613365, 10.777926845210732 ], [ 106.680713972457966, 10.777967415312785 ], [ 106.680713972457966, 10.777286709835757 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2674", "resolution": 17, "center_lat": 10.7775662, "center_lon": 106.6818195, "avg_edge_len": 78.012, "cell_area": 6069.093 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.77724614217396 ], [ 106.682187969247821, 10.777205572623346 ], [ 106.682187969247821, 10.77788627321975 ], [ 106.681450969613365, 10.777926845210732 ], [ 106.681450969613365, 10.77724614217396 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f267c", "resolution": 17, "center_lat": 10.7768855, "center_lon": 106.6818195, "avg_edge_len": 78.012, "cell_area": 6069.06 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.776565444279795 ], [ 106.682187969247821, 10.776524877169557 ], [ 106.682187969247821, 10.777205572623346 ], [ 106.681450969613365, 10.77724614217396 ], [ 106.681450969613365, 10.776565444279795 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f269c", "resolution": 17, "center_lat": 10.7755241, "center_lon": 106.6818195, "avg_edge_len": 78.011, "cell_area": 6068.995 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.681450969613365, 10.775204063920405 ], [ 106.682187969247821, 10.775163501690932 ], [ 106.682187969247821, 10.775844186858654 ], [ 106.681450969613365, 10.775884751528514 ], [ 106.681450969613365, 10.775204063920405 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f26b", "resolution": 16, "center_lat": 10.7745234, "center_lon": 106.681451, "avg_edge_len": 156.022, "cell_area": 24275.786 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.680713972457966, 10.77388325959528 ], [ 106.682187969247821, 10.773802146786133 ], [ 106.682187969247821, 10.775163501690932 ], [ 106.680713972457966, 10.775244624261408 ], [ 106.680713972457966, 10.77388325959528 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f26cc", "resolution": 17, "center_lat": 10.7742439, "center_lon": 106.6803455, "avg_edge_len": 78.011, "cell_area": 6068.931 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679976977782019, 10.773923813167487 ], [ 106.680713972457966, 10.77388325959528 ], [ 106.680713972457966, 10.774563939356479 ], [ 106.679976977782019, 10.774604495368846 ], [ 106.679976977782019, 10.773923813167487 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f27d4", "resolution": 17, "center_lat": 10.7790899, "center_lon": 106.6788715, "avg_edge_len": 78.012, "cell_area": 6069.16 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.678502995869849, 10.778769832224693 ], [ 106.679239985585866, 10.778729265349572 ], [ 106.679239985585866, 10.779409985991098 ], [ 106.678502995869849, 10.77945055530612 ], [ 106.678502995869849, 10.778769832224693 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f280c", "resolution": 17, "center_lat": 10.7798518, "center_lon": 106.6773975, "avg_edge_len": 78.013, "cell_area": 6069.193 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.779531688268587 ], [ 106.677766008634421, 10.779491122731955 ], [ 106.677766008634421, 10.780171853394904 ], [ 106.677029023879811, 10.780212421371203 ], [ 106.677029023879811, 10.779531688268587 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2814", "resolution": 17, "center_lat": 10.7805325, "center_lon": 106.6773975, "avg_edge_len": 78.013, "cell_area": 6069.226 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677029023879811, 10.780212421371203 ], [ 106.677766008634421, 10.780171853394904 ], [ 106.677766008634421, 10.780852589199302 ], [ 106.677029023879811, 10.780893159615264 ], [ 106.677029023879811, 10.780212421371203 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f281c", "resolution": 17, "center_lat": 10.7805731, "center_lon": 106.6766605, "avg_edge_len": 78.013, "cell_area": 6069.226 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.780252987458182 ], [ 106.677029023879811, 10.780212421371203 ], [ 106.677029023879811, 10.780893159615264 ], [ 106.67629204160653, 10.780933728141791 ], [ 106.67629204160653, 10.780252987458182 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f283", "resolution": 16, "center_lat": 10.7815739, "center_lon": 106.677029, "avg_edge_len": 156.026, "cell_area": 24277.098 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.780933728141791 ], [ 106.677766008634421, 10.780852589199302 ], [ 106.677766008634421, 10.782214076231321 ], [ 106.67629204160653, 10.782295224932202 ], [ 106.67629204160653, 10.780933728141791 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f285", "resolution": 16, "center_lat": 10.7814928, "center_lon": 106.678503, "avg_edge_len": 156.026, "cell_area": 24277.094 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.780852589199302 ], [ 106.679239985585866, 10.780771442699097 ], [ 106.679239985585866, 10.782132919971803 ], [ 106.677766008634421, 10.782214076231321 ], [ 106.677766008634421, 10.780852589199302 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f287", "resolution": 16, "center_lat": 10.7801313, "center_lon": 106.678503, "avg_edge_len": 156.025, "cell_area": 24276.834 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.677766008634421, 10.779491122731955 ], [ 106.679239985585866, 10.779409985991098 ], [ 106.679239985585866, 10.780771442699097 ], [ 106.677766008634421, 10.780852589199302 ], [ 106.677766008634421, 10.779491122731955 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f28c", "resolution": 15, "center_lat": 10.7806903, "center_lon": 106.680714, "avg_edge_len": 312.052, "cell_area": 97107.831 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.779409985991098 ], [ 106.682187969247821, 10.779247689839247 ], [ 106.682187969247821, 10.781970584777106 ], [ 106.679239985585866, 10.782132919971803 ], [ 106.679239985585866, 10.779409985991098 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f294", "resolution": 15, "center_lat": 10.7834133, "center_lon": 106.680714, "avg_edge_len": 312.055, "cell_area": 97109.908 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.679239985585866, 10.782132919971803 ], [ 106.682187969247821, 10.781970584777106 ], [ 106.682187969247821, 10.784693561965032 ], [ 106.679239985585866, 10.784855936202286 ], [ 106.679239985585866, 10.782132919971803 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f29c", "resolution": 15, "center_lat": 10.7835756, "center_lon": 106.677766, "avg_edge_len": 312.055, "cell_area": 97109.942 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.782295224932202 ], [ 106.679239985585866, 10.782132919971803 ], [ 106.679239985585866, 10.784855936202286 ], [ 106.67629204160653, 10.785018280197887 ], [ 106.67629204160653, 10.782295224932202 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2b", "resolution": 14, "center_lat": 10.7875791, "center_lon": 106.67924, "avg_edge_len": 624.119, "cell_area": 388452.155 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.67629204160653, 10.785018280197887 ], [ 106.682187969247821, 10.784693561965032 ], [ 106.682187969247821, 10.790139763018653 ], [ 106.67629204160653, 10.790464637405295 ], [ 106.67629204160653, 10.785018280197887 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2d", "resolution": 14, "center_lat": 10.7872542, "center_lon": 106.685136, "avg_edge_len": 624.12, "cell_area": 388451.876 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.784693561965032 ], [ 106.688084055525565, 10.784368722767567 ], [ 106.688084055525565, 10.789814767608535 ], [ 106.682187969247821, 10.790139763018653 ], [ 106.682187969247821, 10.784693561965032 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f2f", "resolution": 14, "center_lat": 10.7818082, "center_lon": 106.685136, "avg_edge_len": 624.107, "cell_area": 388435.266 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.682187969247821, 10.779247689839247 ], [ 106.688084055525565, 10.778923006856742 ], [ 106.688084055525565, 10.784368722767567 ], [ 106.682187969247821, 10.784693561965032 ], [ 106.682187969247821, 10.779247689839247 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f31", "resolution": 14, "center_lat": 10.7814834, "center_lon": 106.6910322, "avg_edge_len": 624.108, "cell_area": 388434.968 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.778923006856742 ], [ 106.693980300251837, 10.778598202972537 ], [ 106.693980300251837, 10.78404376260953 ], [ 106.688084055525565, 10.784368722767567 ], [ 106.688084055525565, 10.778923006856742 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f324", "resolution": 15, "center_lat": 10.785649, "center_lon": 106.6895581, "avg_edge_len": 312.059, "cell_area": 97111.876 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.784368722767567 ], [ 106.691032158094345, 10.784206257808368 ], [ 106.691032158094345, 10.786929200057287 ], [ 106.688084055525565, 10.787091704080835 ], [ 106.688084055525565, 10.784368722767567 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f32c", "resolution": 15, "center_lat": 10.788372, "center_lon": 106.6895581, "avg_edge_len": 312.062, "cell_area": 97113.952 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.787091704080835 ], [ 106.691032158094345, 10.786929200057287 ], [ 106.691032158094345, 10.78965222452093 ], [ 106.688084055525565, 10.789814767608535 ], [ 106.688084055525565, 10.787091704080835 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f331", "resolution": 16, "center_lat": 10.7875693, "center_lon": 106.6917692, "avg_edge_len": 156.031, "cell_area": 24278.351 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.786929200057287 ], [ 106.692506224225951, 10.786847936702861 ], [ 106.692506224225951, 10.788209428891561 ], [ 106.691032158094345, 10.7882907020134 ], [ 106.691032158094345, 10.786929200057287 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3324", "resolution": 17, "center_lat": 10.7886108, "center_lon": 106.6914007, "avg_edge_len": 78.016, "cell_area": 6069.637 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.7882907020134 ], [ 106.691769189923207, 10.788250066397811 ], [ 106.691769189923207, 10.788930822641001 ], [ 106.691032158094345, 10.788971460698379 ], [ 106.691032158094345, 10.7882907020134 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3374", "resolution": 17, "center_lat": 10.787168, "center_lon": 106.6928747, "avg_edge_len": 78.015, "cell_area": 6069.571 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.786847936702861 ], [ 106.693243261002294, 10.786807302190011 ], [ 106.693243261002294, 10.78748804327326 ], [ 106.692506224225951, 10.787528680228139 ], [ 106.692506224225951, 10.786847936702861 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f337c", "resolution": 17, "center_lat": 10.7871274, "center_lon": 106.6936118, "avg_edge_len": 78.015, "cell_area": 6069.57 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693243261002294, 10.786807302190011 ], [ 106.693980300251837, 10.786766665786745 ], [ 106.693980300251837, 10.787447404427851 ], [ 106.693243261002294, 10.78748804327326 ], [ 106.693243261002294, 10.786807302190011 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f33c", "resolution": 15, "center_lat": 10.7854865, "center_lon": 106.6925062, "avg_edge_len": 312.059, "cell_area": 97111.838 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.784206257808368 ], [ 106.693980300251837, 10.78404376260953 ], [ 106.693980300251837, 10.786766665786745 ], [ 106.691032158094345, 10.786929200057287 ], [ 106.691032158094345, 10.784206257808368 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f341", "resolution": 16, "center_lat": 10.7846839, "center_lon": 106.6947173, "avg_edge_len": 156.029, "cell_area": 24277.822 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.78404376260953 ], [ 106.695454386168933, 10.783962503670406 ], [ 106.695454386168933, 10.785323935211682 ], [ 106.693980300251837, 10.785405203920131 ], [ 106.693980300251837, 10.78404376260953 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3424", "resolution": 17, "center_lat": 10.7842825, "center_lon": 106.6958229, "avg_edge_len": 78.014, "cell_area": 6069.438 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.783962503670406 ], [ 106.69619143283586, 10.783921871365944 ], [ 106.69619143283586, 10.784602582124428 ], [ 106.695454386168933, 10.784643216871395 ], [ 106.695454386168933, 10.783962503670406 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f342c", "resolution": 17, "center_lat": 10.7842419, "center_lon": 106.69656, "avg_edge_len": 78.014, "cell_area": 6069.438 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.69619143283586, 10.783921871365944 ], [ 106.696928481974453, 10.783881237171563 ], [ 106.696928481974453, 10.784561945487425 ], [ 106.69619143283586, 10.784602582124428 ], [ 106.69619143283586, 10.783921871365944 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f343c", "resolution": 17, "center_lat": 10.7849633, "center_lon": 106.6958229, "avg_edge_len": 78.015, "cell_area": 6069.471 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.784643216871395 ], [ 106.69619143283586, 10.784602582124428 ], [ 106.69619143283586, 10.785283298022215 ], [ 106.695454386168933, 10.785323935211682 ], [ 106.695454386168933, 10.784643216871395 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f346c", "resolution": 17, "center_lat": 10.7856846, "center_lon": 106.6950859, "avg_edge_len": 78.015, "cell_area": 6069.504 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.785364570510989 ], [ 106.695454386168933, 10.785323935211682 ], [ 106.695454386168933, 10.786004658690983 ], [ 106.694717341974183, 10.78604529643267 ], [ 106.694717341974183, 10.785364570510989 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3474", "resolution": 17, "center_lat": 10.7857253, "center_lon": 106.6943488, "avg_edge_len": 78.015, "cell_area": 6069.505 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.785405203920131 ], [ 106.694717341974183, 10.785364570510989 ], [ 106.694717341974183, 10.78604529643267 ], [ 106.693980300251837, 10.786085932284077 ], [ 106.693980300251837, 10.785405203920131 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f347c", "resolution": 17, "center_lat": 10.786406, "center_lon": 106.6943488, "avg_edge_len": 78.015, "cell_area": 6069.537 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.786085932284077 ], [ 106.694717341974183, 10.78604529643267 ], [ 106.694717341974183, 10.786726027493076 ], [ 106.693980300251837, 10.786766665786745 ], [ 106.693980300251837, 10.786085932284077 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f35d4", "resolution": 17, "center_lat": 10.7842013, "center_lon": 106.697297, "avg_edge_len": 78.014, "cell_area": 6069.437 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.696928481974453, 10.783881237171563 ], [ 106.697665533584427, 10.783840601087265 ], [ 106.697665533584427, 10.784521306960389 ], [ 106.696928481974453, 10.784561945487425 ], [ 106.696928481974453, 10.783881237171563 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3644", "resolution": 17, "center_lat": 10.7821185, "center_lon": 106.6980341, "avg_edge_len": 78.014, "cell_area": 6069.339 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.697665533584427, 10.781798514306562 ], [ 106.698402587665328, 10.781757883660939 ], [ 106.698402587665328, 10.782438571671577 ], [ 106.697665533584427, 10.782479204760067 ], [ 106.697665533584427, 10.781798514306562 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3654", "resolution": 17, "center_lat": 10.7814785, "center_lon": 106.697297, "avg_edge_len": 78.014, "cell_area": 6069.307 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.696928481974453, 10.781158455306807 ], [ 106.697665533584427, 10.7811178289935 ], [ 106.697665533584427, 10.781798514306562 ], [ 106.696928481974453, 10.781839143062625 ], [ 106.696928481974453, 10.781158455306807 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f365c", "resolution": 17, "center_lat": 10.7821592, "center_lon": 106.697297, "avg_edge_len": 78.014, "cell_area": 6069.34 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.696928481974453, 10.781839143062625 ], [ 106.697665533584427, 10.781798514306562 ], [ 106.697665533584427, 10.782479204760067 ], [ 106.696928481974453, 10.782519835958878 ], [ 106.696928481974453, 10.781839143062625 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f367", "resolution": 16, "center_lat": 10.7831599, "center_lon": 106.6976655, "avg_edge_len": 156.028, "cell_area": 24277.553 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.696928481974453, 10.782519835958878 ], [ 106.698402587665328, 10.782438571671577 ], [ 106.698402587665385, 10.78379996311306 ], [ 106.696928481974453, 10.783881237171563 ], [ 106.696928481974453, 10.782519835958878 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f36c", "resolution": 15, "center_lat": 10.7826011, "center_lon": 106.6954544, "avg_edge_len": 312.056, "cell_area": 97109.722 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.781320941665426 ], [ 106.696928481974453, 10.781158455306807 ], [ 106.696928481974453, 10.783881237171563 ], [ 106.693980300251837, 10.78404376260953 ], [ 106.693980300251837, 10.781320941665426 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3704", "resolution": 17, "center_lat": 10.7808384, "center_lon": 106.69656, "avg_edge_len": 78.013, "cell_area": 6069.276 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.69619143283586, 10.780518394672923 ], [ 106.696928481974453, 10.780477772691709 ], [ 106.696928481974453, 10.781158455306807 ], [ 106.69619143283586, 10.781199079730662 ], [ 106.69619143283586, 10.780518394672923 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3714", "resolution": 17, "center_lat": 10.7801984, "center_lon": 106.6958229, "avg_edge_len": 78.013, "cell_area": 6069.244 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.779878332405509 ], [ 106.69619143283586, 10.779837714756178 ], [ 106.69619143283586, 10.780518394672923 ], [ 106.695454386168933, 10.780559014764787 ], [ 106.695454386168933, 10.779878332405509 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f371c", "resolution": 17, "center_lat": 10.780879, "center_lon": 106.6958229, "avg_edge_len": 78.013, "cell_area": 6069.276 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.695454386168933, 10.780559014764787 ], [ 106.69619143283586, 10.780518394672923 ], [ 106.69619143283586, 10.781199079730662 ], [ 106.695454386168933, 10.781239702265056 ], [ 106.695454386168933, 10.780559014764787 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f373", "resolution": 16, "center_lat": 10.7805996, "center_lon": 106.6947173, "avg_edge_len": 156.027, "cell_area": 24277.044 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.779959562036447 ], [ 106.695454386168933, 10.779878332405509 ], [ 106.695454386168933, 10.781239702265056 ], [ 106.693980300251837, 10.781320941665426 ], [ 106.693980300251837, 10.779959562036447 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3744", "resolution": 17, "center_lat": 10.7795583, "center_lon": 106.6950859, "avg_edge_len": 78.013, "cell_area": 6069.212 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.694717341974183, 10.77923826850518 ], [ 106.695454386168933, 10.779197655187509 ], [ 106.695454386168933, 10.779878332405509 ], [ 106.694717341974183, 10.779918948165603 ], [ 106.694717341974183, 10.77923826850518 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f374c", "resolution": 17, "center_lat": 10.7795989, "center_lon": 106.6943488, "avg_edge_len": 78.013, "cell_area": 6069.212 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.779278879933718 ], [ 106.694717341974183, 10.77923826850518 ], [ 106.694717341974183, 10.779918948165603 ], [ 106.693980300251837, 10.779959562036447 ], [ 106.693980300251837, 10.779278879933718 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3754", "resolution": 17, "center_lat": 10.7789182, "center_lon": 106.6943488, "avg_edge_len": 78.013, "cell_area": 6069.18 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.778598202972537 ], [ 106.694717341974183, 10.77855759398631 ], [ 106.694717341974183, 10.77923826850518 ], [ 106.693980300251837, 10.779278879933718 ], [ 106.693980300251837, 10.778598202972537 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f39fc", "resolution": 17, "center_lat": 10.7782376, "center_lon": 106.6943488, "avg_edge_len": 78.013, "cell_area": 6069.148 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.693980300251837, 10.777917531153186 ], [ 106.694717341974183, 10.777876924609277 ], [ 106.694717341974183, 10.77855759398631 ], [ 106.693980300251837, 10.778598202972537 ], [ 106.693980300251837, 10.777917531153186 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a1", "resolution": 16, "center_lat": 10.7779581, "center_lon": 106.6932433, "avg_edge_len": 156.025, "cell_area": 24276.529 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.777318067012734 ], [ 106.693980300251837, 10.777236864475952 ], [ 106.693980300251837, 10.778598202972537 ], [ 106.692506224225951, 10.778679415277901 ], [ 106.692506224225951, 10.777318067012734 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a3c", "resolution": 17, "center_lat": 10.7769574, "center_lon": 106.6928747, "avg_edge_len": 78.012, "cell_area": 6069.084 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.692506224225951, 10.776637400593593 ], [ 106.693243261002294, 10.776596802711698 ], [ 106.693243261002294, 10.777277466688746 ], [ 106.692506224225951, 10.777318067012734 ], [ 106.692506224225951, 10.776637400593593 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a5", "resolution": 16, "center_lat": 10.776678, "center_lon": 106.6917692, "avg_edge_len": 156.024, "cell_area": 24276.274 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.776037924530955 ], [ 106.692506224225951, 10.775956739317124 ], [ 106.692506224225951, 10.777318067012734 ], [ 106.691032158094345, 10.777399261994269 ], [ 106.691032158094345, 10.776037924530955 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3a7", "resolution": 16, "center_lat": 10.7780393, "center_lon": 106.6917692, "avg_edge_len": 156.025, "cell_area": 24276.534 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.777399261994269 ], [ 106.692506224225951, 10.777318067012734 ], [ 106.692506224225951, 10.778679415277901 ], [ 106.691032158094345, 10.778760620027098 ], [ 106.691032158094345, 10.777399261994269 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3ac", "resolution": 15, "center_lat": 10.7774805, "center_lon": 106.6895581, "avg_edge_len": 312.049, "cell_area": 97105.644 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.776200272295393 ], [ 106.691032158094345, 10.776037924530955 ], [ 106.691032158094345, 10.778760620027098 ], [ 106.688084055525565, 10.778923006856742 ], [ 106.688084055525565, 10.776200272295393 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b1", "resolution": 16, "center_lat": 10.7753979, "center_lon": 106.6902951, "avg_edge_len": 156.023, "cell_area": 24276.019 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.774757775532056 ], [ 106.691032158094345, 10.774676607639426 ], [ 106.691032158094345, 10.776037924530955 ], [ 106.689558101859916, 10.776119102190403 ], [ 106.689558101859916, 10.774757775532056 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b3", "resolution": 16, "center_lat": 10.775479, "center_lon": 106.6888211, "avg_edge_len": 156.023, "cell_area": 24276.024 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.774838935871152 ], [ 106.689558101859916, 10.774757775532056 ], [ 106.689558101859916, 10.776119102190403 ], [ 106.688084055525565, 10.776200272295393 ], [ 106.688084055525565, 10.774838935871152 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b5", "resolution": 16, "center_lat": 10.7741177, "center_lon": 106.6888211, "avg_edge_len": 156.022, "cell_area": 24275.764 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.773477620020882 ], [ 106.689558101859916, 10.773396469447718 ], [ 106.689558101859916, 10.774757775532056 ], [ 106.688084055525565, 10.774838935871152 ], [ 106.688084055525565, 10.773477620020882 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b64", "resolution": 17, "center_lat": 10.7737165, "center_lon": 106.6899266, "avg_edge_len": 78.011, "cell_area": 6068.924 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.773396469447718 ], [ 106.690295128739763, 10.773355891328903 ], [ 106.690295128739763, 10.774036539357521 ], [ 106.689558101859916, 10.774077119917996 ], [ 106.689558101859916, 10.773396469447718 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b6c", "resolution": 17, "center_lat": 10.7743972, "center_lon": 106.6899266, "avg_edge_len": 78.011, "cell_area": 6068.956 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.689558101859916, 10.774077119917996 ], [ 106.690295128739763, 10.774036539357521 ], [ 106.690295128739763, 10.774717192529929 ], [ 106.689558101859916, 10.774757775532056 ], [ 106.689558101859916, 10.774077119917996 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3b74", "resolution": 17, "center_lat": 10.7743566, "center_lon": 106.6906636, "avg_edge_len": 78.011, "cell_area": 6068.956 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.690295128739763, 10.774036539357521 ], [ 106.691032158094345, 10.773995956908788 ], [ 106.691032158094345, 10.774676607639426 ], [ 106.690295128739763, 10.774717192529929 ], [ 106.690295128739763, 10.774036539357521 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3ba4", "resolution": 17, "center_lat": 10.7749966, "center_lon": 106.6914007, "avg_edge_len": 78.012, "cell_area": 6068.988 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.774676607639426 ], [ 106.691769189923207, 10.774636020860557 ], [ 106.691769189923207, 10.77531667429283 ], [ 106.691032158094345, 10.775357263513579 ], [ 106.691032158094345, 10.774676607639426 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3bac", "resolution": 17, "center_lat": 10.7756773, "center_lon": 106.6914007, "avg_edge_len": 78.012, "cell_area": 6069.02 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.691032158094345, 10.775357263513579 ], [ 106.691769189923207, 10.77531667429283 ], [ 106.691769189923207, 10.775997332868334 ], [ 106.691032158094345, 10.776037924530955 ], [ 106.691032158094345, 10.775357263513579 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3ca4", "resolution": 17, "center_lat": 10.7730764, "center_lon": 106.6891896, "avg_edge_len": 78.011, "cell_area": 6068.892 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688821077455032, 10.772756397910626 ], [ 106.689558101859916, 10.772715824121512 ], [ 106.689558101859916, 10.773396469447718 ], [ 106.688821077455032, 10.773437045678383 ], [ 106.688821077455032, 10.772756397910626 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3cac", "resolution": 17, "center_lat": 10.773117, "center_lon": 106.6884526, "avg_edge_len": 78.011, "cell_area": 6068.892 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.772796969811695 ], [ 106.688821077455032, 10.772756397910626 ], [ 106.688821077455032, 10.773437045678383 ], [ 106.688084055525565, 10.773477620020882 ], [ 106.688084055525565, 10.772796969811695 ] ] ] } },
{ "type": "Feature", "properties": { "s2": "31752f3cb4", "resolution": 17, "center_lat": 10.7724364, "center_lon": 106.6884526, "avg_edge_len": 78.011, "cell_area": 6068.86 }, "geometry": { "type": "Polygon", "coordinates": [ [ [ 106.688084055525565, 10.772116324746849 ], [ 106.688821077455032, 10.772075755287215 ], [ 106.688821077455032, 10.772756397910626 ], [ 106.688084055525565, 10.772796969811695 ], [ 106.688084055525565, 10.772116324746849 ] ] ] } }
]
}
```

``` bash
> geojson2h3 -r 11 -geojson polygon.geojson # geojson2h3 -r <resolution>[0..15] -geojson <GeoJSON file> -compact [optional]
> geojson2s2 -r 18 -geojson polygon.geojson -compact # geojson2s2 -r <resolution>[0..30] -geojson <GeoJSON file> -compact [optional]
> geojson2rhealpix -r 11 -geojson polygon.geojson # geojson2rhealpix -r <resolution>[1..15] -geojson <GeoJSON file> -compact [optional]
> geojson2isea4t -r 17 -geojson polygon.geojson # geojson2isea4t -r <resolution>[0..25] -geojson <GeoJSON file> -compact [optional]
> geojson2isea3h -r 18 -geojson polygon.geojson  # geojson2isea3h -r <resolution>[1..32] -geojson <GeoJSON file> -compact [optional]
> geojson2ease -r 4 -geojson polygon.geojson  # geojson2ease -r <resolution>[0..6] -geojson <GeoJSON file> -compact [optional]
> geojson2dggrid -t ISEA4H -r 17 -geojson polyline.geojson # Linux only: geojson2dggrid -t <DGGS Type> -r <resolution> -a <address_type, default is SEQNUM> -geojson <GeoJSON path>
# <Address Type> chosen from [Q2DI,SEQNUM,INTERLEAVE,PLANE,Q2DD,PROJTRI,VERTEX2DD,AIGEN,Z3,Z3_STRING,Z7,Z7_STRING,ZORDER,ZORDER_STRING]
> geojson2qtm -r 18 -geojson polygon.geojson  # geojson2qtm -r <resolution>[1..24] -geojson <GeoJSON file> -compact [optional]
> geojson2olc -r 10 -geojson polygon.geojson # geojson2olc -r <resolution>[2,4,6,8,10,11..15] -geojson <GeoJSON file> -compact [optional]
> geojson2geohash -r 7 -geojson polygon.geojson # geojson2geohash -r <resolution>[1..10] -geojson <GeoJSON file> -compact [optional]
> geojson2mgrs -r 3 -geojson polygon.geojson # geojson2mgrs -r <resolution>[0..5] -geojson <GeoJSON file>
> geojson2tilecode -r 18 -geojson polygon.geojson # geojson2tilecode -r <resolution>[0..29] -geojson <GeoJSON file> -compact [optional]
> geojson2quadkey -r 18 -geojson polygon.geojson # geojson2quadkey -r <resolution>[0..29] -geojson <GeoJSON file> -compact [optional]
```
### Raster to DGGS
Convert raster layers in geographic CRS to output DGGS with closest matching resolution.

<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/raster2dggs_h3.png">
</div>

``` bash
> raster2h3 -raster raster.tif # raster2h3 -raster <raster in geographic CRS> -r <resolution>[0..15] [optional, defaults to the H3 resolution nearest to the raster's cell size]
> raster2s2 -raster raster.tif # raster2s2 -raster <raster in geographic CRS> -r <resolution>[0..24] [optional, defaults to the S2 resolution nearest to the raster's cell size]
> raster2rhealpix -raster raster.tif # raster2rhealpix -raster <raster in geographic CRS> -r <resolution>[0..15] [optional, defaults to the rHEALPix resolution nearest to the raster's cell size]
> raster2isea4t -raster raster.tif # Windows only: raster2isea4t -raster <raster in geographic CRS> -r <resolution>[0..23] [optional, defaults to the ISEA4T resolution nearest to the raster's cell size]
> raster2qtm -raster raster.tif # raster2qtm -raster <raster in geographic CRS> -r <resolution>[1..24] [optional, defaults to the QTM resolution nearest to the raster's cell size]
> raster2olc -raster raster.tif # raster2olc -raster <raster in geographic CRS> -r <resolution>[10..12] [optional, defaults to the OLC resolution nearest to the raster's cell size]
> raster2geohash -raster raster.tif # raster2geohash -raster <raster in geographic CRS> -r <resolution>[1..10] [optional, defaults to the Geohash resolution nearest to the raster's cell size]
> raster2tilecode -raster raster.tif # raster2tilecode -raster <raster in geographic CRS> -r <resolution>[0..26] [optional, defaults to the Tilecode resolution nearest to the raster's cell size]
> raster2quadkey -raster raster.tif # raster2quadkey -raster <raster in geographic CRS> -r <resolution>[0..26] [optional, defaults to the Quadkey resolution nearest to the raster's cell size]
```

### DGGS Compact
<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/dggscompact_isea4t.png">
</div>

``` bash
> h3compact -geojson h3.geojson -cellid h3  # h3compact -geojson <H3 in GeoJSON> -cellid [optional, 'h3' by default]
> s2compact -geojson s2.geojson -cellid s2  # s2compact -geojson <S2 in GeoJSON> -cellid [optional, 's2' by default]
> rhealpixcompact -geojson rhealpix.geojson -cellid rhealpix  # rhealpixcompact -geojson <rHEALPix in GeoJSON> -cellid [optional, 'rhealpix' by default]
> isea4tcompact -geojson isea4t.geojson -cellid isea4t  # Windows only: isea4tcompact -geojson <ISEA4T in GeoJSON> -cellid [optional, 'isea4t' by default]
> isea3hcompact -geojson isea3h.geojson -cellid isea3h  # Windows only: isea3hcompact -geojson <ISEA3H in GeoJSON> -cellid [optional, 'isea3h' by default]
> easecompact -geojson ease.geojson -cellid ease  # easecompact -geojson <EASE in GeoJSON> -cellid [optional, 'ease' by default]
> qtmcompact -geojson qtm.geojson -cellid qtm  # qtmcompact -geojson <QTM in GeoJSON> -cellid [optional, 'qtm' by default]
> olccompact -geojson olc.geojson -cellid olc  # olccompact -geojson <OLC in GeoJSON> -cellid [optional, 'olc' by default]
> geohashcompact -geojson geohash.geojson -cellid geohash  # geohashcompact -geojson <Geohash in GeoJSON> -cellid [optional, 'geohash' by default]
> tilecodecompact -geojson tilecode.geojson -cellid tilecode  # tilecodecompact -geojson <Tilecode in GeoJSON> -cellid [optional, 'tilecode' by default]
> quadkeycompact -geojson quadkey.geojson -cellid quadkey  # quadkeycompact -geojson <quadkey in GeoJSON> -cellid [optional, 'quadkey' by default]
```

### DGGS Expand
<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/dggsexpand_isea4t.png">
</div>

``` bash
> h3expand -geojson h3_11.geojson -r 12 -cellid h3 # h3expand -geojson <H3 in GeoJSON> -r <higher resolution>[0..15] -cellid [optional, 'h3' by default]
> s2expand -geojson s2_18.geojson -r 19 -cellid s2 # s2expand -geojson <S2 in GeoJSON> -r <higher resolution>[0..30] -cellid [optional, 's2' by default]
> rhealpixexpand -geojson rhealpix_10.geojson -r 11 -cellid rhealpix # rhealpixexpand -geojson <rHEALPix in GeoJSON> -r <higher resolution>[0..15] -cellid [optional, 'rhealpix' by default]
> isea4texpand -geojson isea4t_18.geojson -r 19 -cellid isea4t # Windows only: isea4texpand -geojson <ISEA4T in GeoJSON> -r <higher resolution>[0..25] -cellid [optional, 'isea4t' by default]
> isea3hexpand -geojson isea3h_18.geojson -r 19 -cellid isea3h # Windows only: isea3hexpand -geojson <ISEA3H in GeoJSON> -r <higher resolution>[0..25] -cellid [optional, 'isea3h' by default]
> easeexpand -geojson ease_4.geojson -r 5 -cellid ease # easeexpand -geojson <EASE in GeoJSON> -r <higher resolution>[0..6] -cellid [optional, 'ease' by default]
> qtmexpand -geojson qtm_18.geojson -r 19 -cellid qtm # qtmexpand -geojson <QTM in GeoJSON> -r <higher resolution>[1..24] -cellid [optional, 'qtm' by default]
> olcexpand -geojson olc_8.geojson -r 10 -cellid olc # olcexpand -geojson <OLC in GeoJSON> -r <higher resolution>[2,4,6,8,10..15] -cellid [optional, 'olc' by default]
> geohashexpand -geojson geohash_7.geojson -r 8 -cellid geohash # geohashexpand -geojson <Geohash in GeoJSON> -r <higher resolution>[1..10] -cellid [optional, 'geohash' by default]
> tilecodeexpand -geojson tilecode_18.geojson -r 19 -cellid tilecode # tilecodeexpand -geojson <Tilecode in GeoJSON> -r <higher resolution>[0..29] -cellid [optional, 'tilecode' by default]
> quadkeyexpand -geojson quadkey_18.geojson -r 19 -cellid quadkey # quadkeyexpand -geojson <Quadkey in GeoJSON> -r <higher resolution>[0..29] -cellid [optional, 'quadkey' by default]
```

### CSV to DGGS
``` bash
> csv2h3 h3.csv  # Convert CSV with 'h3' column to H3 cells.
> csv2s2 s2.csv  # Convert CSV with 's2' column to S2 cells.
> csv2rhealpix rhealpix.csv  # Convert CSV with 'rhealpix' column to rHEALPix cells.
> csv2isea4t isea4t.csv  # Windows only: Convert CSV with 'rhealpisea4t' column to ISEA4T cells.
> csv2isea3h isea3h.csv  # Windows only: Convert CSV with 'isea3h' column to ISEA3H cells.
> csv2sease ease.csv  # Convert CSV with 'ease' column to EASE cells.
> csv2qtm qtm.csv  # Convert CSV with 'qtm' column to QTM cells.
> csv2olc olc.csv  # Convert CSV with 'olc' column to OLC cells.
> csv2geohash geohash.csv  # Convert CSV with 'geohash' column to Geohash cells.
> csv2georef georef.csv  # Convert CSV with 'georef' column to GEOREF cells.
> csv2mgrs mgrs.csv  # Convert CSV with 'mgrs' column to MGRS cells.
> csv2tilecode tilecode.csv  # Convert CSV with 'tilecode' column to Tilecode cells.
> csv2quadkey quadkey.csv  # Convert CSV with 'quadkey' column to Tilecode cells.
> csv2maidenhead maidenhead.csv  # Convert CSV with 'maidenhead' column to Maidenhead cells.
> csv2gars gars.csv  # Convert CSV with 'gars' column to GARS cells.
``` 

## DGGS Binning
Binning point layer to DGGS

<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/dggsbinning_h3.png">
</div>

``` bash
> h3bin -point point.geojson -r 8 -stats count -field numeric_field -category group # h3bin -point <point GeoJSON file> -r <resolution[0..15]> -stats [count, min, max, sum, mean, median, std, var, range, minority, majority, variety] -field [Optional, numeric field to compute statistics] -category [optional, category field for grouping] 
> s2bin -point point.geojson -r 13 -stats count -field numeric_field -category group # s2bin -point <point GeoJSON file> -r <resolution[0..30]> -stats [count, min, max, sum, mean, median, std, var, range, minority, majority, variety] -field [Optional, numeric field to compute statistics] -category [optional, category field for grouping] 
> rhealpixbin -point point.geojson -r 8 -stats count -field numeric_field -category group # rhealpixbin -point <point GeoJSON file> -r <resolution[0..15]> -stats [count, min, max, sum, mean, median, std, var, range, minority, majority, variety] -field [Optional, numeric field to compute statistics] -category [optional, category field for grouping] 
> isea4tbin -point point.geojson -r 13 -stats count -field numeric_field -category group # Windows only: isea4tbin -point <point GeoJSON file> -r <resolution[0..25]> -stats [count, min, max, sum, mean, median, std, var, range, minority, majority, variety] -field [Optional, numeric field to compute statistics] -category [optional, category field for grouping] 
> qtmbin -point point.geojson -r 13 -stats count -field numeric_field -category group # qtmbin -point <point GeoJSON file> -r <resolution[1..24]> -stats [count, min, max, sum, mean, median, std, var, range, minority, majority, variety] -field [Optional, numeric field to compute statistics] -category [optional, category field for grouping] 
> olcbin -point point.geojson -r 9 -stats count -field numeric_field -category group # olcbin -point <point GeoJSON file> -r <resolution[2,4,6,8,10..15]> -stats [count, min, max, sum, mean, median, std, var, range, minority, majority, variety] -field [Optional, numeric field to compute statistics] -category [optional, category field for grouping] 
> geohashbin -point point.geojson -r 6 -stats count -field numeric_field -category group # geohashbin -point <point GeoJSON file> -r <resolution[1..10]> -stats [count, min, max, sum, mean, median, std, var, range, minority, majority, variety] -field [Optional, numeric field to compute statistics] -category [optional, category field for grouping] 
> tilecodebin -point point.geojson -r 15 -stats count -field numeric_field -category group # tilecodebin -point <point GeoJSON file> -r <resolutin[0..25]> -stats [count, min, max, sum, mean, median, std, var, range, minority, majority, variety] -field [Optional, numeric field to compute statistics] -category [optional, category field for grouping] 
> quadkeybin -point point.geojson -r 13 -stats count -field numeric_field -category group # Windows only: quadkeybin -point <point GeoJSON file> -r <resolutin[0..25]> -stats [count, min, max, sum, mean, median, std, var, range, minority, majority, variety] -field [Optional, numeric field to compute statistics] -category [optional, category field for grouping] 
``` 

## DGGS Resampling
Resample input DGGS to output DGGS with closest matching resolution.

<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/dggsresampling_h32s2.png">
</div>

``` bash
> resample -geojson vn_pop_h3_6.geojson -fromdggs h3 -todggs s2 -resamplefield population # resample -geojson <Input DGGS> -fromdggs <Input DGGS type> -todggs <Output DGGS Type> -resamplefield [Optional, Numeric field for resampling]
```

## DGGS Generator
Generate DGGS within a bounding box

<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/dggsgenerator_h3.png">
</div>

``` bash
> h3grid -r 11 -b 106.699007 10.762811 106.717674 10.778649 # h3grid -r <resolution> [0..15] -b <min_lon> <min_lat> <max_lon> <max_lat>
> s2grid -r 18 -b 106.699007 10.762811 106.717674 10.778649 # s2grid -r <resolution> [0..30] -b <min_lon> <min_lat> <max_lon> <max_lat>
> rhealpixgrid -r 11 -b 106.699007 10.762811 106.717674 10.778649 # rhealpix2grid -r <resolution> [0..30] -b <min_lon> <min_lat> <max_lon> <max_lat>
> isea4tgrid -r 17 -b 106.699007 10.762811 106.717674 10.778649 # Windows only: isea4tgrid -r <resolution> [0..25] -b <min_lon> <min_lat> <max_lon> <max_lat>
> isea3hgrid -r 20 -b 106.699007 10.762811 106.717674 10.778649 # isea3hgrid -r <resolution> [0..32] -b <min_lon> <min_lat> <max_lon> <max_lat>> isea3hstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> easegrid -r 4 -b 106.6990073571 10.7628112647 106.71767427 10.778649620 # easegrid -r <resolution> [0..6] -b <min_lon> <min_lat> <max_lon> <max_lat>
> dggridgen -t ISEA3H -r 2 -a ZORDER # Linux only: dggrid -t <DGGS Type> -r <resolution> -a<address_type>. 
> qtmgrid -r 8 -b 106.6990073571 10.7628112647 106.71767427 10.778649620 # qtmgrid -r <resolution> [1..24] -b <min_lon> <min_lat> <max_lon> <max_lat>
> olcgrid -r 8 -b 106.6990073571 10.7628112647 106.71767427 10.778649620 # olcgrid -r <resolution> [2,4,6,8,10..15] -b <min_lon> <min_lat> <max_lon> <max_lat>
> geohashgrid -r 6 -b 106.699007 10.762811 106.717674 10.778649 # geohashgrid -r <resolution> [1..10] -b <min_lon> <min_lat> <max_lon> <max_lat> 1
> geohashgrid -r 2 -b 106.699007 10.762811 106.717674 10.778649 # geohashgrid -r <resolution> [0..5] -b <min_lon> <min_lat> <max_lon> <max_lat> 
> georefgrid -r 2 -b 106.699007 10.762811 106.717674 10.778649 # georefgrid -r <resolution> [-1..5] -b <min_lon> <min_lat> <max_lon> <max_lat> 
> mgrsgrid -r 1 -gzd 48P # geohashgrid -r <resolution> [0..5] -gzd <Grid Zone Designator, e.g. 48P>
> tilecodegrid -r 20 -b 106.699007 10.762811 106.717674 10.778649 # tilecodegrid -r <resolution> [0..26] 
> quadkeygrid -r 20 -b 106.699007 10.762811 106.717674 10.778649 # quadkeygrid -r <resolution> [0..26] 
> maidenheadgrid -r 4 -b 106.699007 10.762811 106.717674 10.778649 # maidenheadgrid -r <resolution> [1..4] -b <min_lon> <min_lat> <max_lon> <max_lat>
> garsgrid -r 1 -b 106.699007 10.762811 106.717674 10.778649 # garsgrid -r <resolution> = [30,15,5,1] minutes -b <min_lon> <min_lat> <max_lon> <max_lat>
```

## DGGS Stats
<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/dggsstats.png">
</div>

``` bash
> h3stats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> s2stats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> rhealpixstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> isea4tstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> isea3hstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> dggridstats -t FULLER3H -r 8 # Linux only: dggrid -t <DGGS Type> -r <resolution>
> easestats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> qtmstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> olcstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> geohashstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> georeftats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> mgrstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> tilecodestats # Number of cells, Cell Width, Cell Height, Cell Area at each resolution
> quadkeystats # Number of cells, Cell Width, Cell Height, Cell Area at each resolution
> maidenheadstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
> garsstats # Number of cells, Avg Edge Length, Avg Cell Area at each resolution
```

### POLYHEDRA GENERATOR
<div align="center">
  <img src="https://raw.githubusercontent.com/thangqd/vgridtools/main/images/readme/polyhedra.png">
</div>

``` bash
> tetrahedron  # Generate Global Tetrahedron
> cube         # Generate Global Cube
> octahedron   # Generate Global Octahedron  
> fuller_icosahedron   # Generate Global Fuller Icosahedron  
> rhombic_icosahedron   # Generate Global rhombic Icosahedron 
``` 
